@@ -16,9 +16,6 @@
  */
 package org.apache.kafka.common.errors;
 
-import org.apache.kafka.common.annotation.InterfaceStability;
-
-@InterfaceStability.Evolving
 public class UnreleasedInstanceIdException extends ApiException {
     public UnreleasedInstanceIdException(String message) {
         super(message);
