@@ -136,7 +136,7 @@ public abstract class AbstractRequest implements AbstractRequestResponse {
     }
 
     @Override
-    public String toString() {
+    public final String toString() {
         return toString(true);
     }
 
@@ -196,14 +196,6 @@ public abstract class AbstractRequest implements AbstractRequestResponse {
                 return LeaveGroupRequest.parse(buffer, apiVersion);
             case SYNC_GROUP:
                 return SyncGroupRequest.parse(buffer, apiVersion);
-            case STOP_REPLICA:
-                return StopReplicaRequest.parse(buffer, apiVersion);
-            case CONTROLLED_SHUTDOWN:
-                return ControlledShutdownRequest.parse(buffer, apiVersion);
-            case UPDATE_METADATA:
-                return UpdateMetadataRequest.parse(buffer, apiVersion);
-            case LEADER_AND_ISR:
-                return LeaderAndIsrRequest.parse(buffer, apiVersion);
             case DESCRIBE_GROUPS:
                 return DescribeGroupsRequest.parse(buffer, apiVersion);
             case LIST_GROUPS:
