@@ -41,6 +41,8 @@ import org.apache.kafka.common.utils.Time;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.time.Duration;
 import java.util.Collections;
 import java.util.List;
@@ -323,8 +325,6 @@ public class ClientTelemetryReporter implements MetricsReporter {
 
             final long timeMs;
             final String apiName;
-            final String msg;
-            final boolean isTraceEnabled = log.isTraceEnabled();
 
             switch (localState) {
                 case SUBSCRIPTION_IN_PROGRESS:
@@ -336,15 +336,19 @@ public class ClientTelemetryReporter implements MetricsReporter {
                     */
                     apiName = (localState == ClientTelemetryState.SUBSCRIPTION_IN_PROGRESS) ? ApiKeys.GET_TELEMETRY_SUBSCRIPTIONS.name : ApiKeys.PUSH_TELEMETRY.name;
                     timeMs = requestTimeoutMs;
-                    msg = isTraceEnabled ? "" : String.format("the remaining wait time for the %s network API request, as specified by %s", apiName, CommonClientConfigs.REQUEST_TIMEOUT_MS_CONFIG);
+                    log.trace("For telemetry state {}, returning the value {} ms; the remaining wait time for the {} network API request, as specified by {}", localState, timeMs, apiName, CommonClientConfigs.REQUEST_TIMEOUT_MS_CONFIG);
                     break;
                 case TERMINATING_PUSH_IN_PROGRESS:
                     timeMs = Long.MAX_VALUE;
-                    msg = isTraceEnabled ? "" : "the terminating push is in progress, disabling telemetry for further requests";
+                    log.trace("For telemetry state {}, returning the value {} ms; the terminating push is in progress, disabling telemetry for further requests", localState, timeMs);
+                    break;
+                case TERMINATED:
+                    timeMs = Long.MAX_VALUE;
+                    log.trace("For telemetry state {}, returning the value {} ms; telemetry is terminated, no further requests will be made", localState, timeMs);
                     break;
                 case TERMINATING_PUSH_NEEDED:
                     timeMs = 0;
-                    msg = isTraceEnabled ? "" : String.format("the client should try to submit the final %s network API request ASAP before closing", ApiKeys.PUSH_TELEMETRY.name);
+                    log.trace("For telemetry state {}, returning the value {} ms; the client should try to submit the final {} network API request ASAP before closing", localState, timeMs, ApiKeys.PUSH_TELEMETRY.name);
                     break;
                 case SUBSCRIPTION_NEEDED:
                 case PUSH_NEEDED:
@@ -352,19 +356,15 @@ public class ClientTelemetryReporter implements MetricsReporter {
                     long timeRemainingBeforeRequest = localLastRequestMs + localIntervalMs - nowMs;
                     if (timeRemainingBeforeRequest <= 0) {
                         timeMs = 0;
-                        msg = isTraceEnabled ? "" : String.format("the wait time before submitting the next %s network API request has elapsed", apiName);
+                        log.trace("For telemetry state {}, returning the value {} ms; the wait time before submitting the next {} network API request has elapsed", localState, timeMs, apiName);
                     } else {
                         timeMs = timeRemainingBeforeRequest;
-                        msg = isTraceEnabled ? "" : String.format("the client will wait before submitting the next %s network API request", apiName);
+                        log.trace("For telemetry state {}, returning the value {} ms; the client will wait before submitting the next {} network API request", localState, timeMs, apiName);
                     }
                     break;
                 default:
                     throw new IllegalStateException("Unknown telemetry state: " + localState);
             }
-
-            if (isTraceEnabled) {
-                log.trace("For telemetry state {}, returning the value {} ms; {}", localState, timeMs, msg);
-            }
             return timeMs;
         }
 
@@ -702,7 +702,7 @@ public class ClientTelemetryReporter implements MetricsReporter {
         }
 
         private Optional<Builder<?>> createPushRequest(ClientTelemetrySubscription localSubscription, boolean terminating) {
-            byte[] payload;
+            MetricsData payload;
             try (MetricsEmitter emitter = new ClientTelemetryEmitter(localSubscription.selector(), localSubscription.deltaTemporality())) {
                 emitter.init();
                 kafkaMetricsCollector.collect(emitter);
@@ -715,12 +715,12 @@ public class ClientTelemetryReporter implements MetricsReporter {
             }
 
             CompressionType compressionType = ClientTelemetryUtils.preferredCompressionType(localSubscription.acceptedCompressionTypes());
-            byte[] compressedPayload;
+            ByteBuffer compressedPayload;
             try {
                 compressedPayload = ClientTelemetryUtils.compress(payload, compressionType);
-            } catch (Throwable e) {
-                log.debug("Failed to compress telemetry payload for compression: {}, sending uncompressed data", compressionType);
-                compressedPayload = payload;
+            } catch (IOException e) {
+                log.info("Failed to compress telemetry payload for compression: {}, sending uncompressed data", compressionType);
+                compressedPayload = ByteBuffer.wrap(payload.toByteArray());
                 compressionType = CompressionType.NONE;
             }
 
@@ -861,14 +861,14 @@ public class ClientTelemetryReporter implements MetricsReporter {
             }
         }
 
-        private byte[] createPayload(List<SinglePointMetric> emittedMetrics) {
+        private MetricsData createPayload(List<SinglePointMetric> emittedMetrics) {
             MetricsData.Builder builder = MetricsData.newBuilder();
             emittedMetrics.forEach(metric -> {
                 Metric m = metric.builder().build();
                 ResourceMetrics rm = buildMetric(m);
                 builder.addResourceMetrics(rm);
             });
-            return builder.build().toByteArray();
+            return builder.build();
         }
 
         // Visible for testing
