@@ -29,7 +29,6 @@ import org.apache.kafka.common.requests.MetadataRequest;
 import org.apache.kafka.common.requests.MetadataResponse;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.common.utils.MockTime;
-import org.apache.kafka.common.utils.Utils;
 import org.apache.kafka.test.TestUtils;
 
 import org.junit.jupiter.api.Test;
@@ -112,7 +111,7 @@ public class AllBrokersStrategyIntegrationTest {
         assertEquals(1, lookupSpecs.size());
         AdminApiDriver.RequestSpec<AllBrokersStrategy.BrokerKey> lookupSpec = lookupSpecs.get(0);
 
-        Set<Integer> brokerIds = Utils.mkSet(1, 2);
+        Set<Integer> brokerIds = Set.of(1, 2);
         driver.onResponse(time.milliseconds(), lookupSpec, responseWithBrokers(brokerIds), Node.noNode());
         assertTrue(result.all().isDone());
 
