@@ -35,6 +35,7 @@ import org.apache.kafka.common.requests.DescribeProducersRequest;
 import org.apache.kafka.common.requests.DescribeProducersResponse;
 import org.apache.kafka.common.utils.CollectionUtils;
 import org.apache.kafka.common.utils.LogContext;
+
 import org.junit.jupiter.api.Test;
 
 import java.util.HashSet;
@@ -50,7 +51,6 @@ import static java.util.Collections.emptyList;
 import static java.util.Collections.emptyMap;
 import static java.util.Collections.singletonList;
 import static java.util.Collections.singletonMap;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
@@ -69,7 +69,7 @@ public class DescribeProducersHandlerTest {
     @Test
     public void testBrokerIdSetInOptions() {
         int brokerId = 3;
-        Set<TopicPartition> topicPartitions = mkSet(
+        Set<TopicPartition> topicPartitions = Set.of(
             new TopicPartition("foo", 5),
             new TopicPartition("bar", 3),
             new TopicPartition("foo", 4)
@@ -88,7 +88,7 @@ public class DescribeProducersHandlerTest {
 
     @Test
     public void testBrokerIdNotSetInOptions() {
-        Set<TopicPartition> topicPartitions = mkSet(
+        Set<TopicPartition> topicPartitions = Set.of(
             new TopicPartition("foo", 5),
             new TopicPartition("bar", 3),
             new TopicPartition("foo", 4)
@@ -107,7 +107,7 @@ public class DescribeProducersHandlerTest {
 
     @Test
     public void testBuildRequest() {
-        Set<TopicPartition> topicPartitions = mkSet(
+        Set<TopicPartition> topicPartitions = Set.of(
             new TopicPartition("foo", 5),
             new TopicPartition("bar", 3),
             new TopicPartition("foo", 4)
@@ -122,13 +122,13 @@ public class DescribeProducersHandlerTest {
 
         List<DescribeProducersRequestData.TopicRequest> topics = request.data.topics();
 
-        assertEquals(mkSet("foo", "bar"), topics.stream()
+        assertEquals(Set.of("foo", "bar"), topics.stream()
             .map(DescribeProducersRequestData.TopicRequest::name)
             .collect(Collectors.toSet()));
 
         topics.forEach(topic -> {
             Set<Integer> expectedTopicPartitions = "foo".equals(topic.name()) ?
-                mkSet(4, 5) : mkSet(3);
+                Set.of(4, 5) : Set.of(3);
             assertEquals(expectedTopicPartitions, new HashSet<>(topic.partitionIndexes()));
         });
     }
@@ -139,7 +139,7 @@ public class DescribeProducersHandlerTest {
         Throwable exception = assertFatalError(topicPartition, Errors.TOPIC_AUTHORIZATION_FAILED);
         assertInstanceOf(TopicAuthorizationException.class, exception);
         TopicAuthorizationException authException = (TopicAuthorizationException) exception;
-        assertEquals(mkSet("foo"), authException.unauthorizedTopics());
+        assertEquals(Set.of("foo"), authException.unauthorizedTopics());
     }
 
     @Test
@@ -148,7 +148,7 @@ public class DescribeProducersHandlerTest {
         Throwable exception = assertFatalError(topicPartition, Errors.INVALID_TOPIC_EXCEPTION);
         assertInstanceOf(InvalidTopicException.class, exception);
         InvalidTopicException invalidTopicException = (InvalidTopicException) exception;
-        assertEquals(mkSet("foo"), invalidTopicException.invalidTopics());
+        assertEquals(Set.of("foo"), invalidTopicException.invalidTopics());
     }
 
     @Test
@@ -183,7 +183,7 @@ public class DescribeProducersHandlerTest {
             handleResponseWithError(options, topicPartition, Errors.NOT_LEADER_OR_FOLLOWER);
         assertEquals(emptyMap(), result.completedKeys);
         assertEquals(emptyList(), result.unmappedKeys);
-        assertEquals(mkSet(topicPartition), result.failedKeys.keySet());
+        assertEquals(Set.of(topicPartition), result.failedKeys.keySet());
         Throwable exception = result.failedKeys.get(topicPartition);
         assertInstanceOf(NotLeaderOrFollowerException.class, exception);
     }
@@ -201,9 +201,9 @@ public class DescribeProducersHandlerTest {
         Node node = new Node(3, "host", 1);
 
         ApiResult<TopicPartition, PartitionProducerState> result =
-            handler.handleResponse(node, mkSet(topicPartition), response);
+            handler.handleResponse(node, Set.of(topicPartition), response);
 
-        assertEquals(mkSet(topicPartition), result.completedKeys.keySet());
+        assertEquals(Set.of(topicPartition), result.completedKeys.keySet());
         assertEquals(emptyMap(), result.failedKeys);
         assertEquals(emptyList(), result.unmappedKeys);
 
@@ -230,7 +230,7 @@ public class DescribeProducersHandlerTest {
             new DescribeProducersOptions(), topicPartition, error);
         assertEquals(emptyMap(), result.completedKeys);
         assertEquals(emptyList(), result.unmappedKeys);
-        assertEquals(mkSet(topicPartition), result.failedKeys.keySet());
+        assertEquals(Set.of(topicPartition), result.failedKeys.keySet());
         return result.failedKeys.get(topicPartition);
     }
 
@@ -242,7 +242,7 @@ public class DescribeProducersHandlerTest {
         DescribeProducersHandler handler = newHandler(options);
         DescribeProducersResponse response = buildResponseWithError(topicPartition, error);
         Node node = new Node(options.brokerId().orElse(3), "host", 1);
-        return handler.handleResponse(node, mkSet(topicPartition), response);
+        return handler.handleResponse(node, Set.of(topicPartition), response);
     }
 
     private DescribeProducersResponse buildResponseWithError(
