@@ -19,7 +19,6 @@ package org.apache.kafka.common.requests;
 
 import org.apache.kafka.common.acl.AclOperation;
 import org.apache.kafka.common.acl.AclPermissionType;
-import org.apache.kafka.common.errors.UnsupportedVersionException;
 import org.apache.kafka.common.message.DeleteAclsResponseData;
 import org.apache.kafka.common.message.DeleteAclsResponseData.DeleteAclsFilterResult;
 import org.apache.kafka.common.message.DeleteAclsResponseData.DeleteAclsMatchingAcl;
@@ -36,7 +35,6 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 
 public class DeleteAclsResponseTest {
-    private static final short V0 = 0;
     private static final short V1 = 1;
 
     private static final DeleteAclsMatchingAcl LITERAL_ACL1 = new DeleteAclsMatchingAcl()
@@ -84,15 +82,6 @@ public class DeleteAclsResponseTest {
     private static final DeleteAclsFilterResult UNKNOWN_RESPONSE = new DeleteAclsFilterResult().setMatchingAcls(singletonList(
             UNKNOWN_ACL));
 
-    @Test
-    public void shouldThrowOnV0IfNotLiteral() {
-        assertThrows(UnsupportedVersionException.class, () -> new DeleteAclsResponse(
-            new DeleteAclsResponseData()
-                .setThrottleTimeMs(10)
-                .setFilterResults(singletonList(PREFIXED_RESPONSE)),
-            V0));
-    }
-
     @Test
     public void shouldThrowOnIfUnknown() {
         assertThrows(IllegalArgumentException.class, () -> new DeleteAclsResponse(
@@ -102,19 +91,6 @@ public class DeleteAclsResponseTest {
             V1));
     }
 
-    @Test
-    public void shouldRoundTripV0() {
-        final DeleteAclsResponse original = new DeleteAclsResponse(
-            new DeleteAclsResponseData()
-                .setThrottleTimeMs(10)
-                .setFilterResults(singletonList(LITERAL_RESPONSE)),
-            V0);
-        final ByteBuffer buffer = original.serialize(V0);
-
-        final DeleteAclsResponse result = DeleteAclsResponse.parse(buffer, V0);
-        assertEquals(original.filterResults(), result.filterResults());
-    }
-
     @Test
     public void shouldRoundTripV1() {
         final DeleteAclsResponse original = new DeleteAclsResponse(
