@@ -19,15 +19,12 @@ package org.apache.kafka.common.requests;
 import org.apache.kafka.common.errors.InvalidConfigurationException;
 import org.apache.kafka.common.errors.UnsupportedVersionException;
 import org.apache.kafka.common.message.JoinGroupRequestData;
-import org.apache.kafka.common.protocol.MessageUtil;
 import org.apache.kafka.test.TestUtils;
 
 import org.junit.jupiter.api.Test;
 
-import java.nio.ByteBuffer;
 import java.util.Arrays;
 
-import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.fail;
 
@@ -68,20 +65,4 @@ public class JoinGroupRequestTest {
                 .setProtocolType("consumer")
         ).build((short) 4));
     }
-
-    @Test
-    public void testRebalanceTimeoutDefaultsToSessionTimeoutV0() {
-        int sessionTimeoutMs = 30000;
-        short version = 0;
-
-        ByteBuffer buffer = MessageUtil.toByteBuffer(new JoinGroupRequestData()
-                .setGroupId("groupId")
-                .setMemberId("consumerId")
-                .setProtocolType("consumer")
-                .setSessionTimeoutMs(sessionTimeoutMs), version);
-
-        JoinGroupRequest request = JoinGroupRequest.parse(buffer, version);
-        assertEquals(sessionTimeoutMs, request.data().sessionTimeoutMs());
-        assertEquals(sessionTimeoutMs, request.data().rebalanceTimeoutMs());
-    }
 }
