@@ -469,17 +469,17 @@ public class CommonNameLoggingTrustManagerFactoryWrapperTest {
             CommonNameLoggingTrustManager testTrustManager = new CommonNameLoggingTrustManager(origTrustManager, 2);
 
             // Call with valid certificate
-            //assertDoesNotThrow(() -> testTrustManager.checkClientTrusted(validChainWithoutCa, "RSA"));
+            assertDoesNotThrow(() -> testTrustManager.checkClientTrusted(validChainWithoutCa, "RSA"));
             // Call with invalid certificate
             assertThrows(CertificateException.class,
                     () -> testTrustManager.checkClientTrusted(invalidChainWithoutCa, "RSA"));
             // Call with valid certificate again
-            //assertDoesNotThrow(() -> testTrustManager.checkClientTrusted(validChainWithoutCa, "RSA"));
+            assertDoesNotThrow(() -> testTrustManager.checkClientTrusted(validChainWithoutCa, "RSA"));
             // Call with invalid certificate
             assertThrows(CertificateException.class,
                     () -> testTrustManager.checkClientTrusted(invalidChainWithoutCa, "RSA"));
             // Call with valid certificate again
-            //assertDoesNotThrow(() -> testTrustManager.checkClientTrusted(validChainWithoutCa, "RSA"));
+            assertDoesNotThrow(() -> testTrustManager.checkClientTrusted(validChainWithoutCa, "RSA"));
         }
     }
 
