@@ -18,7 +18,6 @@ package org.apache.kafka.connect.mirror;
 
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.config.ConfigDef;
-import org.apache.kafka.common.utils.ConfigUtils;
 
 import java.time.Duration;
 import java.util.List;
@@ -40,13 +39,11 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
     public static final String TOPICS_DEFAULT = DefaultTopicFilter.TOPICS_INCLUDE_DEFAULT;
     private static final String TOPICS_DOC = "Topics to replicate. Supports comma-separated topic names and regexes.";
     public static final String TOPICS_EXCLUDE = DefaultTopicFilter.TOPICS_EXCLUDE_CONFIG;
-    public static final String TOPICS_EXCLUDE_ALIAS = DefaultTopicFilter.TOPICS_EXCLUDE_CONFIG_ALIAS;
     public static final String TOPICS_EXCLUDE_DEFAULT = DefaultTopicFilter.TOPICS_EXCLUDE_DEFAULT;
     private static final String TOPICS_EXCLUDE_DOC = "Excluded topics. Supports comma-separated topic names and regexes."
             + " Excludes take precedence over includes.";
 
     public static final String CONFIG_PROPERTIES_EXCLUDE = DefaultConfigPropertyFilter.CONFIG_PROPERTIES_EXCLUDE_CONFIG;
-    public static final String CONFIG_PROPERTIES_EXCLUDE_ALIAS = DefaultConfigPropertyFilter.CONFIG_PROPERTIES_EXCLUDE_ALIAS_CONFIG;
     public static final String CONFIG_PROPERTIES_EXCLUDE_DEFAULT = DefaultConfigPropertyFilter.CONFIG_PROPERTIES_EXCLUDE_DEFAULT;
     private static final String CONFIG_PROPERTIES_EXCLUDE_DOC = "Topic config properties that should not be replicated. Supports "
             + "comma-separated property names and regexes.";
@@ -74,19 +71,6 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
     public static final String SYNC_TOPIC_CONFIGS_INTERVAL_SECONDS = SYNC_TOPIC_CONFIGS + INTERVAL_SECONDS_SUFFIX;
     private static final String SYNC_TOPIC_CONFIGS_INTERVAL_SECONDS_DOC = "Frequency of topic config sync.";
     public static final long SYNC_TOPIC_CONFIGS_INTERVAL_SECONDS_DEFAULT = 10 * 60;
-    @Deprecated
-    public static final String USE_INCREMENTAL_ALTER_CONFIGS = "use.incremental.alter.configs";
-    private static final String USE_INCREMENTAL_ALTER_CONFIG_DOC = "Deprecated. Which API to use for syncing topic configs. " +
-            "The valid values are 'requested', 'required' and 'never'. " +
-            "By default, set to 'requested', which means the IncrementalAlterConfigs API is being used for syncing topic configurations " +
-            "and if any request receives an error from an incompatible broker, it will fallback to using the deprecated AlterConfigs API. " +
-            "If explicitly set to 'required', the IncrementalAlterConfigs API is used without the fallback logic and +" +
-            "if it receives an error from an incompatible broker, the connector will fail." +
-            "If explicitly set to 'never', the AlterConfig is always used." +
-            "This setting will be removed and the behaviour of 'required' will be used in Kafka 4.0, therefore users should ensure that target broker is at least 2.3.0";
-    public static final String REQUEST_INCREMENTAL_ALTER_CONFIGS = "requested";
-    public static final String REQUIRE_INCREMENTAL_ALTER_CONFIGS = "required";
-    public static final String NEVER_USE_INCREMENTAL_ALTER_CONFIGS = "never";
 
     public static final String SYNC_TOPIC_ACLS_ENABLED = SYNC_TOPIC_ACLS + ENABLED_SUFFIX;
     private static final String SYNC_TOPIC_ACLS_ENABLED_DOC = "Whether to periodically configure remote topic ACLs to match their corresponding upstream topics.";
@@ -104,20 +88,19 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
     private static final String OFFSET_LAG_MAX_DOC = "How out-of-sync a remote partition can be before it is resynced.";
     public static final long OFFSET_LAG_MAX_DEFAULT = 100L;
 
-    public static final String ADD_SOURCE_ALIAS_TO_METRICS = "add.source.alias.to.metrics";
-    private static final String ADD_SOURCE_ALIAS_TO_METRICS_DOC = "Deprecated. Whether to tag metrics with the source cluster alias. "
-        + "Metrics have the target, topic and partition tags. When this setting is enabled, it adds the source tag. "
-        + "This configuration will be removed in Kafka 4.0 and the default behavior will be to always have the source tag.";
-    public static final boolean ADD_SOURCE_ALIAS_TO_METRICS_DEFAULT = false;
+    public static final String HEARTBEATS_REPLICATION_ENABLED = "heartbeats.replication" + ENABLED_SUFFIX;
+    private static final String HEARTBEATS_REPLICATION_ENABLED_DOC = "Whether to replicate the heartbeats topics even when the topic filter does not include them." +
+            " If set to true, heartbeats topics identified by the replication policy will always be replicated, regardless of the topic filter configuration." +
+            " If set to false, heartbeats topics will only be replicated if the topic filter allows.";
+    public static final boolean HEARTBEATS_REPLICATION_ENABLED_DEFAULT = true;
+
     public static final String OFFSET_SYNCS_SOURCE_PRODUCER_ROLE = OFFSET_SYNCS_CLIENT_ROLE_PREFIX + "source-producer";
     public static final String OFFSET_SYNCS_TARGET_PRODUCER_ROLE = OFFSET_SYNCS_CLIENT_ROLE_PREFIX + "target-producer";
     public static final String OFFSET_SYNCS_SOURCE_ADMIN_ROLE = OFFSET_SYNCS_CLIENT_ROLE_PREFIX + "source-admin";
     public static final String OFFSET_SYNCS_TARGET_ADMIN_ROLE = OFFSET_SYNCS_CLIENT_ROLE_PREFIX + "target-admin";
 
     public MirrorSourceConfig(Map<String, String> props) {
-        super(CONNECTOR_CONFIG_DEF, ConfigUtils.translateDeprecatedConfigs(props, new String[][]{
-                {TOPICS_EXCLUDE, TOPICS_EXCLUDE_ALIAS},
-                {CONFIG_PROPERTIES_EXCLUDE, CONFIG_PROPERTIES_EXCLUDE_ALIAS}}));
+        super(CONNECTOR_CONFIG_DEF, props);
     }
 
     public MirrorSourceConfig(ConfigDef configDef, Map<String, String> props) {
@@ -183,10 +166,6 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
         }
     }
 
-    String useIncrementalAlterConfigs() {
-        return getString(USE_INCREMENTAL_ALTER_CONFIGS);
-    }
-
     Duration syncTopicAclsInterval() {
         if (getBoolean(SYNC_TOPIC_ACLS_ENABLED)) {
             return Duration.ofSeconds(getLong(SYNC_TOPIC_ACLS_INTERVAL_SECONDS));
@@ -216,14 +195,14 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
         return Duration.ofMillis(getLong(CONSUMER_POLL_TIMEOUT_MILLIS));
     }
 
-    boolean addSourceAliasToMetrics() {
-        return getBoolean(ADD_SOURCE_ALIAS_TO_METRICS);
-    }
-
     boolean emitOffsetSyncsEnabled() {
         return getBoolean(EMIT_OFFSET_SYNCS_ENABLED);
     }
 
+    boolean heartbeatsReplicationEnabled() {
+        return getBoolean(HEARTBEATS_REPLICATION_ENABLED);
+    }
+
     private static ConfigDef defineSourceConfig(ConfigDef baseConfig) {
         return baseConfig
                 .define(
@@ -238,24 +217,12 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
                         TOPICS_EXCLUDE_DEFAULT,
                         ConfigDef.Importance.HIGH,
                         TOPICS_EXCLUDE_DOC)
-                .define(
-                        TOPICS_EXCLUDE_ALIAS,
-                        ConfigDef.Type.LIST,
-                        null,
-                        ConfigDef.Importance.HIGH,
-                        "Deprecated. Use " + TOPICS_EXCLUDE + " instead.")
                 .define(
                         CONFIG_PROPERTIES_EXCLUDE,
                         ConfigDef.Type.LIST,
                         CONFIG_PROPERTIES_EXCLUDE_DEFAULT,
                         ConfigDef.Importance.HIGH,
                         CONFIG_PROPERTIES_EXCLUDE_DOC)
-                .define(
-                        CONFIG_PROPERTIES_EXCLUDE_ALIAS,
-                        ConfigDef.Type.LIST,
-                        null,
-                        ConfigDef.Importance.HIGH,
-                        "Deprecated. Use " + CONFIG_PROPERTIES_EXCLUDE + " instead.")
                 .define(
                         TOPIC_FILTER_CLASS,
                         ConfigDef.Type.CLASS,
@@ -298,13 +265,6 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
                         SYNC_TOPIC_CONFIGS_INTERVAL_SECONDS_DEFAULT,
                         ConfigDef.Importance.LOW,
                         SYNC_TOPIC_CONFIGS_INTERVAL_SECONDS_DOC)
-                .define(
-                        USE_INCREMENTAL_ALTER_CONFIGS,
-                        ConfigDef.Type.STRING,
-                        REQUEST_INCREMENTAL_ALTER_CONFIGS,
-                        in(REQUEST_INCREMENTAL_ALTER_CONFIGS, REQUIRE_INCREMENTAL_ALTER_CONFIGS, NEVER_USE_INCREMENTAL_ALTER_CONFIGS),
-                        ConfigDef.Importance.LOW,
-                        USE_INCREMENTAL_ALTER_CONFIG_DOC)
                 .define(
                         SYNC_TOPIC_ACLS_ENABLED,
                         ConfigDef.Type.BOOLEAN,
@@ -342,18 +302,19 @@ public class MirrorSourceConfig extends MirrorConnectorConfig {
                         in(SOURCE_CLUSTER_ALIAS_DEFAULT, TARGET_CLUSTER_ALIAS_DEFAULT),
                         ConfigDef.Importance.LOW,
                         OFFSET_SYNCS_TOPIC_LOCATION_DOC)
-                .define(
-                        ADD_SOURCE_ALIAS_TO_METRICS,
-                        ConfigDef.Type.BOOLEAN,
-                        ADD_SOURCE_ALIAS_TO_METRICS_DEFAULT,
-                        ConfigDef.Importance.LOW,
-                        ADD_SOURCE_ALIAS_TO_METRICS_DOC)
                 .define(
                         EMIT_OFFSET_SYNCS_ENABLED,
                         ConfigDef.Type.BOOLEAN,
                         EMIT_OFFSET_SYNCS_ENABLED_DEFAULT,
                         ConfigDef.Importance.LOW,
                         EMIT_OFFSET_SYNCS_ENABLED_DOC
+                )
+                .define(
+                        HEARTBEATS_REPLICATION_ENABLED,
+                        ConfigDef.Type.BOOLEAN,
+                        HEARTBEATS_REPLICATION_ENABLED_DEFAULT,
+                        ConfigDef.Importance.LOW,
+                        HEARTBEATS_REPLICATION_ENABLED_DOC
                 );
     }
 
