@@ -16,93 +16,72 @@
  */
 package org.apache.kafka.connect.mirror;
 
-import org.apache.kafka.connect.data.Schema;
-import org.apache.kafka.connect.source.SourceTask;
-import org.apache.kafka.connect.source.SourceRecord;
-import org.apache.kafka.connect.header.Headers;
-import org.apache.kafka.connect.header.ConnectHeaders;
 import org.apache.kafka.clients.consumer.ConsumerRecord;
 import org.apache.kafka.clients.consumer.ConsumerRecords;
 import org.apache.kafka.clients.consumer.KafkaConsumer;
 import org.apache.kafka.clients.producer.RecordMetadata;
-import org.apache.kafka.clients.producer.ProducerRecord;
-import org.apache.kafka.clients.producer.KafkaProducer;
-import org.apache.kafka.common.TopicPartition;
-import org.apache.kafka.common.header.Header;
-import org.apache.kafka.common.errors.WakeupException;
 import org.apache.kafka.common.KafkaException;
+import org.apache.kafka.common.TopicPartition;
+import org.apache.kafka.common.errors.WakeupException;
+import org.apache.kafka.common.header.Header;
 import org.apache.kafka.common.utils.Utils;
+import org.apache.kafka.connect.data.Schema;
+import org.apache.kafka.connect.header.ConnectHeaders;
+import org.apache.kafka.connect.header.Headers;
+import org.apache.kafka.connect.source.SourceRecord;
+import org.apache.kafka.connect.source.SourceTask;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.util.Iterator;
-import java.util.LinkedHashMap;
-import java.util.Map;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Set;
-import java.util.ArrayList;
-import java.util.stream.Collectors;
-import java.util.concurrent.Semaphore;
 import java.time.Duration;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.Semaphore;
+import java.util.stream.Collectors;
 
 /** Replicates a set of topic-partitions. */
 public class MirrorSourceTask extends SourceTask {
 
     private static final Logger log = LoggerFactory.getLogger(MirrorSourceTask.class);
 
-    private static final int MAX_OUTSTANDING_OFFSET_SYNCS = 10;
-
     private KafkaConsumer<byte[], byte[]> consumer;
-    private KafkaProducer<byte[], byte[]> offsetProducer;
     private String sourceClusterAlias;
-    private String offsetSyncsTopic;
     private Duration pollTimeout;
-    private long maxOffsetLag;
-    private Map<TopicPartition, PartitionState> partitionStates;
     private ReplicationPolicy replicationPolicy;
     private MirrorSourceMetrics metrics;
     private boolean stopping = false;
-    private final Map<TopicPartition, OffsetSync> delayedOffsetSyncs = new LinkedHashMap<>();
-    private final Map<TopicPartition, OffsetSync> pendingOffsetSyncs = new LinkedHashMap<>();
-    private Semaphore outstandingOffsetSyncs;
     private Semaphore consumerAccess;
+    private OffsetSyncWriter offsetSyncWriter;
 
     public MirrorSourceTask() {}
 
     // for testing
     MirrorSourceTask(KafkaConsumer<byte[], byte[]> consumer, MirrorSourceMetrics metrics, String sourceClusterAlias,
-                     ReplicationPolicy replicationPolicy, long maxOffsetLag, KafkaProducer<byte[], byte[]> producer,
-                     Semaphore outstandingOffsetSyncs, Map<TopicPartition, PartitionState> partitionStates,
-                     String offsetSyncsTopic) {
+                     ReplicationPolicy replicationPolicy,
+                     OffsetSyncWriter offsetSyncWriter) {
         this.consumer = consumer;
         this.metrics = metrics;
         this.sourceClusterAlias = sourceClusterAlias;
         this.replicationPolicy = replicationPolicy;
-        this.maxOffsetLag = maxOffsetLag;
         consumerAccess = new Semaphore(1);
-        this.offsetProducer = producer;
-        this.outstandingOffsetSyncs = outstandingOffsetSyncs;
-        this.partitionStates = partitionStates;
-        this.offsetSyncsTopic = offsetSyncsTopic;
+        this.offsetSyncWriter = offsetSyncWriter;
     }
 
     @Override
     public void start(Map<String, String> props) {
         MirrorSourceTaskConfig config = new MirrorSourceTaskConfig(props);
-        pendingOffsetSyncs.clear();
-        outstandingOffsetSyncs = new Semaphore(MAX_OUTSTANDING_OFFSET_SYNCS);
         consumerAccess = new Semaphore(1);  // let one thread at a time access the consumer
         sourceClusterAlias = config.sourceClusterAlias();
         metrics = config.metrics();
         pollTimeout = config.consumerPollTimeout();
-        maxOffsetLag = config.maxOffsetLag();
         replicationPolicy = config.replicationPolicy();
-        partitionStates = new HashMap<>();
-        offsetSyncsTopic = config.offsetSyncsTopic();
+        if (config.emitOffsetSyncsEnabled()) {
+            offsetSyncWriter = new OffsetSyncWriter(config);
+        }
         consumer = MirrorUtils.newConsumer(config.sourceConsumerConfig("replication-consumer"));
-        offsetProducer = MirrorUtils.newProducer(config.offsetSyncsTopicProducerConfig());
         Set<TopicPartition> taskTopicPartitions = config.taskTopicPartitions();
         initializeConsumer(taskTopicPartitions);
 
@@ -112,12 +91,15 @@ public class MirrorSourceTask extends SourceTask {
 
     @Override
     public void commit() {
-        // Offset syncs which were not emitted immediately due to their offset spacing should be sent periodically
-        // This ensures that low-volume topics aren't left with persistent lag at the end of the topic
-        promoteDelayedOffsetSyncs();
-        // Publish any offset syncs that we've queued up, but have not yet been able to publish
-        // (likely because we previously reached our limit for number of outstanding syncs)
-        firePendingOffsetSyncs();
+        // Handle delayed and pending offset syncs only when offsetSyncWriter is available
+        if (offsetSyncWriter != null) {
+            // Offset syncs which were not emitted immediately due to their offset spacing should be sent periodically
+            // This ensures that low-volume topics aren't left with persistent lag at the end of the topic
+            offsetSyncWriter.promoteDelayedOffsetSyncs();
+            // Publish any offset syncs that we've queued up, but have not yet been able to publish
+            // (likely because we previously reached our limit for number of outstanding syncs)
+            offsetSyncWriter.firePendingOffsetSyncs();
+        }
     }
 
     @Override
@@ -131,7 +113,7 @@ public class MirrorSourceTask extends SourceTask {
             log.warn("Interrupted waiting for access to consumer. Will try closing anyway."); 
         }
         Utils.closeQuietly(consumer, "source consumer");
-        Utils.closeQuietly(offsetProducer, "offset producer");
+        Utils.closeQuietly(offsetSyncWriter, "offset sync writer");
         Utils.closeQuietly(metrics, "metrics");
         log.info("Stopping {} took {} ms.", Thread.currentThread().getName(), System.currentTimeMillis() - start);
     }
@@ -197,79 +179,16 @@ public class MirrorSourceTask extends SourceTask {
         long latency = System.currentTimeMillis() - record.timestamp();
         metrics.countRecord(topicPartition);
         metrics.replicationLatency(topicPartition, latency);
-        TopicPartition sourceTopicPartition = MirrorUtils.unwrapPartition(record.sourcePartition());
-        long upstreamOffset = MirrorUtils.unwrapOffset(record.sourceOffset());
-        long downstreamOffset = metadata.offset();
-        maybeQueueOffsetSyncs(sourceTopicPartition, upstreamOffset, downstreamOffset);
-        // We may be able to immediately publish an offset sync that we've queued up here
-        firePendingOffsetSyncs();
-    }
-
-    // updates partition state and queues up OffsetSync if necessary
-    private void maybeQueueOffsetSyncs(TopicPartition topicPartition, long upstreamOffset,
-                                       long downstreamOffset) {
-        PartitionState partitionState =
-            partitionStates.computeIfAbsent(topicPartition, x -> new PartitionState(maxOffsetLag));
-        OffsetSync offsetSync = new OffsetSync(topicPartition, upstreamOffset, downstreamOffset);
-        if (partitionState.update(upstreamOffset, downstreamOffset)) {
-            // Queue this sync for an immediate send, as downstream state is sufficiently stale
-            synchronized (this) {
-                delayedOffsetSyncs.remove(topicPartition);
-                pendingOffsetSyncs.put(topicPartition, offsetSync);
-            }
-            partitionState.reset();
-        } else {
-            // Queue this sync to be delayed until the next periodic offset commit
-            synchronized (this) {
-                delayedOffsetSyncs.put(topicPartition, offsetSync);
-            }
+        // Queue offset syncs only when offsetWriter is available
+        if (offsetSyncWriter != null) {
+            TopicPartition sourceTopicPartition = MirrorUtils.unwrapPartition(record.sourcePartition());
+            long upstreamOffset = MirrorUtils.unwrapOffset(record.sourceOffset());
+            long downstreamOffset = metadata.offset();
+            offsetSyncWriter.maybeQueueOffsetSyncs(sourceTopicPartition, upstreamOffset, downstreamOffset);
+            // We may be able to immediately publish an offset sync that we've queued up here
+            offsetSyncWriter.firePendingOffsetSyncs();
         }
     }
-
-    private synchronized void promoteDelayedOffsetSyncs() {
-        pendingOffsetSyncs.putAll(delayedOffsetSyncs);
-        delayedOffsetSyncs.clear();
-    }
-
-    private void firePendingOffsetSyncs() {
-        while (true) {
-            OffsetSync pendingOffsetSync;
-            synchronized (this) {
-                Iterator<OffsetSync> syncIterator = pendingOffsetSyncs.values().iterator();
-                if (!syncIterator.hasNext()) {
-                    // Nothing to sync
-                    log.trace("No more pending offset syncs");
-                    return;
-                }
-                pendingOffsetSync = syncIterator.next();
-                if (!outstandingOffsetSyncs.tryAcquire()) {
-                    // Too many outstanding syncs
-                    log.trace("Too many in-flight offset syncs; will try to send remaining offset syncs later");
-                    return;
-                }
-                syncIterator.remove();
-            }
-            // Publish offset sync outside of synchronized block; we may have to
-            // wait for producer metadata to update before Producer::send returns
-            sendOffsetSync(pendingOffsetSync);
-            log.trace("Dispatched offset sync for {}", pendingOffsetSync.topicPartition());
-        }
-    }
-
-    // sends OffsetSync record to internal offsets topic
-    private void sendOffsetSync(OffsetSync offsetSync) {
-        ProducerRecord<byte[], byte[]> record = new ProducerRecord<>(offsetSyncsTopic, 0,
-                offsetSync.recordKey(), offsetSync.recordValue());
-        offsetProducer.send(record, (x, e) -> {
-            if (e != null) {
-                log.error("Failure sending offset sync.", e);
-            } else {
-                log.trace("Sync'd offsets for {}: {}=={}", offsetSync.topicPartition(),
-                    offsetSync.upstreamOffset(), offsetSync.downstreamOffset());
-            }
-            outstandingOffsetSyncs.release();
-        });
-    }
  
     private Map<TopicPartition, Long> loadOffsets(Set<TopicPartition> topicPartitions) {
         return topicPartitions.stream().collect(Collectors.toMap(x -> x, this::loadOffset));
@@ -336,38 +255,4 @@ public class MirrorSourceTask extends SourceTask {
     private boolean isUncommitted(Long offset) {
         return offset == null || offset < 0;
     }
-
-    static class PartitionState {
-        long previousUpstreamOffset = -1L;
-        long previousDownstreamOffset = -1L;
-        long lastSyncDownstreamOffset = -1L;
-        long maxOffsetLag;
-        boolean shouldSyncOffsets;
-
-        PartitionState(long maxOffsetLag) {
-            this.maxOffsetLag = maxOffsetLag;
-        }
-
-        // true if we should emit an offset sync
-        boolean update(long upstreamOffset, long downstreamOffset) {
-            // Emit an offset sync if any of the following conditions are true
-            boolean noPreviousSyncThisLifetime = lastSyncDownstreamOffset == -1L;
-            // the OffsetSync::translateDownstream method will translate this offset 1 past the last sync, so add 1.
-            // TODO: share common implementation to enforce this relationship
-            boolean translatedOffsetTooStale = downstreamOffset - (lastSyncDownstreamOffset + 1) >= maxOffsetLag;
-            boolean skippedUpstreamRecord = upstreamOffset - previousUpstreamOffset != 1L;
-            boolean truncatedDownstreamTopic = downstreamOffset < previousDownstreamOffset;
-            if (noPreviousSyncThisLifetime || translatedOffsetTooStale || skippedUpstreamRecord || truncatedDownstreamTopic) {
-                lastSyncDownstreamOffset = downstreamOffset;
-                shouldSyncOffsets = true;
-            }
-            previousUpstreamOffset = upstreamOffset;
-            previousDownstreamOffset = downstreamOffset;
-            return shouldSyncOffsets;
-        }
-
-        void reset() {
-            shouldSyncOffsets = false;
-        }
-    }
 }
