@@ -27,7 +27,7 @@ import org.apache.kafka.common.metrics.stats.TokenBucket
 import org.apache.kafka.common.protocol.Errors
 import org.apache.kafka.common.utils.Time
 import org.apache.kafka.network.Session
-import org.apache.kafka.server.quota.ClientQuotaCallback
+import org.apache.kafka.server.quota.{ClientQuotaCallback, QuotaType}
 import org.apache.kafka.server.config.ClientQuotaManagerConfig
 
 import scala.jdk.CollectionConverters._
@@ -166,16 +166,16 @@ class ControllerMutationQuotaManager(private val config: ClientQuotaManagerConfi
                                      private val time: Time,
                                      private val threadNamePrefix: String,
                                      private val quotaCallback: Option[ClientQuotaCallback])
-    extends ClientQuotaManager(config, metrics, QuotaType.ControllerMutation, time, threadNamePrefix, quotaCallback) {
+    extends ClientQuotaManager(config, metrics, QuotaType.CONTROLLER_MUTATION, time, threadNamePrefix, quotaCallback) {
 
   override protected def clientQuotaMetricName(quotaMetricTags: Map[String, String]): MetricName = {
-    metrics.metricName("tokens", QuotaType.ControllerMutation.toString,
+    metrics.metricName("tokens", QuotaType.CONTROLLER_MUTATION.toString,
       "Tracking remaining tokens in the token bucket per user/client-id",
       quotaMetricTags.asJava)
   }
 
   private def clientRateMetricName(quotaMetricTags: Map[String, String]): MetricName = {
-    metrics.metricName("mutation-rate", QuotaType.ControllerMutation.toString,
+    metrics.metricName("mutation-rate", QuotaType.CONTROLLER_MUTATION.toString,
       "Tracking mutation-rate per user/client-id",
       quotaMetricTags.asJava)
   }
