@@ -17,12 +17,9 @@
 
 package kafka.server
 
-import com.yammer.metrics.core.MetricName
 import kafka.log.LogManager
 import kafka.log.remote.RemoteLogManager
-import kafka.metrics.LinuxIoMetricsCollector
 import kafka.network.SocketServer
-import kafka.security.CredentialProvider
 import kafka.utils.Logging
 import org.apache.kafka.common.ClusterResource
 import org.apache.kafka.common.internals.ClusterResourceListeners
@@ -32,18 +29,21 @@ import org.apache.kafka.common.security.token.delegation.internals.DelegationTok
 import org.apache.kafka.common.utils.Time
 import org.apache.kafka.coordinator.group.GroupCoordinator
 import org.apache.kafka.metadata.BrokerState
-import org.apache.kafka.server.NodeToControllerChannelManager
+import org.apache.kafka.security.CredentialProvider
 import org.apache.kafka.server.authorizer.Authorizer
-import org.apache.kafka.server.metrics.{KafkaMetricsGroup, KafkaYammerMetrics}
+import org.apache.kafka.server.common.NodeToControllerChannelManager
+import org.apache.kafka.server.metrics.{KafkaMetricsGroup, KafkaYammerMetrics, LinuxIoMetricsCollector}
 import org.apache.kafka.server.util.Scheduler
+import org.apache.kafka.storage.internals.log.LogDirFailureChannel
+import org.apache.kafka.storage.log.metrics.BrokerTopicStats
 
-import java.util
+import java.time.Duration
 import scala.collection.Seq
 import scala.jdk.CollectionConverters._
 
 object KafkaBroker {
   //properties for MetricsContext
-  val MetricsTypeName: String = "KafkaServer"
+  private val MetricsTypeName: String = "KafkaServer"
 
   private[server] def notifyClusterListeners(clusterId: String,
                                              clusterListeners: Seq[AnyRef]): Unit = {
@@ -69,9 +69,14 @@ object KafkaBroker {
    * you do change it, be sure to make it match that regex or the system tests will fail.
    */
   val STARTED_MESSAGE = "Kafka Server started"
+
+  val MIN_INCREMENTAL_FETCH_SESSION_EVICTION_MS: Long = 120000
 }
 
 trait KafkaBroker extends Logging {
+  // Number of shards to split FetchSessionCache into. This is to reduce contention when trying to
+  // acquire lock while handling Fetch requests.
+  val NumFetchSessionCacheShards: Int = 8
 
   def authorizer: Option[Authorizer]
   def brokerState: BrokerState
@@ -81,6 +86,7 @@ trait KafkaBroker extends Logging {
   def dataPlaneRequestProcessor: KafkaApis
   def kafkaScheduler: Scheduler
   def kafkaYammerMetrics: KafkaYammerMetrics
+  def logDirFailureChannel: LogDirFailureChannel
   def logManager: LogManager
   def remoteLogManagerOpt: Option[RemoteLogManager]
   def metrics: Metrics
@@ -92,25 +98,23 @@ trait KafkaBroker extends Logging {
   def boundPort(listenerName: ListenerName): Int
   def startup(): Unit
   def awaitShutdown(): Unit
-  def shutdown(): Unit
+  def shutdown(): Unit = shutdown(Duration.ofMinutes(5))
+  def shutdown(timeout: Duration): Unit
+  def isShutdown(): Boolean
   def brokerTopicStats: BrokerTopicStats
   def credentialProvider: CredentialProvider
   def clientToControllerChannelManager: NodeToControllerChannelManager
   def tokenCache: DelegationTokenCache
 
-  private val metricsGroup = new KafkaMetricsGroup(this.getClass) {
-    // For backwards compatibility, we need to keep older metrics tied
-    // to their original name when this class was named `KafkaServer`
-    override def metricName(name: String, tags: util.Map[String, String]): MetricName = {
-      KafkaMetricsGroup.explicitMetricName(Server.MetricsPrefix, KafkaBroker.MetricsTypeName, name, tags)
-    }
-  }
+  // For backwards compatibility, we need to keep older metrics tied
+  // to their original name when this class was named `KafkaServer`
+  private val metricsGroup = new KafkaMetricsGroup(Server.MetricsPrefix, KafkaBroker.MetricsTypeName)
 
   metricsGroup.newGauge("BrokerState", () => brokerState.value)
   metricsGroup.newGauge("ClusterId", () => clusterId)
   metricsGroup.newGauge("yammer-metrics-count", () =>  KafkaYammerMetrics.defaultRegistry.allMetrics.size)
 
-  private val linuxIoMetricsCollector = new LinuxIoMetricsCollector("/proc", Time.SYSTEM, logger.underlying)
+  private val linuxIoMetricsCollector = new LinuxIoMetricsCollector("/proc", Time.SYSTEM)
 
   if (linuxIoMetricsCollector.usable()) {
     metricsGroup.newGauge("linux-disk-read-bytes", () => linuxIoMetricsCollector.readBytes())
