@@ -23,6 +23,7 @@ import org.apache.kafka.common.{TopicPartition, Uuid}
 import org.apache.kafka.common.requests.FetchResponse
 import org.apache.kafka.server.common.{DirectoryEventHandler, OffsetAndEpoch, TopicIdPartition}
 import org.apache.kafka.storage.internals.log.{LogAppendInfo, LogStartOffsetIncrementReason}
+import org.apache.kafka.storage.log.metrics.BrokerTopicStats
 
 import java.util.concurrent.ConcurrentHashMap
 import scala.collection.{Map, Set}
@@ -65,12 +66,9 @@ class ReplicaAlterLogDirsThread(name: String,
   }
 
   // process fetched data
-  override def processPartitionData(
-    topicPartition: TopicPartition,
-    fetchOffset: Long,
-    partitionLeaderEpoch: Int,
-    partitionData: FetchData
-  ): Option[LogAppendInfo] = {
+  override def processPartitionData(topicPartition: TopicPartition,
+                                    fetchOffset: Long,
+                                    partitionData: FetchData): Option[LogAppendInfo] = {
     val partition = replicaMgr.getPartitionOrException(topicPartition)
     val futureLog = partition.futureLocalLogOrException
     val records = toMemoryRecords(FetchResponse.recordsOrFail(partitionData))
@@ -80,7 +78,7 @@ class ReplicaAlterLogDirsThread(name: String,
         topicPartition, fetchOffset, futureLog.logEndOffset))
 
     val logAppendInfo = if (records.sizeInBytes() > 0)
-      partition.appendRecordsToFollowerOrFutureReplica(records, isFuture = true, partitionLeaderEpoch)
+      partition.appendRecordsToFollowerOrFutureReplica(records, isFuture = true)
     else
       None
 
@@ -171,8 +169,6 @@ class ReplicaAlterLogDirsThread(name: String,
     }
   }
 
-  override protected val isOffsetForLeaderEpochSupported: Boolean = true
-
   /**
    * Truncate the log for each partition based on current replica's returned epoch and offset.
    *
