@@ -31,8 +31,6 @@ import org.apache.kafka.common.memory.MemoryPool;
 import org.apache.kafka.common.message.DescribeTopicPartitionsRequestData;
 import org.apache.kafka.common.message.DescribeTopicPartitionsResponseData;
 import org.apache.kafka.common.message.DescribeTopicPartitionsResponseData.DescribeTopicPartitionsResponseTopic;
-import org.apache.kafka.common.message.UpdateMetadataRequestData;
-import org.apache.kafka.common.message.UpdateMetadataRequestData.UpdateMetadataBroker;
 import org.apache.kafka.common.metadata.PartitionRecord;
 import org.apache.kafka.common.metadata.RegisterBrokerRecord;
 import org.apache.kafka.common.metadata.RegisterBrokerRecord.BrokerEndpoint;
@@ -59,12 +57,13 @@ import org.apache.kafka.image.MetadataDelta;
 import org.apache.kafka.image.MetadataImage;
 import org.apache.kafka.image.MetadataProvenance;
 import org.apache.kafka.metadata.LeaderRecoveryState;
+import org.apache.kafka.network.SocketServerConfigs;
+import org.apache.kafka.network.metrics.RequestChannelMetrics;
 import org.apache.kafka.raft.QuorumConfig;
 import org.apache.kafka.server.authorizer.Action;
 import org.apache.kafka.server.authorizer.AuthorizationResult;
 import org.apache.kafka.server.authorizer.Authorizer;
 import org.apache.kafka.server.common.KRaftVersion;
-import org.apache.kafka.server.common.MetadataVersion;
 import org.apache.kafka.server.config.KRaftConfigs;
 
 import org.junit.jupiter.api.Test;
@@ -90,7 +89,7 @@ import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
 class DescribeTopicPartitionsRequestHandlerTest {
-    private final RequestChannel.Metrics requestChannelMetrics = mock(RequestChannel.Metrics.class);
+    private final RequestChannelMetrics requestChannelMetrics = mock(RequestChannelMetrics.class);
     private final KafkaPrincipalSerde kafkaPrincipalSerde = new KafkaPrincipalSerde() {
         @Override
         public byte[] serialize(KafkaPrincipal principal) throws SerializationException {
@@ -104,16 +103,13 @@ class DescribeTopicPartitionsRequestHandlerTest {
     };
 
     ListenerName plaintextListener = ListenerName.forSecurityProtocol(SecurityProtocol.PLAINTEXT);
-    UpdateMetadataBroker broker = new UpdateMetadataBroker()
-        .setId(0)
-        .setRack("rack")
-        .setEndpoints(Arrays.asList(
-            new UpdateMetadataRequestData.UpdateMetadataEndpoint()
-                .setHost("broker0")
-                .setPort(9092)
-                .setSecurityProtocol(SecurityProtocol.PLAINTEXT.id)
-                .setListener(plaintextListener.value())
-        ));
+    String rack = "rack";
+    int brokerId = 0;
+    BrokerEndpoint brokerEndpoint = new BrokerEndpoint()
+        .setName(plaintextListener.value())
+        .setHost("broker0")
+        .setPort(9092)
+        .setSecurityProtocol(SecurityProtocol.PLAINTEXT.id);
 
     @Test
     void testDescribeTopicPartitionsRequest() {
@@ -148,19 +144,14 @@ class DescribeTopicPartitionsRequestHandlerTest {
         topicIds.put(unauthorizedTopic, unauthorizedTopicId);
 
         BrokerEndpointCollection collection = new BrokerEndpointCollection();
-        collection.add(new BrokerEndpoint()
-            .setName(broker.endpoints().get(0).listener())
-            .setHost(broker.endpoints().get(0).host())
-            .setPort(broker.endpoints().get(0).port())
-            .setSecurityProtocol(broker.endpoints().get(0).securityProtocol())
-        );
+        collection.add(brokerEndpoint);
         List<ApiMessage> records = Arrays.asList(
             new RegisterBrokerRecord()
-                .setBrokerId(broker.id())
+                .setBrokerId(brokerId)
                 .setBrokerEpoch(0)
                 .setIncarnationId(Uuid.randomUuid())
                 .setEndPoints(collection)
-                .setRack(broker.rack())
+                .setRack(rack)
                 .setFenced(false),
             new TopicRecord().setName(authorizedTopic).setTopicId(topicIds.get(authorizedTopic)),
             new TopicRecord().setName(unauthorizedTopic).setTopicId(topicIds.get(unauthorizedTopic)),
@@ -350,19 +341,14 @@ class DescribeTopicPartitionsRequestHandlerTest {
         topicIds.put(authorizedTopic2, authorizedTopicId2);
 
         BrokerEndpointCollection collection = new BrokerEndpointCollection();
-        collection.add(new BrokerEndpoint()
-                .setName(broker.endpoints().get(0).listener())
-                .setHost(broker.endpoints().get(0).host())
-                .setPort(broker.endpoints().get(0).port())
-                .setSecurityProtocol(broker.endpoints().get(0).securityProtocol())
-        );
+        collection.add(brokerEndpoint);
         List<ApiMessage> records = Arrays.asList(
             new RegisterBrokerRecord()
-                .setBrokerId(broker.id())
+                .setBrokerId(brokerId)
                 .setBrokerEpoch(0)
                 .setIncarnationId(Uuid.randomUuid())
                 .setEndPoints(collection)
-                .setRack(broker.rack())
+                .setRack(rack)
                 .setFenced(false),
             new TopicRecord().setName(authorizedTopic).setTopicId(topicIds.get(authorizedTopic)),
             new TopicRecord().setName(authorizedTopic2).setTopicId(topicIds.get(authorizedTopic2)),
@@ -494,7 +480,7 @@ class DescribeTopicPartitionsRequestHandlerTest {
     void updateKraftMetadataCache(KRaftMetadataCache kRaftMetadataCache, List<ApiMessage> records) {
         MetadataImage image = kRaftMetadataCache.currentImage();
         MetadataImage partialImage = new MetadataImage(
-            new MetadataProvenance(100L, 10, 1000L),
+            new MetadataProvenance(100L, 10, 1000L, true),
             image.features(),
             ClusterImage.EMPTY,
             image.topics(),
@@ -507,7 +493,7 @@ class DescribeTopicPartitionsRequestHandlerTest {
         );
         MetadataDelta delta = new MetadataDelta.Builder().setImage(partialImage).build();
         records.stream().forEach(record -> delta.replay(record));
-        kRaftMetadataCache.setImage(delta.apply(new MetadataProvenance(100L, 10, 1000L)));
+        kRaftMetadataCache.setImage(delta.apply(new MetadataProvenance(100L, 10, 1000L, true)));
     }
 
     private RequestChannel.Request buildRequest(AbstractRequest request,
@@ -533,7 +519,6 @@ class DescribeTopicPartitionsRequestHandlerTest {
         int brokerId = 1;
         Properties properties = TestUtils.createBrokerConfig(
             brokerId,
-            "",
             true,
             true,
             TestUtils.RandomPort(),
@@ -558,7 +543,7 @@ class DescribeTopicPartitionsRequestHandlerTest {
         int voterId = brokerId + 1;
         properties.put(QuorumConfig.QUORUM_VOTERS_CONFIG, voterId + "@localhost:9093");
         properties.put(KRaftConfigs.CONTROLLER_LISTENER_NAMES_CONFIG, "SSL");
-        TestUtils.setIbpAndMessageFormatVersions(properties, MetadataVersion.latestProduction());
+        properties.put(SocketServerConfigs.LISTENER_SECURITY_PROTOCOL_MAP_CONFIG, "PLAINTEXT:PLAINTEXT,SSL:SSL");
         return new KafkaConfig(properties);
     }
-}
\ No newline at end of file
+}
