@@ -17,37 +17,25 @@
 
 package kafka.api
 
-import java.io.File
-import java.util
-import java.util.Properties
-import javax.security.auth.login.Configuration
-import scala.collection.Seq
+import kafka.security.JaasTestUtils
+import kafka.security.JaasTestUtils.JaasSection
 import kafka.security.minikdc.MiniKdc
-import kafka.server.KafkaConfig
-import kafka.utils.JaasTestUtils.{JaasSection, Krb5LoginModule, ZkDigestModule}
-import kafka.utils.{JaasTestUtils, TestUtils}
-import kafka.zk.{AdminZkClient, KafkaZkClient}
+import kafka.utils.TestUtils
 import org.apache.kafka.clients.admin.{Admin, AdminClientConfig, ScramCredentialInfo, UserScramCredentialUpsertion, ScramMechanism => PublicScramMechanism}
 import org.apache.kafka.common.config.SaslConfigs
 import org.apache.kafka.common.config.internals.BrokerSecurityConfigs
 import org.apache.kafka.common.security.JaasUtils
 import org.apache.kafka.common.security.auth.SecurityProtocol
 import org.apache.kafka.common.security.authenticator.LoginManager
-import org.apache.kafka.common.security.scram.internals.{ScramCredentialUtils, ScramFormatter, ScramMechanism}
-import org.apache.kafka.common.utils.Time
-import org.apache.kafka.server.config.ConfigType
-import org.apache.zookeeper.client.ZKClientConfig
 
-import scala.util.Using
-
-/*
- * Implements an enumeration for the modes enabled here:
- * zk only, kafka only, both, custom KafkaServer.
- */
-sealed trait SaslSetupMode
-case object ZkSasl extends SaslSetupMode
-case object KafkaSasl extends SaslSetupMode
-case object Both extends SaslSetupMode
+import java.io.File
+import java.util
+import java.util.Properties
+import javax.security.auth.login.Configuration
+import scala.collection.Seq
+import scala.jdk.CollectionConverters._
+import scala.jdk.OptionConverters._
+import scala.jdk.javaapi.OptionConverters
 
 /*
  * Trait used in SaslTestHarness and EndToEndAuthorizationTest to setup keytab and jaas files.
@@ -62,29 +50,23 @@ trait SaslSetup {
   def startSasl(jaasSections: Seq[JaasSection]): Unit = {
     // Important if tests leak consumers, producers or brokers
     LoginManager.closeAll()
-    val hasKerberos = jaasSections.exists(_.modules.exists {
-      case _: Krb5LoginModule => true
-      case _ => false
-    })
+
+    val hasKerberos = jaasSections.exists(_.getModules.asScala.exists(_.name().endsWith("Krb5LoginModule")))
+
     if (hasKerberos) {
       initializeKerberos()
     }
+
     writeJaasConfigurationToFile(jaasSections)
-    val hasZk = jaasSections.exists(_.modules.exists {
-      case _: ZkDigestModule => true
-      case _ => false
-    })
-    if (hasZk)
-      System.setProperty("zookeeper.authProvider.1", "org.apache.zookeeper.server.auth.SASLAuthenticationProvider")
   }
 
   protected def initializeKerberos(): Unit = {
     val (serverKeytabFile, clientKeytabFile) = maybeCreateEmptyKeytabFiles()
     kdc = new MiniKdc(kdcConf, workDir)
     kdc.start()
-    kdc.createPrincipal(serverKeytabFile, JaasTestUtils.KafkaServerPrincipalUnqualifiedName + "/localhost")
+    kdc.createPrincipal(serverKeytabFile, List(JaasTestUtils.KAFKA_SERVER_PRINCIPAL_UNQUALIFIED_NAME + "/localhost").asJava)
     kdc.createPrincipal(clientKeytabFile,
-      JaasTestUtils.KafkaClientPrincipalUnqualifiedName, JaasTestUtils.KafkaClientPrincipalUnqualifiedName2)
+      List(JaasTestUtils.KAFKA_CLIENT_PRINCIPAL_UNQUALIFIED_NAME, JaasTestUtils.KAFKA_CLIENT_PRINCIPAL_UNQUALIFIED_NAME_2).asJava)
   }
 
   /** Return a tuple with the path to the server keytab file and client keytab file */
@@ -97,25 +79,17 @@ trait SaslSetup {
   }
 
   def jaasSections(kafkaServerSaslMechanisms: Seq[String],
-                             kafkaClientSaslMechanism: Option[String],
-                             mode: SaslSetupMode = Both,
-                             kafkaServerEntryName: String = JaasTestUtils.KafkaServerContextName): Seq[JaasSection] = {
-    val hasKerberos = mode != ZkSasl &&
-      (kafkaServerSaslMechanisms.contains("GSSAPI") || kafkaClientSaslMechanism.contains("GSSAPI"))
+                   kafkaClientSaslMechanism: Option[String],
+                   kafkaServerEntryName: String = JaasTestUtils.KAFKA_SERVER_CONTEXT_NAME): Seq[JaasSection] = {
+    val hasKerberos = kafkaServerSaslMechanisms.contains("GSSAPI") || kafkaClientSaslMechanism.contains("GSSAPI")
     if (hasKerberos)
       maybeCreateEmptyKeytabFiles()
-    mode match {
-      case ZkSasl => JaasTestUtils.zkSections
-      case KafkaSasl =>
-        Seq(JaasTestUtils.kafkaServerSection(kafkaServerEntryName, kafkaServerSaslMechanisms, serverKeytabFile),
-          JaasTestUtils.kafkaClientSection(kafkaClientSaslMechanism, clientKeytabFile))
-      case Both => Seq(JaasTestUtils.kafkaServerSection(kafkaServerEntryName, kafkaServerSaslMechanisms, serverKeytabFile),
-        JaasTestUtils.kafkaClientSection(kafkaClientSaslMechanism, clientKeytabFile)) ++ JaasTestUtils.zkSections
-    }
+    Seq(JaasTestUtils.kafkaServerSection(kafkaServerEntryName, kafkaServerSaslMechanisms.asJava, serverKeytabFile.toJava),
+      JaasTestUtils.kafkaClientSection(kafkaClientSaslMechanism.toJava, clientKeytabFile.toJava))
   }
 
   private def writeJaasConfigurationToFile(jaasSections: Seq[JaasSection]): Unit = {
-    val file = JaasTestUtils.writeJaasContextsToFile(jaasSections)
+    val file = JaasTestUtils.writeJaasContextsToFile(jaasSections.asJava)
     System.setProperty(JaasUtils.JAVA_LOGIN_CONFIG_PARAM, file.getAbsolutePath)
     // This will cause a reload of the Configuration singleton when `getConfiguration` is called
     Configuration.setConfiguration(null)
@@ -127,7 +101,6 @@ trait SaslSetup {
     // Important if tests leak consumers, producers or brokers
     LoginManager.closeAll()
     System.clearProperty(JaasUtils.JAVA_LOGIN_CONFIG_PARAM)
-    System.clearProperty("zookeeper.authProvider.1")
     Configuration.setConfiguration(null)
   }
 
@@ -148,16 +121,16 @@ trait SaslSetup {
 
   def jaasClientLoginModule(clientSaslMechanism: String, serviceName: Option[String] = None): String = {
     if (serviceName.isDefined)
-      JaasTestUtils.clientLoginModule(clientSaslMechanism, clientKeytabFile, serviceName.get)
+      JaasTestUtils.clientLoginModule(clientSaslMechanism, clientKeytabFile.toJava, serviceName.get)
     else
-      JaasTestUtils.clientLoginModule(clientSaslMechanism, clientKeytabFile)
+      JaasTestUtils.clientLoginModule(clientSaslMechanism, clientKeytabFile.toJava)
   }
 
   def jaasAdminLoginModule(clientSaslMechanism: String, serviceName: Option[String] = None): String = {
     if (serviceName.isDefined)
-      JaasTestUtils.adminLoginModule(clientSaslMechanism, serverKeytabFile, serviceName.get)
+      JaasTestUtils.adminLoginModule(clientSaslMechanism, serverKeytabFile.toJava, serviceName.get)
     else
-      JaasTestUtils.adminLoginModule(clientSaslMechanism, serverKeytabFile)
+      JaasTestUtils.adminLoginModule(clientSaslMechanism, serverKeytabFile.toJava)
   }
 
   def jaasScramClientLoginModule(clientSaslScramMechanism: String, scramUser: String, scramPassword: String): String = {
@@ -174,7 +147,7 @@ trait SaslSetup {
     val config = new util.HashMap[String, Object]
     config.put(AdminClientConfig.BOOTSTRAP_SERVERS_CONFIG, brokerList)
     val securityProps: util.Map[Object, Object] =
-      TestUtils.adminClientSecurityConfigs(securityProtocol, trustStoreFile, clientSaslProperties)
+      JaasTestUtils.adminClientSecurityConfigs(securityProtocol, OptionConverters.toJava(trustStoreFile), OptionConverters.toJava(clientSaslProperties))
     securityProps.forEach { (key, value) => config.put(key.asInstanceOf[String], value) }
     config.put(SaslConfigs.SASL_JAAS_CONFIG, jaasScramClientLoginModule(scramMechanism, user, password))
     Admin.create(config)
@@ -198,26 +171,4 @@ trait SaslSetup {
       results.all.get
     })
   }
-
-  def createScramCredentials(zkConnect: String, userName: String, password: String): Unit = {
-    val zkClientConfig = new ZKClientConfig()
-    Using.resource(KafkaZkClient(
-      zkConnect, JaasUtils.isZkSaslEnabled || KafkaConfig.zkTlsClientAuthEnabled(zkClientConfig), 30000, 30000,
-      Int.MaxValue, Time.SYSTEM, name = "SaslSetup", zkClientConfig = zkClientConfig, enableEntityConfigControllerCheck = false)) { zkClient =>
-      val adminZkClient = new AdminZkClient(zkClient)
-
-      val entityType = ConfigType.USER
-      val entityName = userName
-      val configs = adminZkClient.fetchEntityConfig(entityType, entityName)
-
-      ScramMechanism.values().foreach(mechanism => {
-        val credential = new ScramFormatter(mechanism).generateCredential(password, 4096)
-        val credentialString = ScramCredentialUtils.credentialToString(credential)
-        configs.setProperty(mechanism.mechanismName, credentialString)
-      })
-
-      adminZkClient.changeConfigs(entityType, entityName, configs)
-    }
-  }
-
 }
