@@ -20,12 +20,9 @@ import kafka.log.UnifiedLog
 import kafka.server.{KafkaConfig, KafkaRaftServer}
 import kafka.utils.TestUtils
 import org.apache.kafka.common.compress.Compression
-import org.apache.kafka.common.errors.CorruptRecordException
 import org.apache.kafka.common.errors.{InvalidConfigurationException, RecordTooLargeException}
 import org.apache.kafka.common.protocol
 import org.apache.kafka.common.protocol.{ObjectSerializationCache, Writable}
-import org.apache.kafka.common.record.ArbitraryMemoryRecords
-import org.apache.kafka.common.record.InvalidMemoryRecordsProvider
 import org.apache.kafka.common.record.{MemoryRecords, SimpleRecord}
 import org.apache.kafka.common.utils.Utils
 import org.apache.kafka.raft._
@@ -37,14 +34,7 @@ import org.apache.kafka.snapshot.{FileRawSnapshotWriter, RawSnapshotReader, RawS
 import org.apache.kafka.storage.internals.log.{LogConfig, LogStartOffsetIncrementReason}
 import org.apache.kafka.test.TestUtils.assertOptional
 import org.junit.jupiter.api.Assertions._
-import org.junit.jupiter.api.function.Executable
 import org.junit.jupiter.api.{AfterEach, BeforeEach, Test}
-import org.junit.jupiter.params.ParameterizedTest
-import org.junit.jupiter.params.provider.ArgumentsSource
-
-import net.jqwik.api.AfterFailureMode
-import net.jqwik.api.ForAll
-import net.jqwik.api.Property
 
 import java.io.File
 import java.nio.ByteBuffer
@@ -119,93 +109,12 @@ final class KafkaMetadataLogTest {
       classOf[RuntimeException],
       () => {
         log.appendAsFollower(
-          MemoryRecords.withRecords(initialOffset, Compression.NONE, currentEpoch, recordFoo),
-          currentEpoch
+          MemoryRecords.withRecords(initialOffset, Compression.NONE, currentEpoch, recordFoo)
         )
       }
     )
   }
 
-  @Test
-  def testEmptyAppendNotAllowed(): Unit = {
-    val log = buildMetadataLog(tempDir, mockTime)
-
-    assertThrows(classOf[IllegalArgumentException], () => log.appendAsFollower(MemoryRecords.EMPTY, 1));
-    assertThrows(classOf[IllegalArgumentException], () => log.appendAsLeader(MemoryRecords.EMPTY, 1));
-  }
-
-  @ParameterizedTest
-  @ArgumentsSource(classOf[InvalidMemoryRecordsProvider])
-  def testInvalidMemoryRecords(records: MemoryRecords, expectedException: Optional[Class[Exception]]): Unit = {
-    val log = buildMetadataLog(tempDir, mockTime)
-    val previousEndOffset = log.endOffset().offset()
-
-    val action: Executable = () => log.appendAsFollower(records, Int.MaxValue)
-    if (expectedException.isPresent()) {
-      assertThrows(expectedException.get, action)
-    } else {
-      assertThrows(classOf[CorruptRecordException], action)
-    }
-
-    assertEquals(previousEndOffset, log.endOffset().offset())
-  }
-
-  @Property(tries = 100, afterFailure = AfterFailureMode.SAMPLE_ONLY)
-  def testRandomRecords(
-    @ForAll(supplier = classOf[ArbitraryMemoryRecords]) records: MemoryRecords
-  ): Unit = {
-    val tempDir = TestUtils.tempDir()
-    try {
-      val log = buildMetadataLog(tempDir, mockTime)
-      val previousEndOffset = log.endOffset().offset()
-
-      assertThrows(
-        classOf[CorruptRecordException],
-        () => log.appendAsFollower(records, Int.MaxValue)
-      )
-
-      assertEquals(previousEndOffset, log.endOffset().offset())
-    } finally {
-      Utils.delete(tempDir)
-    }
-  }
-
-  @Test
-  def testInvalidLeaderEpoch(): Unit = {
-    val log = buildMetadataLog(tempDir, mockTime)
-    val previousEndOffset = log.endOffset().offset()
-    val epoch = log.lastFetchedEpoch() + 1
-    val numberOfRecords = 10
-
-    val batchWithValidEpoch = MemoryRecords.withRecords(
-      previousEndOffset,
-      Compression.NONE,
-      epoch,
-      (0 until numberOfRecords).map(number => new SimpleRecord(number.toString.getBytes)): _*
-    )
-
-    val batchWithInvalidEpoch = MemoryRecords.withRecords(
-      previousEndOffset + numberOfRecords,
-      Compression.NONE,
-      epoch + 1,
-      (0 until numberOfRecords).map(number => new SimpleRecord(number.toString.getBytes)): _*
-    )
-
-    val buffer = ByteBuffer.allocate(batchWithValidEpoch.sizeInBytes() + batchWithInvalidEpoch.sizeInBytes())
-    buffer.put(batchWithValidEpoch.buffer())
-    buffer.put(batchWithInvalidEpoch.buffer())
-    buffer.flip()
-
-    val records = MemoryRecords.readableRecords(buffer)
-
-    log.appendAsFollower(records, epoch)
-
-    // Check that only the first batch was appended
-    assertEquals(previousEndOffset + numberOfRecords, log.endOffset().offset())
-    // Check that the last fetched epoch matches the first batch
-    assertEquals(epoch, log.lastFetchedEpoch())
-  }
-
   @Test
   def testCreateSnapshot(): Unit = {
     val numberOfRecords = 10
@@ -233,13 +142,24 @@ final class KafkaMetadataLogTest {
 
     // Test finding the first epoch
     log.createNewSnapshot(new OffsetAndEpoch(numberOfRecords, firstEpoch)).get().close()
-    log.createNewSnapshot(new OffsetAndEpoch(numberOfRecords - 1, firstEpoch)).get().close()
-    log.createNewSnapshot(new OffsetAndEpoch(1, firstEpoch)).get().close()
 
     // Test finding the second epoch
     log.createNewSnapshot(new OffsetAndEpoch(2 * numberOfRecords, secondEpoch)).get().close()
-    log.createNewSnapshot(new OffsetAndEpoch(2 * numberOfRecords - 1, secondEpoch)).get().close()
-    log.createNewSnapshot(new OffsetAndEpoch(numberOfRecords + 1, secondEpoch)).get().close()
+  }
+
+  @Test
+  def testCreateSnapshotInMiddleOfBatch(): Unit = {
+    val numberOfRecords = 10
+    val epoch = 1
+    val log = buildMetadataLog(tempDir, mockTime)
+
+    append(log, numberOfRecords, epoch)
+    log.updateHighWatermark(new LogOffsetMetadata(numberOfRecords))
+
+    assertThrows(
+      classOf[IllegalArgumentException],
+      () => log.createNewSnapshot(new OffsetAndEpoch(numberOfRecords - 1, epoch))
+    )
   }
 
   @Test
@@ -297,7 +217,7 @@ final class KafkaMetadataLogTest {
     val snapshotId = new OffsetAndEpoch(numberOfRecords-4, epoch)
     val log = buildMetadataLog(tempDir, mockTime)
 
-    append(log, numberOfRecords, epoch)
+    (1 to numberOfRecords).foreach(_ => append(log, 1, epoch))
     log.updateHighWatermark(new LogOffsetMetadata(numberOfRecords))
     createNewSnapshot(log, snapshotId)
 
@@ -373,7 +293,7 @@ final class KafkaMetadataLogTest {
   def testCreateExistingSnapshot(): Unit = {
     val numberOfRecords = 10
     val epoch = 1
-    val snapshotId = new OffsetAndEpoch(numberOfRecords - 1, epoch)
+    val snapshotId = new OffsetAndEpoch(numberOfRecords, epoch)
     val log = buildMetadataLog(tempDir, mockTime)
 
     append(log, numberOfRecords, epoch)
@@ -1142,4 +1062,4 @@ object KafkaMetadataLogTest {
     }
     dir
   }
-}
+}
\ No newline at end of file
