@@ -26,9 +26,9 @@ import java.util.Properties
 import java.util.stream.IntStream
 import kafka.log.{LogTestUtils, UnifiedLog}
 import kafka.raft.{KafkaMetadataLog, MetadataLogConfig}
-import kafka.server.{BrokerTopicStats, KafkaRaftServer}
-import kafka.tools.DumpLogSegments.{OffsetsMessageParser, TimeIndexDumpErrors}
-import kafka.utils.{Exit, TestUtils, VerifiableProperties}
+import kafka.server.KafkaRaftServer
+import kafka.tools.DumpLogSegments.{OffsetsMessageParser, ShareGroupStateMessageParser, TimeIndexDumpErrors}
+import kafka.utils.TestUtils
 import org.apache.kafka.clients.consumer.ConsumerPartitionAssignor.{Assignment, Subscription}
 import org.apache.kafka.clients.consumer.internals.ConsumerProtocol
 import org.apache.kafka.common.{TopicIdPartition, TopicPartition, Uuid}
@@ -36,20 +36,25 @@ import org.apache.kafka.common.compress.Compression
 import org.apache.kafka.common.config.TopicConfig
 import org.apache.kafka.common.metadata.{PartitionChangeRecord, RegisterBrokerRecord, TopicRecord}
 import org.apache.kafka.common.protocol.{ByteBufferAccessor, ObjectSerializationCache}
-import org.apache.kafka.common.record.{ControlRecordType, EndTransactionMarker, MemoryRecords, Record, RecordVersion, SimpleRecord}
-import org.apache.kafka.common.utils.Utils
-import org.apache.kafka.coordinator.group.{CoordinatorRecord, CoordinatorRecordSerde}
+import org.apache.kafka.common.record.{ControlRecordType, EndTransactionMarker, MemoryRecords, Record, RecordBatch, RecordVersion, SimpleRecord}
+import org.apache.kafka.common.utils.{Exit, Utils}
+import org.apache.kafka.coordinator.common.runtime.CoordinatorRecord
+import org.apache.kafka.coordinator.group.GroupCoordinatorRecordSerde
 import org.apache.kafka.coordinator.group.generated.{ConsumerGroupMemberMetadataValue, ConsumerGroupMetadataKey, ConsumerGroupMetadataValue, GroupMetadataKey, GroupMetadataValue}
-import org.apache.kafka.coordinator.transaction.TransactionLogConfigs
+import org.apache.kafka.coordinator.share.generated.{ShareSnapshotKey, ShareSnapshotValue, ShareUpdateKey, ShareUpdateValue}
+import org.apache.kafka.coordinator.share.{ShareCoordinator, ShareCoordinatorRecordSerde}
+import org.apache.kafka.coordinator.transaction.TransactionLogConfig
 import org.apache.kafka.metadata.MetadataRecordSerde
 import org.apache.kafka.raft.{KafkaRaftClient, OffsetAndEpoch, VoterSetTest}
 import org.apache.kafka.server.common.{ApiMessageAndVersion, KRaftVersion}
 import org.apache.kafka.server.config.ServerLogConfigs
 import org.apache.kafka.server.log.remote.metadata.storage.serialization.RemoteLogMetadataSerde
 import org.apache.kafka.server.log.remote.storage.{RemoteLogSegmentId, RemoteLogSegmentMetadata, RemoteLogSegmentMetadataUpdate, RemoteLogSegmentState, RemotePartitionDeleteMetadata, RemotePartitionDeleteState}
+import org.apache.kafka.server.storage.log.FetchIsolation
 import org.apache.kafka.server.util.MockTime
 import org.apache.kafka.snapshot.RecordsSnapshotWriter
-import org.apache.kafka.storage.internals.log.{AppendOrigin, FetchIsolation, LogConfig, LogDirFailureChannel, ProducerStateManagerConfig}
+import org.apache.kafka.storage.internals.log.{AppendOrigin, LogConfig, LogDirFailureChannel, ProducerStateManagerConfig}
+import org.apache.kafka.storage.log.metrics.BrokerTopicStats
 import org.junit.jupiter.api.Assertions._
 import org.junit.jupiter.api.{AfterEach, Test}
 
@@ -82,6 +87,8 @@ class DumpLogSegmentsTest {
   private def createTestLog = {
     val props = new Properties
     props.setProperty(TopicConfig.INDEX_INTERVAL_BYTES_CONFIG, "128")
+    // This test uses future timestamps beyond the default of 1 hour.
+    props.setProperty(TopicConfig.MESSAGE_TIMESTAMP_AFTER_MAX_MS_CONFIG, Long.MaxValue.toString)
     log = UnifiedLog(
       dir = logDir,
       config = new LogConfig(props),
@@ -91,8 +98,8 @@ class DumpLogSegmentsTest {
       time = time,
       brokerTopicStats = new BrokerTopicStats,
       maxTransactionTimeoutMs = 5 * 60 * 1000,
-      producerStateManagerConfig = new ProducerStateManagerConfig(TransactionLogConfigs.PRODUCER_ID_EXPIRATION_MS_DEFAULT, false),
-      producerIdExpirationCheckIntervalMs = TransactionLogConfigs.PRODUCER_ID_EXPIRATION_CHECK_INTERVAL_MS_DEFAULT,
+      producerStateManagerConfig = new ProducerStateManagerConfig(TransactionLogConfig.PRODUCER_ID_EXPIRATION_MS_DEFAULT, false),
+      producerIdExpirationCheckIntervalMs = TransactionLogConfig.PRODUCER_ID_EXPIRATION_CHECK_INTERVAL_MS_DEFAULT,
       logDirFailureChannel = new LogDirFailureChannel(10),
       topicId = None,
       keepPartitionMetadataFile = true
@@ -218,9 +225,9 @@ class DumpLogSegmentsTest {
     // Verify that records are printed with --print-data-log if --deep-iteration is also specified
     verifyRecordsInOutput(checkKeysAndValues = true, Array("--print-data-log", "--deep-iteration", "--files", logFilePath))
     // Verify that records are printed with --value-decoder even if --print-data-log is not specified
-    verifyRecordsInOutput(checkKeysAndValues = true, Array("--value-decoder-class", "kafka.serializer.StringDecoder", "--files", logFilePath))
+    verifyRecordsInOutput(checkKeysAndValues = true, Array("--value-decoder-class", "org.apache.kafka.tools.api.StringDecoder", "--files", logFilePath))
     // Verify that records are printed with --key-decoder even if --print-data-log is not specified
-    verifyRecordsInOutput(checkKeysAndValues = true, Array("--key-decoder-class", "kafka.serializer.StringDecoder", "--files", logFilePath))
+    verifyRecordsInOutput(checkKeysAndValues = true, Array("--key-decoder-class", "org.apache.kafka.tools.api.StringDecoder", "--files", logFilePath))
     // Verify that records are printed with --deep-iteration even if --print-data-log is not specified
     verifyRecordsInOutput(checkKeysAndValues = false, Array("--deep-iteration", "--files", logFilePath))
 
@@ -395,7 +402,7 @@ class DumpLogSegmentsTest {
     log = LogTestUtils.createLog(logDir, logConfig, new BrokerTopicStats, time.scheduler, time)
     log.appendAsLeader(MemoryRecords.withRecords(Compression.NONE, metadataRecords:_*), leaderEpoch = 0)
     val secondSegment = log.roll()
-    secondSegment.append(1L, MemoryRecords.withRecords(Compression.NONE, metadataRecords: _*))
+    secondSegment.append(1L, RecordBatch.NO_TIMESTAMP, 1L, MemoryRecords.withRecords(Compression.NONE, metadataRecords:_*))
     secondSegment.flush()
     log.flush(true)
     
@@ -450,7 +457,7 @@ class DumpLogSegmentsTest {
 
   @Test
   def testDumpRemoteLogMetadataNoFilesFlag(): Unit = {
-    Exit.setExitProcedure((_, message) => throw new IllegalArgumentException(message.orNull))
+    Exit.setExitProcedure((_, message) => throw new IllegalArgumentException(message))
     try {
       val thrown = assertThrows(classOf[IllegalArgumentException], () => runDumpLogSegments(Array("--remote-log-metadata-decoder")))
       assertTrue(thrown.getMessage.equals("Missing required argument \"[files]\""))
@@ -640,7 +647,7 @@ class DumpLogSegmentsTest {
 
   @Test
   def testOffsetsMessageParser(): Unit = {
-    val serde = new CoordinatorRecordSerde()
+    val serde = new GroupCoordinatorRecordSerde()
     val parser = new OffsetsMessageParser()
 
     def serializedRecord(key: ApiMessageAndVersion, value: ApiMessageAndVersion): Record = {
@@ -826,26 +833,6 @@ class DumpLogSegmentsTest {
     )
   }
 
-  @Test
-  def testNewDecoder(): Unit = {
-    // Decoder translate should pass without exception
-    DumpLogSegments.newDecoder(classOf[DumpLogSegmentsTest.TestDecoder].getName)
-    DumpLogSegments.newDecoder(classOf[kafka.serializer.DefaultDecoder].getName)
-    assertThrows(classOf[Exception], () => DumpLogSegments.newDecoder(classOf[DumpLogSegmentsTest.TestDecoderWithoutVerifiableProperties].getName))
-  }
-
-  @Test
-  def testConvertDeprecatedDecoderClass(): Unit = {
-    assertEquals(classOf[org.apache.kafka.tools.api.DefaultDecoder].getName, DumpLogSegments.convertDeprecatedDecoderClass(
-      classOf[kafka.serializer.DefaultDecoder].getName))
-    assertEquals(classOf[org.apache.kafka.tools.api.IntegerDecoder].getName, DumpLogSegments.convertDeprecatedDecoderClass(
-      classOf[kafka.serializer.IntegerDecoder].getName))
-    assertEquals(classOf[org.apache.kafka.tools.api.LongDecoder].getName, DumpLogSegments.convertDeprecatedDecoderClass(
-      classOf[kafka.serializer.LongDecoder].getName))
-    assertEquals(classOf[org.apache.kafka.tools.api.StringDecoder].getName, DumpLogSegments.convertDeprecatedDecoderClass(
-      classOf[kafka.serializer.StringDecoder].getName))
-  }
-
   private def readBatchMetadata(lines: util.ListIterator[String]): Option[String] = {
     while (lines.hasNext) {
       val line = lines.next()
@@ -979,14 +966,148 @@ class DumpLogSegmentsTest {
       }
     }
   }
-}
 
-object DumpLogSegmentsTest {
-  class TestDecoder(props: VerifiableProperties) extends kafka.serializer.Decoder[Array[Byte]] {
-    override def fromBytes(bytes: Array[Byte]): Array[Byte] = bytes
-  }
+  @Test
+  def testShareGroupStateMessageParser(): Unit = {
+    val serde = new ShareCoordinatorRecordSerde()
+    val parser = new ShareGroupStateMessageParser()
 
-  class TestDecoderWithoutVerifiableProperties() extends kafka.serializer.Decoder[Array[Byte]] {
-    override def fromBytes(bytes: Array[Byte]): Array[Byte] = bytes
+    def serializedRecord(key: ApiMessageAndVersion, value: ApiMessageAndVersion): Record = {
+      val record = new CoordinatorRecord(key, value)
+      TestUtils.singletonRecords(
+        key = serde.serializeKey(record),
+        value = serde.serializeValue(record)
+      ).records.iterator.next
+    }
+
+    // The key is mandatory.
+    assertEquals(
+      "Failed to decode message at offset 0 using share group state topic decoder (message had a missing key)",
+      assertThrows(
+        classOf[RuntimeException],
+        () => parser.parse(TestUtils.singletonRecords(key = null, value = null).records.iterator.next)
+      ).getMessage
+    )
+
+    // A valid key and value should work (ShareSnapshot).
+    assertEquals(
+      (
+        Some("{\"type\":\"0\",\"data\":{\"groupId\":\"gs1\",\"topicId\":\"Uj5wn_FqTXirEASvVZRY1w\",\"partition\":0}}"),
+        Some("{\"type\":\"0\",\"data\":{\"snapshotEpoch\":0,\"stateEpoch\":0,\"leaderEpoch\":0,\"startOffset\":0,\"stateBatches\":[{\"firstOffset\":0,\"lastOffset\":4,\"deliveryState\":2,\"deliveryCount\":1}]}}")
+      ),
+      parser.parse(serializedRecord(
+        new ApiMessageAndVersion(new ShareSnapshotKey()
+          .setGroupId("gs1")
+          .setTopicId(Uuid.fromString("Uj5wn_FqTXirEASvVZRY1w"))
+          .setPartition(0),
+          ShareCoordinator.SHARE_SNAPSHOT_RECORD_KEY_VERSION),
+        new ApiMessageAndVersion(new ShareSnapshotValue()
+          .setSnapshotEpoch(0)
+          .setStateEpoch(0)
+          .setLeaderEpoch(0)
+          .setStartOffset(0)
+          .setStateBatches(List[ShareSnapshotValue.StateBatch](
+            new ShareSnapshotValue.StateBatch()
+              .setFirstOffset(0)
+              .setLastOffset(4)
+              .setDeliveryState(2)
+              .setDeliveryCount(1)
+          ).asJava),
+          ShareCoordinator.SHARE_SNAPSHOT_RECORD_VALUE_VERSION)
+      ))
+    )
+
+    // A valid key and value should work (ShareUpdate).
+    assertEquals(
+      (
+        Some("{\"type\":\"1\",\"data\":{\"groupId\":\"gs1\",\"topicId\":\"Uj5wn_FqTXirEASvVZRY1w\",\"partition\":0}}"),
+        Some("{\"type\":\"0\",\"data\":{\"snapshotEpoch\":0,\"leaderEpoch\":0,\"startOffset\":0,\"stateBatches\":[{\"firstOffset\":0,\"lastOffset\":4,\"deliveryState\":2,\"deliveryCount\":1}]}}")
+      ),
+      parser.parse(serializedRecord(
+        new ApiMessageAndVersion(new ShareUpdateKey()
+          .setGroupId("gs1")
+          .setTopicId(Uuid.fromString("Uj5wn_FqTXirEASvVZRY1w"))
+          .setPartition(0),
+          ShareCoordinator.SHARE_UPDATE_RECORD_KEY_VERSION),
+        new ApiMessageAndVersion(new ShareUpdateValue()
+          .setSnapshotEpoch(0)
+          .setLeaderEpoch(0)
+          .setStartOffset(0)
+          .setStateBatches(List[ShareUpdateValue.StateBatch](
+            new ShareUpdateValue.StateBatch()
+              .setFirstOffset(0)
+              .setLastOffset(4)
+              .setDeliveryState(2)
+              .setDeliveryCount(1)
+          ).asJava),
+          0.toShort)
+      ))
+    )
+
+    // A valid key with a tombstone should work.
+    assertEquals(
+      (
+        Some("{\"type\":\"0\",\"data\":{\"groupId\":\"gs1\",\"topicId\":\"Uj5wn_FqTXirEASvVZRY1w\",\"partition\":0}}"),
+        Some("<DELETE>")
+      ),
+      parser.parse(serializedRecord(
+        new ApiMessageAndVersion(
+          new ShareSnapshotKey()
+            .setGroupId("gs1")
+            .setTopicId(Uuid.fromString("Uj5wn_FqTXirEASvVZRY1w"))
+            .setPartition(0),
+          0.toShort
+        ),
+        null
+      ))
+    )
+
+    // An unknown record type should be handled and reported as such.
+    assertEquals(
+      (
+        Some(
+          "Unknown record type 32767 at offset 0, skipping."
+        ),
+        None
+      ),
+      parser.parse(serializedRecord(
+        new ApiMessageAndVersion(
+          new ShareSnapshotKey()
+            .setGroupId("group")
+            .setTopicId(Uuid.fromString("Uj5wn_FqTXirEASvVZRY1w"))
+            .setPartition(0),
+          Short.MaxValue // Invalid record id.
+        ),
+        new ApiMessageAndVersion(
+          new ShareSnapshotValue()
+            .setSnapshotEpoch(0),
+          0.toShort
+        )
+      ))
+    )
+
+    // Any parsing error is swallowed and reported.
+    assertEquals(
+      (
+        Some(
+          "Error at offset 0, skipping. Could not read record with version 0 from value's buffer due to: " +
+          "non-nullable field stateBatches was serialized as null."
+        ),
+        None
+      ),
+      parser.parse(serializedRecord(
+        new ApiMessageAndVersion(
+          new ShareUpdateKey()
+            .setGroupId("group")
+            .setTopicId(Uuid.fromString("Uj5wn_FqTXirEASvVZRY1w"))
+            .setPartition(0),
+          1.toShort
+        ),
+        new ApiMessageAndVersion(
+          new ShareSnapshotValue(), // incorrect class to deserialize the snapshot update value
+          0.toShort
+        )
+      ))
+    )
   }
 }
