@@ -62,6 +62,8 @@ public final class ApiMessageTypeGenerator implements TypeClassGenerator {
         String requestSchema() {
             if (requestSpec == null) {
                 return "null";
+            } else if (!requestSpec.hasValidVersion()) {
+                return "new Schema[0]";
             } else {
                 return String.format("%sData.SCHEMAS", requestSpec.name());
             }
@@ -70,6 +72,8 @@ public final class ApiMessageTypeGenerator implements TypeClassGenerator {
         String responseSchema() {
             if (responseSpec == null) {
                 return "null";
+            } else if (!requestSpec.hasValidVersion()) {
+                return "new Schema[0]";
             } else {
                 return String.format("%sData.SCHEMAS", responseSpec.name());
             }
@@ -301,12 +305,15 @@ public final class ApiMessageTypeGenerator implements TypeClassGenerator {
         buffer.printf("switch (apiKey) {%n");
         buffer.incrementIndent();
         for (Map.Entry<Short, ApiData> entry : apis.entrySet()) {
-            buffer.printf("case %d:%n", entry.getKey());
-            buffer.incrementIndent();
-            buffer.printf("return new %s%sData();%n",
-                entry.getValue().name(),
-                MessageGenerator.capitalizeFirst(type));
-            buffer.decrementIndent();
+            MessageSpec spec = messageSpec(type, entry.getKey(), entry.getValue());
+            if (spec.hasValidVersion()) {
+                buffer.printf("case %d:%n", entry.getKey());
+                buffer.incrementIndent();
+                buffer.printf("return new %s%sData();%n",
+                    entry.getValue().name(),
+                    MessageGenerator.capitalizeFirst(type));
+                buffer.decrementIndent();
+            }
         }
         buffer.printf("default:%n");
         buffer.incrementIndent();
@@ -346,6 +353,11 @@ public final class ApiMessageTypeGenerator implements TypeClassGenerator {
             short apiKey = entry.getKey();
             ApiData apiData = entry.getValue();
             String name = apiData.name();
+
+            MessageSpec spec = messageSpec(type, apiKey, entry.getValue());
+            if (!spec.hasValidVersion())
+                continue;
+
             buffer.printf("case %d: // %s%n", apiKey, MessageGenerator.capitalizeFirst(name));
             buffer.incrementIndent();
             if (type.equals("response") && apiKey == 18) {
@@ -355,28 +367,6 @@ public final class ApiMessageTypeGenerator implements TypeClassGenerator {
                 buffer.decrementIndent();
                 continue;
             }
-            if (type.equals("request") && apiKey == 7) {
-                buffer.printf("// Version 0 of ControlledShutdownRequest has a non-standard request header%n");
-                buffer.printf("// which does not include clientId.  Version 1 of ControlledShutdownRequest%n");
-                buffer.printf("// and later use the standard request header.%n");
-                buffer.printf("if (_version == 0) {%n");
-                buffer.incrementIndent();
-                buffer.printf("return (short) 0;%n");
-                buffer.decrementIndent();
-                buffer.printf("}%n");
-            }
-            ApiData data = entry.getValue();
-            MessageSpec spec = null;
-            if (type.equals("request")) {
-                spec = data.requestSpec;
-            } else if (type.equals("response")) {
-                spec = data.responseSpec;
-            } else {
-                throw new RuntimeException("Invalid type " + type + " for generateHeaderVersion");
-            }
-            if (spec == null) {
-                throw new RuntimeException("failed to find " + type + " for API key " + apiKey);
-            }
             VersionConditional.forVersions(spec.flexibleVersions(),
                 spec.validVersions()).
                 ifMember(__ -> {
@@ -407,6 +397,20 @@ public final class ApiMessageTypeGenerator implements TypeClassGenerator {
         buffer.printf("}%n");
     }
 
+    private static MessageSpec messageSpec(String type, short apiKey, ApiData apiData) {
+        MessageSpec spec;
+        if (type.equals("request")) {
+            spec = apiData.requestSpec;
+        } else if (type.equals("response")) {
+            spec = apiData.responseSpec;
+        } else {
+            throw new RuntimeException("Invalid type " + type);
+        }
+        if (spec == null)
+            throw new RuntimeException("failed to find " + type + " for API key " + apiKey);
+        return spec;
+    }
+
     private void generateListenerTypesEnum() {
         buffer.printf("public enum ListenerType {%n");
         buffer.incrementIndent();
