@@ -40,6 +40,7 @@ import org.apache.kafka.common.message.OffsetDeleteRequestData;
 import org.apache.kafka.common.message.OffsetDeleteResponseData;
 import org.apache.kafka.common.message.OffsetFetchRequestData;
 import org.apache.kafka.common.message.OffsetFetchResponseData;
+import org.apache.kafka.common.message.ShareGroupDescribeResponseData;
 import org.apache.kafka.common.message.ShareGroupDescribeResponseData.DescribedGroup;
 import org.apache.kafka.common.message.ShareGroupHeartbeatRequestData;
 import org.apache.kafka.common.message.ShareGroupHeartbeatResponseData;
@@ -53,25 +54,27 @@ import org.apache.kafka.common.requests.ConsumerGroupDescribeRequest;
 import org.apache.kafka.common.requests.DeleteGroupsRequest;
 import org.apache.kafka.common.requests.DescribeGroupsRequest;
 import org.apache.kafka.common.requests.OffsetCommitRequest;
-import org.apache.kafka.common.requests.OffsetFetchResponse;
 import org.apache.kafka.common.requests.RequestContext;
+import org.apache.kafka.common.requests.ShareGroupDescribeRequest;
 import org.apache.kafka.common.requests.TransactionResult;
 import org.apache.kafka.common.requests.TxnOffsetCommitRequest;
 import org.apache.kafka.common.utils.BufferSupplier;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.common.utils.Utils;
-import org.apache.kafka.coordinator.group.metrics.CoordinatorRuntimeMetrics;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorEventProcessor;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorLoader;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorRecord;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorResult;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorRuntime;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorRuntimeMetrics;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorShardBuilderSupplier;
+import org.apache.kafka.coordinator.common.runtime.MultiThreadedEventProcessor;
+import org.apache.kafka.coordinator.common.runtime.PartitionWriter;
 import org.apache.kafka.coordinator.group.metrics.GroupCoordinatorMetrics;
-import org.apache.kafka.coordinator.group.runtime.CoordinatorEventProcessor;
-import org.apache.kafka.coordinator.group.runtime.CoordinatorLoader;
-import org.apache.kafka.coordinator.group.runtime.CoordinatorResult;
-import org.apache.kafka.coordinator.group.runtime.CoordinatorRuntime;
-import org.apache.kafka.coordinator.group.runtime.CoordinatorShardBuilderSupplier;
-import org.apache.kafka.coordinator.group.runtime.MultiThreadedEventProcessor;
-import org.apache.kafka.coordinator.group.runtime.PartitionWriter;
 import org.apache.kafka.image.MetadataDelta;
 import org.apache.kafka.image.MetadataImage;
+import org.apache.kafka.server.authorizer.Authorizer;
 import org.apache.kafka.server.record.BrokerCompressionType;
 import org.apache.kafka.server.util.FutureUtils;
 import org.apache.kafka.server.util.timer.Timer;
@@ -84,16 +87,19 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.OptionalInt;
 import java.util.Properties;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CompletionException;
 import java.util.concurrent.ExecutionException;
+import java.util.concurrent.Executors;
 import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.function.BiFunction;
 import java.util.function.IntSupplier;
 import java.util.stream.Collectors;
 
+import static org.apache.kafka.coordinator.common.runtime.CoordinatorOperationExceptionHelper.handleOperationException;
+
 /**
  * The group coordinator service.
  */
@@ -108,6 +114,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
         private Timer timer;
         private CoordinatorRuntimeMetrics coordinatorRuntimeMetrics;
         private GroupCoordinatorMetrics groupCoordinatorMetrics;
+        private GroupConfigManager groupConfigManager;
+        private Optional<Authorizer> authorizer;
 
         public Builder(
             int nodeId,
@@ -147,27 +155,33 @@ public class GroupCoordinatorService implements GroupCoordinator {
             return this;
         }
 
+        public Builder withGroupConfigManager(GroupConfigManager groupConfigManager) {
+            this.groupConfigManager = groupConfigManager;
+            return this;
+        }
+
+        public Builder withAuthorizer(Optional<Authorizer> authorizer) {
+            this.authorizer = authorizer;
+            return this;
+        }
+
         public GroupCoordinatorService build() {
-            if (config == null)
-                throw new IllegalArgumentException("Config must be set.");
-            if (writer == null)
-                throw new IllegalArgumentException("Writer must be set.");
-            if (loader == null)
-                throw new IllegalArgumentException("Loader must be set.");
-            if (time == null)
-                throw new IllegalArgumentException("Time must be set.");
-            if (timer == null)
-                throw new IllegalArgumentException("Timer must be set.");
-            if (coordinatorRuntimeMetrics == null)
-                throw new IllegalArgumentException("CoordinatorRuntimeMetrics must be set.");
-            if (groupCoordinatorMetrics == null)
-                throw new IllegalArgumentException("GroupCoordinatorMetrics must be set.");
+            requireNonNull(config, new IllegalArgumentException("Config must be set."));
+            requireNonNull(writer, new IllegalArgumentException("Writer must be set."));
+            requireNonNull(loader, new IllegalArgumentException("Loader must be set."));
+            requireNonNull(time, new IllegalArgumentException("Time must be set."));
+            requireNonNull(timer, new IllegalArgumentException("Timer must be set."));
+            requireNonNull(coordinatorRuntimeMetrics, new IllegalArgumentException("CoordinatorRuntimeMetrics must be set."));
+            requireNonNull(groupCoordinatorMetrics, new IllegalArgumentException("GroupCoordinatorMetrics must be set."));
+            requireNonNull(groupConfigManager, new IllegalArgumentException("GroupConfigManager must be set."));
+            requireNonNull(authorizer, new IllegalArgumentException("Authorizer must be set."));
 
             String logPrefix = String.format("GroupCoordinator id=%d", nodeId);
             LogContext logContext = new LogContext(String.format("[%s] ", logPrefix));
 
             CoordinatorShardBuilderSupplier<GroupCoordinatorShard, CoordinatorRecord> supplier = () ->
-                new GroupCoordinatorShard.Builder(config);
+                new GroupCoordinatorShard.Builder(config, groupConfigManager)
+                    .withAuthorizer(authorizer);
 
             CoordinatorEventProcessor processor = new MultiThreadedEventProcessor(
                 logContext,
@@ -190,16 +204,18 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     .withDefaultWriteTimeOut(Duration.ofMillis(config.offsetCommitTimeoutMs()))
                     .withCoordinatorRuntimeMetrics(coordinatorRuntimeMetrics)
                     .withCoordinatorMetrics(groupCoordinatorMetrics)
-                    .withSerializer(new CoordinatorRecordSerde())
+                    .withSerializer(new GroupCoordinatorRecordSerde())
                     .withCompression(Compression.of(config.offsetTopicCompressionType()).build())
                     .withAppendLingerMs(config.appendLingerMs())
+                    .withExecutorService(Executors.newSingleThreadExecutor())
                     .build();
 
             return new GroupCoordinatorService(
                 logContext,
                 config,
                 runtime,
-                groupCoordinatorMetrics
+                groupCoordinatorMetrics,
+                groupConfigManager
             );
         }
     }
@@ -224,6 +240,11 @@ public class GroupCoordinatorService implements GroupCoordinator {
      */
     private final GroupCoordinatorMetrics groupCoordinatorMetrics;
 
+    /**
+     * The group config manager.
+     */
+    private final GroupConfigManager groupConfigManager;
+
     /**
      * Boolean indicating whether the coordinator is active or not.
      */
@@ -241,17 +262,20 @@ public class GroupCoordinatorService implements GroupCoordinator {
      * @param config                    The group coordinator config.
      * @param runtime                   The runtime.
      * @param groupCoordinatorMetrics   The group coordinator metrics.
+     * @param groupConfigManager        The group config manager.
      */
     GroupCoordinatorService(
         LogContext logContext,
         GroupCoordinatorConfig config,
         CoordinatorRuntime<GroupCoordinatorShard, CoordinatorRecord> runtime,
-        GroupCoordinatorMetrics groupCoordinatorMetrics
+        GroupCoordinatorMetrics groupCoordinatorMetrics,
+        GroupConfigManager groupConfigManager
     ) {
         this.log = logContext.logger(GroupCoordinatorService.class);
         this.config = config;
         this.runtime = runtime;
         this.groupCoordinatorMetrics = groupCoordinatorMetrics;
+        this.groupConfigManager = groupConfigManager;
     }
 
     /**
@@ -272,6 +296,14 @@ public class GroupCoordinatorService implements GroupCoordinator {
         return new TopicPartition(Topic.GROUP_METADATA_TOPIC_NAME, partitionFor(groupId));
     }
 
+    /**
+     * See {@link GroupCoordinator#isNewGroupCoordinator()}
+     */
+    @Override
+    public boolean isNewGroupCoordinator() {
+        return true;
+    }
+
     /**
      * See {@link GroupCoordinator#partitionFor(String)}
      */
@@ -308,7 +340,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
             exception,
             (error, message) -> new ConsumerGroupHeartbeatResponseData()
                 .setErrorCode(error.code())
-                .setErrorMessage(message)
+                .setErrorMessage(message),
+            log
         ));
     }
 
@@ -337,7 +370,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
             exception,
             (error, message) -> new ShareGroupHeartbeatResponseData()
                 .setErrorCode(error.code())
-                .setErrorMessage(message)
+                .setErrorMessage(message),
+            log
         ));
     }
 
@@ -385,7 +419,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     "classic-group-join",
                     request,
                     exception,
-                    (error, __) -> new JoinGroupResponseData().setErrorCode(error.code())
+                    (error, __) -> new JoinGroupResponseData().setErrorCode(error.code()),
+                    log
                 ));
             }
             return null;
@@ -428,7 +463,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     "classic-group-sync",
                     request,
                     exception,
-                    (error, __) -> new SyncGroupResponseData().setErrorCode(error.code())
+                    (error, __) -> new SyncGroupResponseData().setErrorCode(error.code()),
+                    log
                 ));
             }
             return null;
@@ -475,7 +511,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     return new HeartbeatResponseData()
                         .setErrorCode(error.code());
                 }
-            }
+            },
+            log
         ));
     }
 
@@ -523,7 +560,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     return new LeaveGroupResponseData()
                         .setErrorCode(error.code());
                 }
-            }
+            },
+            log
         ));
     }
 
@@ -567,7 +605,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                 "list-groups",
                 request,
                 exception,
-                (error, __) -> new ListGroupsResponseData().setErrorCode(error.code())
+                (error, __) -> new ListGroupsResponseData().setErrorCode(error.code()),
+                log
             ));
     }
 
@@ -613,7 +652,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     "consumer-group-describe",
                     groupList,
                     exception,
-                    (error, __) -> ConsumerGroupDescribeRequest.getErrorDescribedGroupList(groupList, error)
+                    (error, __) -> ConsumerGroupDescribeRequest.getErrorDescribedGroupList(groupList, error),
+                    log
                 ));
 
             futures.add(future);
@@ -629,8 +669,48 @@ public class GroupCoordinatorService implements GroupCoordinator {
     public CompletableFuture<List<DescribedGroup>> shareGroupDescribe(
         RequestContext context,
         List<String> groupIds) {
-        // TODO: Implement this method as part of KIP-932.
-        throw new UnsupportedOperationException();
+        if (!isActive.get()) {
+            return CompletableFuture.completedFuture(ShareGroupDescribeRequest.getErrorDescribedGroupList(
+                groupIds,
+                Errors.COORDINATOR_NOT_AVAILABLE
+            ));
+        }
+
+        final List<CompletableFuture<List<ShareGroupDescribeResponseData.DescribedGroup>>> futures =
+            new ArrayList<>(groupIds.size());
+        final Map<TopicPartition, List<String>> groupsByTopicPartition = new HashMap<>();
+        groupIds.forEach(groupId -> {
+            if (isGroupIdNotEmpty(groupId)) {
+                groupsByTopicPartition
+                    .computeIfAbsent(topicPartitionFor(groupId), __ -> new ArrayList<>())
+                    .add(groupId);
+            } else {
+                futures.add(CompletableFuture.completedFuture(Collections.singletonList(
+                    new ShareGroupDescribeResponseData.DescribedGroup()
+                        .setGroupId(null)
+                        .setErrorCode(Errors.INVALID_GROUP_ID.code())
+                )));
+            }
+        });
+
+        groupsByTopicPartition.forEach((topicPartition, groupList) -> {
+            CompletableFuture<List<ShareGroupDescribeResponseData.DescribedGroup>> future =
+                runtime.scheduleReadOperation(
+                    "share-group-describe",
+                    topicPartition,
+                    (coordinator, lastCommittedOffset) -> coordinator.shareGroupDescribe(groupIds, lastCommittedOffset)
+                ).exceptionally(exception -> handleOperationException(
+                    "share-group-describe",
+                    groupList,
+                    exception,
+                    (error, __) -> ShareGroupDescribeRequest.getErrorDescribedGroupList(groupList, error),
+                    log
+                ));
+
+            futures.add(future);
+        });
+
+        return FutureUtils.combineFutures(futures, ArrayList::new, List::addAll);
     }
 
     /**
@@ -677,7 +757,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     "describe-groups",
                     groupList,
                     exception,
-                    (error, __) -> DescribeGroupsRequest.getErrorDescribedGroupList(groupList, error)
+                    (error, __) -> DescribeGroupsRequest.getErrorDescribedGroupList(groupList, error),
+                    log
                 ));
 
             futures.add(future);
@@ -732,7 +813,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     "delete-groups",
                     groupList,
                     exception,
-                    (error, __) -> DeleteGroupsRequest.getErrorResultCollection(groupList, error)
+                    (error, __) -> DeleteGroupsRequest.getErrorResultCollection(groupList, error),
+                    log
                 ));
 
             futures.add(future);
@@ -754,20 +836,18 @@ public class GroupCoordinatorService implements GroupCoordinator {
         boolean requireStable
     ) {
         if (!isActive.get()) {
-            return CompletableFuture.completedFuture(OffsetFetchResponse.groupError(
-                request,
-                Errors.COORDINATOR_NOT_AVAILABLE,
-                context.requestVersion()
-            ));
+            return CompletableFuture.completedFuture(new OffsetFetchResponseData.OffsetFetchResponseGroup()
+                .setGroupId(request.groupId())
+                .setErrorCode(Errors.COORDINATOR_NOT_AVAILABLE.code())
+            );
         }
 
         // For backwards compatibility, we support fetch commits for the empty group id.
         if (request.groupId() == null) {
-            return CompletableFuture.completedFuture(OffsetFetchResponse.groupError(
-                request,
-                Errors.INVALID_GROUP_ID,
-                context.requestVersion()
-            ));
+            return CompletableFuture.completedFuture(new OffsetFetchResponseData.OffsetFetchResponseGroup()
+                .setGroupId(request.groupId())
+                .setErrorCode(Errors.INVALID_GROUP_ID.code())
+            );
         }
 
         // The require stable flag when set tells the broker to hold on returning unstable
@@ -787,7 +867,11 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     Collections.emptyList(),
                     coordinator.fetchOffsets(request, Long.MAX_VALUE)
                 )
-            );
+            ).exceptionally(exception -> handleOffsetFetchException(
+                "fetch-offsets",
+                request,
+                exception
+            ));
         } else {
             return runtime.scheduleReadOperation(
                 "fetch-offsets",
@@ -807,20 +891,18 @@ public class GroupCoordinatorService implements GroupCoordinator {
         boolean requireStable
     ) {
         if (!isActive.get()) {
-            return CompletableFuture.completedFuture(OffsetFetchResponse.groupError(
-                request,
-                Errors.COORDINATOR_NOT_AVAILABLE,
-                context.requestVersion()
-            ));
+            return CompletableFuture.completedFuture(new OffsetFetchResponseData.OffsetFetchResponseGroup()
+                .setGroupId(request.groupId())
+                .setErrorCode(Errors.COORDINATOR_NOT_AVAILABLE.code())
+            );
         }
 
         // For backwards compatibility, we support fetch commits for the empty group id.
         if (request.groupId() == null) {
-            return CompletableFuture.completedFuture(OffsetFetchResponse.groupError(
-                request,
-                Errors.INVALID_GROUP_ID,
-                context.requestVersion()
-            ));
+            return CompletableFuture.completedFuture(new OffsetFetchResponseData.OffsetFetchResponseGroup()
+                .setGroupId(request.groupId())
+                .setErrorCode(Errors.INVALID_GROUP_ID.code())
+            );
         }
 
         // The require stable flag when set tells the broker to hold on returning unstable
@@ -840,7 +922,11 @@ public class GroupCoordinatorService implements GroupCoordinator {
                     Collections.emptyList(),
                     coordinator.fetchAllOffsets(request, Long.MAX_VALUE)
                 )
-            );
+            ).exceptionally(exception -> handleOffsetFetchException(
+                "fetch-all-offsets",
+                request,
+                exception
+            ));
         } else {
             return runtime.scheduleReadOperation(
                 "fetch-all-offsets",
@@ -883,7 +969,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
             "commit-offset",
             request,
             exception,
-            (error, __) -> OffsetCommitRequest.getErrorResponse(request, error)
+            (error, __) -> OffsetCommitRequest.getErrorResponse(request, error),
+            log
         ));
     }
 
@@ -923,7 +1010,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
             "txn-commit-offset",
             request,
             exception,
-            (error, __) -> TxnOffsetCommitRequest.getErrorResponse(request, error)
+            (error, __) -> TxnOffsetCommitRequest.getErrorResponse(request, error),
+            log
         ));
     }
 
@@ -957,7 +1045,8 @@ public class GroupCoordinatorService implements GroupCoordinator {
             "delete-offsets",
             request,
             exception,
-            (error, __) -> new OffsetDeleteResponseData().setErrorCode(error.code())
+            (error, __) -> new OffsetDeleteResponseData().setErrorCode(error.code()),
+            log
         ));
     }
 
@@ -1088,6 +1177,22 @@ public class GroupCoordinatorService implements GroupCoordinator {
         return properties;
     }
 
+    /**
+     * See {@link GroupCoordinator#groupConfig(String)}.
+     */
+    @Override
+    public Optional<GroupConfig> groupConfig(String groupId) {
+        return groupConfigManager.groupConfig(groupId);
+    }
+
+    /**
+     * See {@link GroupCoordinator#updateGroupConfig(String, Properties)}.
+     */
+    @Override
+    public void updateGroupConfig(String groupId, Properties newGroupConfig) {
+        groupConfigManager.updateGroupConfig(groupId, newGroupConfig);
+    }
+
     /**
      * See {@link GroupCoordinator#startup(IntSupplier)}.
      */
@@ -1120,6 +1225,7 @@ public class GroupCoordinatorService implements GroupCoordinator {
         isActive.set(false);
         Utils.closeQuietly(runtime, "coordinator runtime");
         Utils.closeQuietly(groupCoordinatorMetrics, "group coordinator metrics");
+        Utils.closeQuietly(groupConfigManager, "group config manager");
         log.info("Shutdown complete.");
     }
 
@@ -1128,57 +1234,54 @@ public class GroupCoordinatorService implements GroupCoordinator {
     }
 
     /**
-     * This is the handler commonly used by all the operations that requires to convert errors to
-     * coordinator errors. The handler also handles and log unexpected errors.
+     * This is the handler used by offset fetch operations to convert errors to coordinator errors.
+     * The handler also handles and logs unexpected errors.
      *
      * @param operationName     The name of the operation.
-     * @param operationInput    The operation's input for logging purposes.
+     * @param request           The OffsetFetchRequestGroup request.
      * @param exception         The exception to handle.
-     * @param handler           A function which takes an Errors and a String and builds the expected
-     *                          output. The String can be null. Note that the function could further
-     *                          transform the error depending on the context.
-     * @return The output built by the handler.
-     * @param <IN> The type of the operation input. It must be a toString'able object.
-     * @param <OUT> The type of the value returned by handler.
+     * @return The OffsetFetchRequestGroup response.
      */
-    private <IN, OUT> OUT handleOperationException(
+    private OffsetFetchResponseData.OffsetFetchResponseGroup handleOffsetFetchException(
         String operationName,
-        IN operationInput,
-        Throwable exception,
-        BiFunction<Errors, String, OUT> handler
+        OffsetFetchRequestData.OffsetFetchRequestGroup request,
+        Throwable exception
     ) {
         ApiError apiError = ApiError.fromThrowable(exception);
 
         switch (apiError.error()) {
-            case UNKNOWN_SERVER_ERROR:
-                log.error("Operation {} with {} hit an unexpected exception: {}.",
-                    operationName, operationInput, exception.getMessage(), exception);
-                return handler.apply(Errors.UNKNOWN_SERVER_ERROR, null);
-
-            case NETWORK_EXCEPTION:
-                // When committing offsets transactionally, we now verify the transaction with the
-                // transaction coordinator. Verification can fail with `NETWORK_EXCEPTION`, a
-                // retriable error which older clients may not expect and retry correctly. We
-                // translate the error to `COORDINATOR_LOAD_IN_PROGRESS` because it causes clients
-                // to retry the request without an unnecessary coordinator lookup.
-                return handler.apply(Errors.COORDINATOR_LOAD_IN_PROGRESS, null);
-
             case UNKNOWN_TOPIC_OR_PARTITION:
             case NOT_ENOUGH_REPLICAS:
             case REQUEST_TIMED_OUT:
-                return handler.apply(Errors.COORDINATOR_NOT_AVAILABLE, null);
-
-            case NOT_LEADER_OR_FOLLOWER:
-            case KAFKA_STORAGE_ERROR:
-                return handler.apply(Errors.NOT_COORDINATOR, null);
-
-            case MESSAGE_TOO_LARGE:
-            case RECORD_LIST_TOO_LARGE:
-            case INVALID_FETCH_SIZE:
-                return handler.apply(Errors.UNKNOWN_SERVER_ERROR, null);
+                // Remap REQUEST_TIMED_OUT to NOT_COORDINATOR, since consumers on versions prior
+                // to 3.9 do not expect the error and won't retry the request. NOT_COORDINATOR
+                // additionally triggers coordinator re-lookup, which is necessary if the client is
+                // talking to a zombie coordinator.
+                //
+                // While handleOperationException does remap UNKNOWN_TOPIC_OR_PARTITION,
+                // NOT_ENOUGH_REPLICAS and REQUEST_TIMED_OUT to COORDINATOR_NOT_AVAILABLE,
+                // COORDINATOR_NOT_AVAILABLE is also not handled by consumers on versions prior to
+                // 3.9.
+                return new OffsetFetchResponseData.OffsetFetchResponseGroup()
+                    .setGroupId(request.groupId())
+                    .setErrorCode(Errors.NOT_COORDINATOR.code());
 
             default:
-                return handler.apply(apiError.error(), apiError.message());
+                return handleOperationException(
+                    operationName,
+                    request,
+                    exception,
+                    (error, __) -> new OffsetFetchResponseData.OffsetFetchResponseGroup()
+                        .setGroupId(request.groupId())
+                        .setErrorCode(error.code()),
+                    log
+                );
+        }
+    }
+
+    private static void requireNonNull(Object obj, RuntimeException throwable) {
+        if (obj == null) {
+            throw throwable;
         }
     }
 }
