@@ -49,17 +49,6 @@ import java.util.Set;
  *      Balance > Stickiness.
  */
 public class UniformHomogeneousAssignmentBuilder {
-    private static final Class<?> UNMODIFIABLE_MAP_CLASS = Collections.unmodifiableMap(new HashMap<>()).getClass();
-    private static final Class<?> EMPTY_MAP_CLASS = Collections.emptyMap().getClass();
-
-    /**
-     * @return True if the provided map is an UnmodifiableMap or EmptyMap. Those classes are not
-     * public hence we cannot use the `instanceof` operator.
-     */
-    private static boolean isImmutableMap(Map<?, ?> map) {
-        return UNMODIFIABLE_MAP_CLASS.isInstance(map) || EMPTY_MAP_CLASS.isInstance(map);
-    }
-
     /**
      * The assignment specification which includes member metadata.
      */
@@ -171,7 +160,7 @@ public class UniformHomogeneousAssignmentBuilder {
 
             // The assignor expects to receive the assignment as an immutable map. It leverages
             // this knowledge in order to avoid having to copy all assignments.
-            if (!isImmutableMap(oldAssignment)) {
+            if (!AssignorHelpers.isImmutableMap(oldAssignment)) {
                 throw new IllegalStateException("The assignor expect an immutable map.");
             }
 
@@ -196,7 +185,7 @@ public class UniformHomogeneousAssignmentBuilder {
                                 if (newAssignment == null) {
                                     // If the new assignment is null, we create a deep copy of the
                                     // original assignment so that we can alter it.
-                                    newAssignment = deepCopy(oldAssignment);
+                                    newAssignment = AssignorHelpers.deepCopyAssignment(oldAssignment);
                                 }
                                 // Remove the partition from the new assignment.
                                 Set<Integer> parts = newAssignment.get(topicId);
@@ -213,7 +202,7 @@ public class UniformHomogeneousAssignmentBuilder {
                     if (newAssignment == null) {
                         // If the new assignment is null, we create a deep copy of the
                         // original assignment so that we can alter it.
-                        newAssignment = deepCopy(oldAssignment);
+                        newAssignment = AssignorHelpers.deepCopyAssignment(oldAssignment);
                     }
                     // Remove the entire topic.
                     newAssignment.remove(topicId);
@@ -243,10 +232,10 @@ public class UniformHomogeneousAssignmentBuilder {
             int remainingQuota = unfilledMember.remainingQuota;
 
             Map<Uuid, Set<Integer>> newAssignment = targetAssignment.get(memberId).partitions();
-            if (isImmutableMap(newAssignment)) {
+            if (AssignorHelpers.isImmutableMap(newAssignment)) {
                 // If the new assignment is immutable, we must create a deep copy of it
                 // before altering it.
-                newAssignment = deepCopy(newAssignment);
+                newAssignment = AssignorHelpers.deepCopyAssignment(newAssignment);
                 targetAssignment.put(memberId, new MemberAssignmentImpl(newAssignment));
             }
 
@@ -264,14 +253,6 @@ public class UniformHomogeneousAssignmentBuilder {
         }
     }
 
-    private static Map<Uuid, Set<Integer>> deepCopy(Map<Uuid, Set<Integer>> map) {
-        Map<Uuid, Set<Integer>> copy = new HashMap<>(map.size());
-        for (Map.Entry<Uuid, Set<Integer>> entry : map.entrySet()) {
-            copy.put(entry.getKey(), new HashSet<>(entry.getValue()));
-        }
-        return copy;
-    }
-
     private static class MemberWithRemainingQuota {
         final String memberId;
         final int remainingQuota;
