@@ -33,15 +33,14 @@ import org.apache.kafka.common.protocol.types.SchemaException;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.common.utils.Utils;
-import org.apache.kafka.coordinator.group.CoordinatorRecord;
-import org.apache.kafka.coordinator.group.CoordinatorRecordHelpers;
+import org.apache.kafka.coordinator.common.runtime.CoordinatorRecord;
 import org.apache.kafka.coordinator.group.Group;
+import org.apache.kafka.coordinator.group.GroupCoordinatorRecordHelpers;
 import org.apache.kafka.coordinator.group.OffsetExpirationCondition;
 import org.apache.kafka.coordinator.group.OffsetExpirationConditionImpl;
-import org.apache.kafka.coordinator.group.metrics.GroupCoordinatorMetricsShard;
 import org.apache.kafka.coordinator.group.modern.consumer.ConsumerGroup;
+import org.apache.kafka.coordinator.group.modern.consumer.ConsumerGroupMember;
 import org.apache.kafka.image.MetadataImage;
-import org.apache.kafka.server.common.MetadataVersion;
 
 import org.slf4j.Logger;
 
@@ -182,24 +181,17 @@ public class ClassicGroup implements Group {
      */
     private boolean newMemberAdded = false;
 
-    /**
-     * Coordinator metrics.
-     */
-    private final GroupCoordinatorMetricsShard metrics;
-
     public ClassicGroup(
         LogContext logContext,
         String groupId,
         ClassicGroupState initialState,
-        Time time,
-        GroupCoordinatorMetricsShard metrics
+        Time time
     ) {
         this(
             logContext,
             groupId,
             initialState,
             time,
-            metrics,
             0,
             Optional.empty(),
             Optional.empty(),
@@ -213,7 +205,6 @@ public class ClassicGroup implements Group {
         String groupId,
         ClassicGroupState initialState,
         Time time,
-        GroupCoordinatorMetricsShard metrics,
         int generationId,
         Optional<String> protocolType,
         Optional<String> protocolName,
@@ -226,7 +217,6 @@ public class ClassicGroup implements Group {
         this.state = Objects.requireNonNull(initialState);
         this.previousState = DEAD;
         this.time = Objects.requireNonNull(time);
-        this.metrics = Objects.requireNonNull(metrics);
         this.generationId = generationId;
         this.protocolType = protocolType;
         this.protocolName = protocolName;
@@ -292,6 +282,13 @@ public class ClassicGroup implements Group {
         return this.protocolType;
     }
 
+    /**
+     * @return True if the group is a simple group.
+     */
+    public boolean isSimpleGroup() {
+        return protocolType.isEmpty() && isEmpty() && pendingJoinMembers.isEmpty();
+    }
+
     /**
      * @return the current group state.
      */
@@ -450,7 +447,7 @@ public class ClassicGroup implements Group {
             throw new IllegalStateException("None of the member's protocols can be supported.");
         }
 
-        if (!leaderId.isPresent()) {
+        if (leaderId.isEmpty()) {
             leaderId = Optional.of(member.memberId());
         }
 
@@ -925,7 +922,7 @@ public class ClassicGroup implements Group {
      */
     @Override
     public void createGroupTombstoneRecords(List<CoordinatorRecord> records) {
-        records.add(CoordinatorRecordHelpers.newGroupMetadataTombstoneRecord(groupId()));
+        records.add(GroupCoordinatorRecordHelpers.newGroupMetadataTombstoneRecord(groupId()));
     }
 
     @Override
@@ -973,23 +970,6 @@ public class ClassicGroup implements Group {
         return statesFilter.contains(state.toLowerCaseString());
     }
 
-    /**
-     * Verify the member id is up to date for static members. Return true if both conditions met:
-     *   1. given member is a known static member to group
-     *   2. group stored member id doesn't match with given member id
-     *
-     * @param groupInstanceId  the group instance id.
-     * @param memberId         the member id.
-     * @return whether the static member is fenced based on the condition above.
-     */
-    public boolean isStaticMemberFenced(
-        String groupInstanceId,
-        String memberId
-    ) {
-        String existingMemberId = staticMemberId(groupInstanceId);
-        return existingMemberId != null && !existingMemberId.equals(memberId);
-    }
-
     /**
      * @return whether the group can rebalance.
      */
@@ -1006,7 +986,6 @@ public class ClassicGroup implements Group {
         previousState = state;
         state = groupState;
         currentStateTimestamp = Optional.of(time.milliseconds());
-        metrics.onClassicGroupStateTransition(previousState, state);
     }
 
     /**
@@ -1163,7 +1142,7 @@ public class ClassicGroup implements Group {
      * @return the subscribed topics or Empty based on the condition above.
      */
     public Optional<Set<String>> computeSubscribedTopics() {
-        if (!protocolType.isPresent()) {
+        if (protocolType.isEmpty()) {
             return Optional.empty();
         }
         String type = protocolType.get();
@@ -1356,11 +1335,11 @@ public class ClassicGroup implements Group {
 
     /**
      * Convert the given ConsumerGroup to a corresponding ClassicGroup.
-     * The member with leavingMemberId will not be converted to the new ClassicGroup as it's the last
-     * member using new consumer protocol that left and triggered the downgrade.
      *
      * @param consumerGroup                 The converted ConsumerGroup.
-     * @param leavingMemberId               The member that will not be converted in the ClassicGroup.
+     * @param leavingMembers                The members that will not be converted in the ClassicGroup.
+     * @param joiningMember                 The member that needs to be converted and added to the ClassicGroup.
+     *                                      When not null, must have an instanceId that matches an existing member.
      * @param logContext                    The logContext to create the ClassicGroup.
      * @param time                          The time to create the ClassicGroup.
      * @param metadataImage                 The MetadataImage.
@@ -1368,10 +1347,10 @@ public class ClassicGroup implements Group {
      */
     public static ClassicGroup fromConsumerGroup(
         ConsumerGroup consumerGroup,
-        String leavingMemberId,
+        Set<ConsumerGroupMember> leavingMembers,
+        ConsumerGroupMember joiningMember,
         LogContext logContext,
         Time time,
-        GroupCoordinatorMetricsShard metrics,
         MetadataImage metadataImage
     ) {
         ClassicGroup classicGroup = new ClassicGroup(
@@ -1379,16 +1358,16 @@ public class ClassicGroup implements Group {
             consumerGroup.groupId(),
             ClassicGroupState.STABLE,
             time,
-            metrics,
             consumerGroup.groupEpoch(),
-            Optional.ofNullable(ConsumerProtocol.PROTOCOL_TYPE),
+            Optional.of(ConsumerProtocol.PROTOCOL_TYPE),
             Optional.empty(),
             Optional.empty(),
             Optional.of(time.milliseconds())
         );
 
         consumerGroup.members().forEach((memberId, member) -> {
-            if (!memberId.equals(leavingMemberId)) {
+            if (!leavingMembers.contains(member) &&
+                (joiningMember == null || joiningMember.instanceId() == null || !joiningMember.instanceId().equals(member.instanceId()))) {
                 classicGroup.add(
                     new ClassicGroupMember(
                         memberId,
@@ -1405,15 +1384,42 @@ public class ClassicGroup implements Group {
             }
         });
 
+        if (joiningMember != null) {
+            classicGroup.add(
+                new ClassicGroupMember(
+                    joiningMember.memberId(),
+                    Optional.ofNullable(joiningMember.instanceId()),
+                    joiningMember.clientId(),
+                    joiningMember.clientHost(),
+                    joiningMember.rebalanceTimeoutMs(),
+                    joiningMember.classicProtocolSessionTimeout().get(),
+                    ConsumerProtocol.PROTOCOL_TYPE,
+                    joiningMember.supportedJoinGroupRequestProtocols(),
+                    null
+                )
+            );
+        }
+
         classicGroup.setProtocolName(Optional.of(classicGroup.selectProtocol()));
         classicGroup.setSubscribedTopics(classicGroup.computeSubscribedTopics());
 
         classicGroup.allMembers().forEach(classicGroupMember -> {
             // Set the assignment with serializing the ConsumerGroup's targetAssignment.
             // The serializing version should align with that of the member's JoinGroupRequestProtocol.
+            String memberId = classicGroupMember.memberId();
+            if (joiningMember != null && memberId.equals(joiningMember.memberId())) {
+                // If the downgraded is triggered by the joining static member replacing
+                // the leaving static member, the joining member should take the assignment
+                // of the leaving one.
+                ConsumerGroupMember replacedMember = consumerGroup.staticMember(joiningMember.instanceId());
+                if (replacedMember == null) {
+                    throw new IllegalArgumentException("joiningMember must be a static member when not null.");
+                }
+                memberId = replacedMember.memberId();
+            }
             byte[] assignment = Utils.toArray(ConsumerProtocol.serializeAssignment(
                 toConsumerProtocolAssignment(
-                    consumerGroup.targetAssignment().get(classicGroupMember.memberId()).partitions(),
+                    consumerGroup.targetAssignment().get(memberId).partitions(),
                     metadataImage.topics()
                 ),
                 ConsumerProtocol.deserializeVersion(
@@ -1430,11 +1436,9 @@ public class ClassicGroup implements Group {
     /**
      * Populate the record list with the records needed to create the given classic group.
      *
-     * @param metadataVersion   The MetadataVersion.
      * @param records           The list to which the new records are added.
      */
     public void createClassicGroupRecords(
-        MetadataVersion metadataVersion,
         List<CoordinatorRecord> records
     ) {
         Map<String, byte[]> assignments = new HashMap<>();
@@ -1442,7 +1446,7 @@ public class ClassicGroup implements Group {
             assignments.put(classicGroupMember.memberId(), classicGroupMember.assignment())
         );
 
-        records.add(CoordinatorRecordHelpers.newGroupMetadataRecord(this, assignments, metadataVersion));
+        records.add(GroupCoordinatorRecordHelpers.newGroupMetadataRecord(this, assignments));
     }
 
     /**
@@ -1458,6 +1462,13 @@ public class ClassicGroup implements Group {
         }
     }
 
+    /**
+     * For testing only.
+     */
+    public void setLeaderId(Optional<String> leaderId) {
+        this.leaderId = leaderId;
+    }
+
     @Override
     public String toString() {
         return "ClassicGroupMetadata(" +
