@@ -97,7 +97,6 @@ public class PartitionChangeBuilder {
     private List<Integer> uncleanShutdownReplicas;
     private Election election = Election.ONLINE;
     private LeaderRecoveryState targetLeaderRecoveryState;
-    private boolean zkMigrationEnabled;
     private boolean eligibleLeaderReplicasEnabled;
     private DefaultDirProvider defaultDirProvider;
 
@@ -118,7 +117,6 @@ public class PartitionChangeBuilder {
         this.partitionId = partitionId;
         this.isAcceptableLeader = isAcceptableLeader;
         this.metadataVersion = metadataVersion;
-        this.zkMigrationEnabled = false;
         this.eligibleLeaderReplicasEnabled = false;
         this.minISR = minISR;
 
@@ -179,11 +177,6 @@ public class PartitionChangeBuilder {
         return this;
     }
 
-    public PartitionChangeBuilder setZkMigrationEnabled(boolean zkMigrationEnabled) {
-        this.zkMigrationEnabled = zkMigrationEnabled;
-        return this;
-    }
-
     public PartitionChangeBuilder setEligibleLeaderReplicasEnabled(boolean eligibleLeaderReplicasEnabled) {
         this.eligibleLeaderReplicasEnabled = eligibleLeaderReplicasEnabled;
         return this;
@@ -297,14 +290,13 @@ public class PartitionChangeBuilder {
 
     private boolean canElectLastKnownLeader() {
         if (!eligibleLeaderReplicasEnabled || !useLastKnownLeaderInBalancedRecovery) {
-            log.trace("Try to elect last known leader for " + topicId + "-" + partitionId +
-                " but elrEnabled=" + eligibleLeaderReplicasEnabled + ", useLastKnownLeaderInBalancedRecovery=" +
-                useLastKnownLeaderInBalancedRecovery);
+            log.trace("Try to elect last known leader for {}-{} but elrEnabled={}, useLastKnownLeaderInBalancedRecovery={}",
+                    topicId, partitionId, eligibleLeaderReplicasEnabled, useLastKnownLeaderInBalancedRecovery);
             return false;
         }
         if (!targetElr.isEmpty() || !targetIsr.isEmpty()) {
-            log.trace("Try to elect last known leader for " + topicId + "-" + partitionId +
-                " but ELR/ISR is not empty. ISR=" + targetIsr + ", ELR=" + targetElr);
+            log.trace("Try to elect last known leader for {}-{} but ELR/ISR is not empty. ISR={}, ELR={}",
+                    topicId, partitionId, targetIsr, targetElr);
             return false;
         }
 
@@ -316,14 +308,13 @@ public class PartitionChangeBuilder {
         //    in the field even if useLastKnownLeaderInBalancedRecovery is set to true again. In this case, we can't
         //    refer to the lastKnownElr.
         if (partition.lastKnownElr.length != 1) {
-            log.trace("Try to elect last known leader for " + topicId + "-" + partitionId +
-                " but lastKnownElr does not only have 1 member. lastKnownElr=" +
-                Arrays.toString(partition.lastKnownElr));
+            log.trace("Try to elect last known leader for {}-{} but lastKnownElr does not only have 1 member. lastKnownElr={}",
+                    topicId, partitionId, Arrays.toString(partition.lastKnownElr));
             return false;
         }
         if (isAcceptableLeader.test(partition.lastKnownElr[0])) {
-            log.trace("Try to elect last known leader for " + topicId + "-" + partitionId +
-                " but last known leader is not alive. last known leader=" + partition.lastKnownElr[0]);
+            log.trace("Try to elect last known leader for {}-{} but last known leader is not alive. last known leader={}",
+                    topicId, partitionId, partition.lastKnownElr[0]);
         }
         return true;
     }
@@ -358,8 +349,7 @@ public class PartitionChangeBuilder {
                 // If the election was unclean, we have to forcibly set the ISR to just the
                 // new leader. This can result in data loss!
                 record.setIsr(Collections.singletonList(electionResult.node));
-                if (partition.leaderRecoveryState != LeaderRecoveryState.RECOVERING &&
-                    metadataVersion.isLeaderRecoverySupported()) {
+                if (partition.leaderRecoveryState != LeaderRecoveryState.RECOVERING) {
                     // And mark the leader recovery state as RECOVERING
                     record.setLeaderRecoveryState(LeaderRecoveryState.RECOVERING.value());
                 }
@@ -394,17 +384,11 @@ public class PartitionChangeBuilder {
      * the PartitionChangeRecord.
      */
     void triggerLeaderEpochBumpForIsrShrinkIfNeeded(PartitionChangeRecord record) {
-        if (!(metadataVersion.isLeaderEpochBumpRequiredOnIsrShrink() || zkMigrationEnabled)) {
-            // We only need to bump the leader epoch on an ISR shrink in two cases:
-            //
-            // 1. In older metadata versions before 3.6, there was a bug (KAFKA-15021) in the
-            //    broker replica manager that required that the leader epoch be bumped whenever
-            //    the ISR shrank. (This was never necessary for EXPANSIONS, only SHRINKS.)
-            //
-            // 2. During ZK migration, we bump the leader epoch during all ISR shrinks, in order
-            // to maintain compatibility with migrating brokers that are still in ZK mode.
-            //
-            // If we're not in either case, we can exit here.
+        if (!metadataVersion.isLeaderEpochBumpRequiredOnIsrShrink()) {
+            // We only need to bump the leader epoch on an ISR shrink in older metadata versions
+            // before 3.6, where there was a bug (KAFKA-15021) in the broker replica manager that
+            // required that the leader epoch be bumped whenever the ISR shrank. (This was never
+            // necessary for EXPANSIONS, only SHRINKS.)
             return;
         }
         if (record.leader() != NO_LEADER_CHANGE) {
@@ -431,7 +415,7 @@ public class PartitionChangeBuilder {
 
         Optional<PartitionReassignmentReplicas.CompletedReassignment> completedReassignmentOpt =
             reassignmentReplicas.maybeCompleteReassignment(targetIsr);
-        if (!completedReassignmentOpt.isPresent()) {
+        if (completedReassignmentOpt.isEmpty()) {
             return;
         }
 
@@ -463,7 +447,7 @@ public class PartitionChangeBuilder {
             !targetIsr.equals(Replicas.toList(partition.isr))) {
             // Set the new ISR if it is different from the current ISR and unclean leader election didn't already set it.
             if (targetIsr.isEmpty()) {
-                log.debug("A partition will have an empty ISR. " + this);
+                log.debug("A partition will have an empty ISR. {}", this);
             }
             record.setIsr(targetIsr);
         }
