@@ -30,9 +30,6 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
 
-import static org.apache.kafka.server.common.MetadataVersion.MINIMUM_BOOTSTRAP_VERSION;
-
-
 /**
  * The bootstrap metadata. On startup, if the metadata log is empty, we will populate the log with
  * these records. Alternately, if log is not empty, but the metadata version is not set, we will
@@ -112,11 +109,6 @@ public class BootstrapMetadata {
         String source
     ) {
         this.records = Objects.requireNonNull(records);
-        if (metadataVersion.isLessThan(MINIMUM_BOOTSTRAP_VERSION)) {
-            throw new RuntimeException("Bootstrap metadata.version before " +
-                    MINIMUM_BOOTSTRAP_VERSION + " are not supported. Can't load metadata from " +
-                    source);
-        }
         this.metadataVersion = metadataVersion;
         Objects.requireNonNull(source);
         this.source = source;
@@ -134,19 +126,17 @@ public class BootstrapMetadata {
         return source;
     }
 
-    public BootstrapMetadata copyWithOnlyVersion() {
-        ApiMessageAndVersion versionRecord = null;
+    public short featureLevel(String featureName) {
+        short result = 0;
         for (ApiMessageAndVersion record : records) {
-            if (recordToMetadataVersion(record.message()).isPresent()) {
-                versionRecord = record;
+            if (record.message() instanceof FeatureLevelRecord) {
+                FeatureLevelRecord message = (FeatureLevelRecord) record.message();
+                if (message.name().equals(featureName)) {
+                    result = message.featureLevel();
+                }
             }
         }
-        if (versionRecord == null) {
-            throw new RuntimeException("No FeatureLevelRecord for " + MetadataVersion.FEATURE_NAME +
-                    " was found in " + source);
-        }
-        return new BootstrapMetadata(Collections.singletonList(versionRecord),
-                metadataVersion, source);
+        return result;
     }
 
     public BootstrapMetadata copyWithFeatureRecord(String featureName, short level) {
@@ -168,8 +158,8 @@ public class BootstrapMetadata {
         }
         if (i == records.size()) {
             FeatureLevelRecord newRecord = new FeatureLevelRecord().
-                    setName(featureName).
-                    setFeatureLevel(level);
+                setName(featureName).
+                setFeatureLevel(level);
             newRecords.add(new ApiMessageAndVersion(newRecord, (short) 0));
         }
         return BootstrapMetadata.fromRecords(newRecords, source);
