@@ -48,7 +48,6 @@ import org.apache.kafka.server.authorizer.AuthorizableRequestContext;
 import org.apache.kafka.server.authorizer.AuthorizationResult;
 import org.apache.kafka.server.authorizer.AuthorizerServerInfo;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
-import org.apache.kafka.server.mutable.BoundedListTooLongException;
 import org.apache.kafka.timeline.SnapshotRegistry;
 
 import org.junit.jupiter.api.Test;
@@ -72,7 +71,6 @@ import static org.apache.kafka.common.acl.AclPermissionType.ALLOW;
 import static org.apache.kafka.common.resource.PatternType.LITERAL;
 import static org.apache.kafka.common.resource.PatternType.MATCH;
 import static org.apache.kafka.common.resource.ResourceType.TOPIC;
-import static org.apache.kafka.controller.QuorumController.MAX_RECORDS_PER_USER_OP;
 import static org.apache.kafka.metadata.authorizer.StandardAclWithIdTest.TEST_ACLS;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -233,7 +231,7 @@ public class AclControlManagerTest {
     @Test
     public void testLoadSnapshot() {
         SnapshotRegistry snapshotRegistry = new SnapshotRegistry(new LogContext());
-        snapshotRegistry.getOrCreateSnapshot(0);
+        snapshotRegistry.idempotentCreateSnapshot(0);
         AclControlManager manager = new AclControlManager.Builder().
             setSnapshotRegistry(snapshotRegistry).
             build();
@@ -368,60 +366,4 @@ public class AclControlManagerTest {
         assertEquals(id, ((RemoveAccessControlEntryRecord) deleteAclResultsBothFilters.records().get(0).message()).id());
         assertEquals(2, deleteAclResultsBothFilters.response().size());
     }
-
-    @Test
-    public void testDeleteExceedsMaxRecords() {
-        AclControlManager manager = new AclControlManager.Builder().build();
-        MockClusterMetadataAuthorizer authorizer = new MockClusterMetadataAuthorizer();
-        authorizer.loadSnapshot(manager.idToAcl());
-
-        List<AclBinding> firstCreate = new ArrayList<>();
-        List<AclBinding> secondCreate = new ArrayList<>();
-
-        // create MAX_RECORDS_PER_USER_OP + 2 ACLs
-        for (int i = 0; i < MAX_RECORDS_PER_USER_OP + 2; i++) {
-            StandardAclWithId acl = new StandardAclWithId(Uuid.randomUuid(),
-                new StandardAcl(
-                    ResourceType.TOPIC,
-                    "mytopic_" + i,
-                    PatternType.LITERAL,
-                    "User:alice",
-                    "127.0.0.1",
-                    AclOperation.READ,
-                    AclPermissionType.ALLOW));
-
-            // split acl creations between two create requests
-            if (i % 2 == 0) {
-                firstCreate.add(acl.toBinding());
-            } else {
-                secondCreate.add(acl.toBinding());
-            }
-        }
-        ControllerResult<List<AclCreateResult>> firstCreateResult = manager.createAcls(firstCreate);
-        assertEquals((MAX_RECORDS_PER_USER_OP / 2) + 1, firstCreateResult.response().size());
-        for (AclCreateResult result : firstCreateResult.response()) {
-            assertTrue(!result.exception().isPresent());
-        }
-
-        ControllerResult<List<AclCreateResult>> secondCreateResult = manager.createAcls(secondCreate);
-        assertEquals((MAX_RECORDS_PER_USER_OP / 2) + 1, secondCreateResult.response().size());
-        for (AclCreateResult result : secondCreateResult.response()) {
-            assertTrue(!result.exception().isPresent());
-        }
-
-        RecordTestUtils.replayAll(manager, firstCreateResult.records());
-        RecordTestUtils.replayAll(manager, secondCreateResult.records());
-        assertFalse(manager.idToAcl().isEmpty());
-
-        ArrayList<AclBindingFilter> filters = new ArrayList<>();
-        for (int i = 0; i < MAX_RECORDS_PER_USER_OP + 2; i++) {
-            filters.add(new AclBindingFilter(
-                new ResourcePatternFilter(ResourceType.TOPIC, "mytopic_" + i, PatternType.LITERAL),
-                AccessControlEntryFilter.ANY));
-        }
-
-        Exception exception = assertThrows(InvalidRequestException.class, () -> manager.deleteAcls(filters));
-        assertEquals(BoundedListTooLongException.class, exception.getCause().getClass());
-        assertEquals("Cannot remove more than " + MAX_RECORDS_PER_USER_OP + " acls in a single delete operation.", exception.getCause().getMessage());
-    }
 }
