@@ -29,10 +29,11 @@ import org.apache.kafka.raft.Batch;
 import org.apache.kafka.raft.LeaderAndEpoch;
 import org.apache.kafka.raft.OffsetAndEpoch;
 import org.apache.kafka.raft.RaftClient;
+import org.apache.kafka.raft.errors.BufferAllocationException;
 import org.apache.kafka.raft.errors.NotLeaderException;
-import org.apache.kafka.raft.errors.UnexpectedBaseOffsetException;
 import org.apache.kafka.raft.internals.MemoryBatchReader;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
+import org.apache.kafka.server.common.KRaftVersion;
 import org.apache.kafka.snapshot.MockRawSnapshotReader;
 import org.apache.kafka.snapshot.MockRawSnapshotWriter;
 import org.apache.kafka.snapshot.RawSnapshotReader;
@@ -41,6 +42,7 @@ import org.apache.kafka.snapshot.RecordsSnapshotReader;
 import org.apache.kafka.snapshot.RecordsSnapshotWriter;
 import org.apache.kafka.snapshot.SnapshotReader;
 import org.apache.kafka.snapshot.SnapshotWriter;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -66,9 +68,6 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
-import static org.junit.jupiter.api.Assertions.assertEquals;
-import static org.junit.jupiter.api.Assertions.assertFalse;
-
 /**
  * The LocalLogManager is a test implementation that relies on the contents of memory.
  */
@@ -99,8 +98,7 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         public boolean equals(Object o) {
             if (!(o instanceof LeaderChangeBatch)) return false;
             LeaderChangeBatch other = (LeaderChangeBatch) o;
-            if (!other.newLeader.equals(newLeader)) return false;
-            return true;
+            return other.newLeader.equals(newLeader);
         }
 
         @Override
@@ -162,7 +160,7 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
     }
 
     public static class SharedLogData {
-        private final Logger log = LoggerFactory.getLogger(SharedLogData.class);
+        private static final Logger log = LoggerFactory.getLogger(SharedLogData.class);
 
         /**
          * Maps node IDs to the matching log managers.
@@ -223,21 +221,18 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         synchronized long tryAppend(
             int nodeId,
             int epoch,
-            OptionalLong requiredBaseOffset,
             List<ApiMessageAndVersion> batch
         ) {
             // No easy access to the concept of time. Use the base offset as the append timestamp
             long appendTimestamp = (prevOffset + 1) * 10;
             return tryAppend(nodeId,
                     epoch,
-                    requiredBaseOffset,
                     new LocalRecordBatch(epoch, appendTimestamp, batch));
         }
 
         synchronized long tryAppend(
             int nodeId,
             int epoch,
-            OptionalLong requiredBaseOffset,
             LocalBatch batch
         ) {
             if (!leader.isLeader(nodeId)) {
@@ -255,22 +250,15 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
             }
 
             log.trace("tryAppend(nodeId={}): appending {}.", nodeId, batch);
-            long offset = append(requiredBaseOffset, batch);
+            long offset = append(batch);
             electLeaderIfNeeded();
             return offset;
         }
 
         public synchronized long append(
-            OptionalLong requiredBaseOffset,
             LocalBatch batch
         ) {
             long nextEndOffset = prevOffset + batch.size();
-            requiredBaseOffset.ifPresent(r -> {
-                if (r != prevOffset + 1) {
-                    throw new UnexpectedBaseOffsetException("Wanted base offset " + r +
-                            ", but the next offset was " + nextEndOffset);
-                }
-            });
             log.debug("append(batch={}, nextEndOffset={})", batch, nextEndOffset);
             batches.put(nextEndOffset, batch);
             if (batch instanceof LeaderChangeBatch) {
@@ -296,7 +284,7 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
             }
             LeaderAndEpoch newLeader = new LeaderAndEpoch(OptionalInt.of(nextLeaderNode), leader.epoch() + 1);
             log.info("Elected new leader: {}.", newLeader);
-            append(OptionalLong.empty(), new LeaderChangeBatch(newLeader));
+            append(new LeaderChangeBatch(newLeader));
         }
 
         synchronized LeaderAndEpoch leaderAndEpoch() {
@@ -482,6 +470,11 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
      */
     private final EventQueue eventQueue;
 
+    /**
+     * The latest kraft version used by this local log manager.
+     */
+    private final KRaftVersion lastKRaftVersion;
+
     /**
      * Whether this LocalLogManager has been shut down.
      */
@@ -491,7 +484,7 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
      * An offset that the log manager will not read beyond. This exists only for testing
      * purposes.
      */
-    private long maxReadOffset = Long.MAX_VALUE;
+    private long maxReadOffset;
 
     /**
      * The listener objects attached to this local log manager.
@@ -504,22 +497,22 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
     private volatile LeaderAndEpoch leader = new LeaderAndEpoch(OptionalInt.empty(), 0);
 
     /*
-     * If this variable is true the next non-atomic append with more than 1 record will
-     * result is half the records getting appended with leader election following that.
-     * This is done to emulate having some of the records not getting committed.
+     * If this variable is true the next scheduleAppend will fail
      */
-    private final AtomicBoolean resignAfterNonAtomicCommit = new AtomicBoolean(false);
+    private final AtomicBoolean throwOnNextAppend = new AtomicBoolean(false);
 
     public LocalLogManager(LogContext logContext,
                            int nodeId,
                            SharedLogData shared,
-                           String threadNamePrefix) {
+                           String threadNamePrefix,
+                           KRaftVersion lastKRaftVersion) {
         this.log = logContext.logger(LocalLogManager.class);
         this.nodeId = nodeId;
         this.shared = shared;
         this.maxReadOffset = shared.initialMaxReadOffset();
         this.eventQueue = new KafkaEventQueue(Time.SYSTEM, logContext,
                 threadNamePrefix, new ShutdownEvent());
+        this.lastKRaftVersion = lastKRaftVersion;
         shared.registerLogManager(this);
     }
 
@@ -570,10 +563,10 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
                             if (batch.newLeader.equals(sharedLeader)) {
                                 log.debug("Node {}: Executing handleLeaderChange {}",
                                     nodeId, sharedLeader);
-                                listenerData.handleLeaderChange(entryOffset, batch.newLeader);
                                 if (batch.newLeader.epoch() > leader.epoch()) {
                                     leader = batch.newLeader;
                                 }
+                                listenerData.handleLeaderChange(entryOffset, batch.newLeader);
                             } else {
                                 log.debug("Node {}: Ignoring {} since it doesn't match the latest known leader {}",
                                         nodeId, batch.newLeader, sharedLeader);
@@ -728,53 +721,24 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
     }
 
     @Override
-    public long scheduleAppend(int epoch, List<ApiMessageAndVersion> batch) {
-        if (batch.isEmpty()) {
-            throw new IllegalArgumentException("Batch cannot be empty");
-        }
-
-        List<ApiMessageAndVersion> first = batch.subList(0, batch.size() / 2);
-        List<ApiMessageAndVersion> second = batch.subList(batch.size() / 2, batch.size());
-
-        assertEquals(batch.size(), first.size() + second.size());
-        assertFalse(second.isEmpty());
-
-        OptionalLong firstOffset = first
-            .stream()
-            .mapToLong(record -> scheduleAtomicAppend(epoch,
-                    OptionalLong.empty(),
-                    Collections.singletonList(record)))
-            .max();
-
-        if (firstOffset.isPresent() && resignAfterNonAtomicCommit.getAndSet(false)) {
-            // Emulate losing leadership in the middle of a non-atomic append by not writing
-            // the rest of the batch and instead writing a leader change message
-            resign(leader.epoch());
-
-            return firstOffset.getAsLong() + second.size();
-        } else {
-            return second
-                .stream()
-                .mapToLong(record -> scheduleAtomicAppend(epoch,
-                        OptionalLong.empty(),
-                        Collections.singletonList(record)))
-                .max()
-                .getAsLong();
-        }
-    }
-
-    @Override
-    public long scheduleAtomicAppend(
+    public long prepareAppend(
         int epoch,
-        OptionalLong requiredEndOffset,
         List<ApiMessageAndVersion> batch
     ) {
         if (batch.isEmpty()) {
             throw new IllegalArgumentException("Batch cannot be empty");
         }
-        return shared.tryAppend(nodeId, leader.epoch(), requiredEndOffset, batch);
+
+        if (throwOnNextAppend.getAndSet(false)) {
+            throw new BufferAllocationException("Test asked to fail the next prepareAppend");
+        }
+
+        return shared.tryAppend(nodeId, epoch, batch);
     }
 
+    @Override
+    public void schedulePreparedAppend() { }
+
     @Override
     public void resign(int epoch) {
         if (epoch < 0) {
@@ -801,7 +765,6 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         try {
             shared.tryAppend(nodeId,
                     currentEpoch,
-                    OptionalLong.empty(),
                     new LeaderChangeBatch(nextLeader));
         } catch (NotLeaderException exp) {
             // the leader epoch has already advanced. resign is a no op.
@@ -853,9 +816,9 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
 
     public List<RaftClient.Listener<ApiMessageAndVersion>> listeners() {
         final CompletableFuture<List<RaftClient.Listener<ApiMessageAndVersion>>> future = new CompletableFuture<>();
-        eventQueue.append(() -> {
-            future.complete(listeners.values().stream().map(l -> l.listener).collect(Collectors.toList()));
-        });
+        eventQueue.append(() ->
+            future.complete(listeners.values().stream().map(l -> l.listener).collect(Collectors.toList()))
+        );
         try {
             return future.get();
         } catch (ExecutionException | InterruptedException e) {
@@ -878,7 +841,12 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         }
     }
 
-    public void resignAfterNonAtomicCommit() {
-        resignAfterNonAtomicCommit.set(true);
+    public void throwOnNextAppend() {
+        throwOnNextAppend.set(true);
+    }
+
+    @Override
+    public KRaftVersion kraftVersion() {
+        return lastKRaftVersion;
     }
 }
