@@ -19,7 +19,6 @@ package org.apache.kafka.raft;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.Uuid;
 import org.apache.kafka.common.compress.Compression;
-import org.apache.kafka.common.errors.CorruptRecordException;
 import org.apache.kafka.common.errors.OffsetOutOfRangeException;
 import org.apache.kafka.common.record.MemoryRecords;
 import org.apache.kafka.common.record.MemoryRecordsBuilder;
@@ -168,7 +167,7 @@ public class MockLog implements ReplicatedLog {
     }
 
     private void assertValidHighWatermarkMetadata(LogOffsetMetadata offsetMetadata) {
-        if (!offsetMetadata.metadata().isPresent()) {
+        if (offsetMetadata.metadata().isEmpty()) {
             return;
         }
 
@@ -280,7 +279,7 @@ public class MockLog implements ReplicatedLog {
 
     @Override
     public LogAppendInfo appendAsLeader(Records records, int epoch) {
-        return append(records, epoch, true);
+        return append(records, OptionalInt.of(epoch));
     }
 
     private long appendBatch(LogBatch batch) {
@@ -293,18 +292,16 @@ public class MockLog implements ReplicatedLog {
     }
 
     @Override
-    public LogAppendInfo appendAsFollower(Records records, int epoch) {
-        return append(records, epoch, false);
+    public LogAppendInfo appendAsFollower(Records records) {
+        return append(records, OptionalInt.empty());
     }
 
-    private LogAppendInfo append(Records records, int epoch, boolean isLeader) {
-        if (records.sizeInBytes() == 0) {
+    private LogAppendInfo append(Records records, OptionalInt epoch) {
+        if (records.sizeInBytes() == 0)
             throw new IllegalArgumentException("Attempt to append an empty record set");
-        }
 
         long baseOffset = endOffset().offset();
         long lastOffset = baseOffset;
-        boolean hasBatches = false;
         for (RecordBatch batch : records.batches()) {
             if (batch.baseOffset() != endOffset().offset()) {
                 /* KafkaMetadataLog throws an kafka.common.UnexpectedAppendOffsetException this is the
@@ -317,47 +314,26 @@ public class MockLog implements ReplicatedLog {
                         endOffset().offset()
                     )
                 );
-            } else if (isLeader && epoch != batch.partitionLeaderEpoch()) {
-                // the partition leader epoch is set and does not match the one set in the batch
-                throw new RuntimeException(
-                    String.format(
-                        "Epoch %s doesn't match batch leader epoch %s",
-                        epoch,
-                        batch.partitionLeaderEpoch()
-                    )
-                );
-            } else if (!isLeader && batch.partitionLeaderEpoch() > epoch) {
-                /* To avoid inconsistent log replication, follower should only append record
-                 * batches with an epoch less than or equal to the leader epoch. There is more
-                 * details on this issue and scenario in KAFKA-18723.
-                 */
-                break;
             }
 
-            hasBatches = true;
             LogBatch logBatch = new LogBatch(
-                batch.partitionLeaderEpoch(),
+                epoch.orElseGet(batch::partitionLeaderEpoch),
                 batch.isControlBatch(),
                 buildEntries(batch, Record::offset)
             );
 
             if (logger.isDebugEnabled()) {
-                logger.debug(
-                    "{} appending to the log {}",
-                    isLeader ? "Leader" : "Follower",
-                    logBatch
-                );
+                String nodeState = "Follower";
+                if (epoch.isPresent()) {
+                    nodeState = "Leader";
+                }
+                logger.debug("{} appending to the log {}", nodeState, logBatch);
             }
 
             appendBatch(logBatch);
             lastOffset = logBatch.last().offset;
         }
 
-        if (!hasBatches) {
-            // This emulates the default handling when records doesn't have enough bytes for a batch
-            throw new CorruptRecordException("Append failed unexpectedly");
-        }
-
         return new LogAppendInfo(baseOffset, lastOffset);
     }
 
@@ -514,6 +490,18 @@ public class MockLog implements ReplicatedLog {
             );
         }
 
+        long baseOffset = read(snapshotId.offset(), Isolation.COMMITTED).startOffsetMetadata.offset();
+        if (snapshotId.offset() != baseOffset) {
+            throw new IllegalArgumentException(
+                String.format(
+                    "Cannot create snapshot at offset (%s) because it is not batch aligned. " +
+                    "The batch containing the requested offset has a base offset of (%s)",
+                    snapshotId.offset(),
+                    baseOffset
+                )
+            );
+        }
+
         return createNewSnapshotUnchecked(snapshotId);
     }
 
