@@ -21,13 +21,15 @@ import java.util.Map;
 
 public enum KRaftVersion implements FeatureVersion {
     // Version 0 is the initial version of KRaft.
-    KRAFT_VERSION_0(0, MetadataVersion.MINIMUM_KRAFT_VERSION),
+    KRAFT_VERSION_0(0, MetadataVersion.MINIMUM_VERSION),
 
     // Version 1 enables KIP-853.
     KRAFT_VERSION_1(1, MetadataVersion.IBP_3_9_IV0);
 
     public static final String FEATURE_NAME = "kraft.version";
 
+    public static final KRaftVersion LATEST_PRODUCTION = KRAFT_VERSION_1;
+
     private final short featureLevel;
     private final MetadataVersion bootstrapMetadataVersion;
 
@@ -71,7 +73,12 @@ public enum KRaftVersion implements FeatureVersion {
 
     @Override
     public Map<String, Short> dependencies() {
-        return Collections.emptyMap();
+        if (this.featureLevel == 0) {
+            return Collections.emptyMap();
+        } else {
+            return Collections.singletonMap(
+                MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_9_IV0.featureLevel());
+        }
     }
 
     public short quorumStateVersion() {
