@@ -16,11 +16,10 @@
  */
 package org.apache.kafka.server.config;
 
-import java.util.Arrays;
 import java.util.List;
 
 /**
- * Represents all the entities that can be configured via ZK
+ * Represents all the entities that can be configured.
  */
 public class ConfigType {
     public static final String TOPIC = "topics";
@@ -29,7 +28,7 @@ public class ConfigType {
     public static final String BROKER = "brokers";
     public static final String IP = "ips";
     public static final String CLIENT_METRICS = "client-metrics";
+    public static final String GROUP = "groups";
 
-    // Do not include ClientMetrics in `all` as ClientMetrics is not supported on ZK.
-    public static final List<String> ALL = Arrays.asList(TOPIC, CLIENT, USER, BROKER, IP);
+    public static final List<String> ALL = List.of(TOPIC, CLIENT, USER, BROKER, IP, CLIENT_METRICS, GROUP);
 }
