@@ -17,8 +17,6 @@
 
 package org.apache.kafka.shell.command;
 
-import static org.junit.jupiter.api.Assertions.assertEquals;
-
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
 
@@ -26,43 +24,45 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.Optional;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+
 @Timeout(value = 120)
 public class CommandTest {
     @Test
     public void testParseCommands() {
-        assertEquals(new CatCommandHandler(Arrays.asList("foo")),
+        assertEquals(new CatCommandHandler(Collections.singletonList("foo")),
             new Commands(true).parseCommand(Arrays.asList("cat", "foo")));
         assertEquals(new CdCommandHandler(Optional.empty()),
-            new Commands(true).parseCommand(Arrays.asList("cd")));
+            new Commands(true).parseCommand(Collections.singletonList("cd")));
         assertEquals(new CdCommandHandler(Optional.of("foo")),
             new Commands(true).parseCommand(Arrays.asList("cd", "foo")));
         assertEquals(new ExitCommandHandler(),
-            new Commands(true).parseCommand(Arrays.asList("exit")));
+            new Commands(true).parseCommand(Collections.singletonList("exit")));
         assertEquals(new HelpCommandHandler(),
-            new Commands(true).parseCommand(Arrays.asList("help")));
+            new Commands(true).parseCommand(Collections.singletonList("help")));
         assertEquals(new HistoryCommandHandler(3),
             new Commands(true).parseCommand(Arrays.asList("history", "3")));
         assertEquals(new HistoryCommandHandler(Integer.MAX_VALUE),
-            new Commands(true).parseCommand(Arrays.asList("history")));
+            new Commands(true).parseCommand(Collections.singletonList("history")));
         assertEquals(new LsCommandHandler(Collections.emptyList()),
-            new Commands(true).parseCommand(Arrays.asList("ls")));
+            new Commands(true).parseCommand(Collections.singletonList("ls")));
         assertEquals(new LsCommandHandler(Arrays.asList("abc", "123")),
             new Commands(true).parseCommand(Arrays.asList("ls", "abc", "123")));
         assertEquals(new PwdCommandHandler(),
-            new Commands(true).parseCommand(Arrays.asList("pwd")));
+            new Commands(true).parseCommand(Collections.singletonList("pwd")));
     }
 
     @Test
     public void testParseInvalidCommand() {
         assertEquals(new ErroneousCommandHandler("invalid choice: 'blah' (choose " +
             "from 'cat', 'cd', 'exit', 'find', 'help', 'history', 'ls', 'man', 'pwd', 'tree')"),
-            new Commands(true).parseCommand(Arrays.asList("blah")));
+            new Commands(true).parseCommand(Collections.singletonList("blah")));
     }
 
     @Test
     public void testEmptyCommandLine() {
         assertEquals(new NoOpCommandHandler(),
-            new Commands(true).parseCommand(Arrays.asList("")));
+            new Commands(true).parseCommand(Collections.singletonList("")));
         assertEquals(new NoOpCommandHandler(),
             new Commands(true).parseCommand(Collections.emptyList()));
     }
