@@ -21,7 +21,6 @@ import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.compress.Compression;
 import org.apache.kafka.common.errors.CorruptRecordException;
 import org.apache.kafka.common.errors.InvalidTimestampException;
-import org.apache.kafka.common.errors.UnsupportedCompressionTypeException;
 import org.apache.kafka.common.errors.UnsupportedForMessageFormatException;
 import org.apache.kafka.common.protocol.Errors;
 import org.apache.kafka.common.record.AbstractRecords;
@@ -40,7 +39,6 @@ import org.apache.kafka.common.utils.CloseableIterator;
 import org.apache.kafka.common.utils.PrimitiveRef;
 import org.apache.kafka.common.utils.PrimitiveRef.LongRef;
 import org.apache.kafka.common.utils.Time;
-import org.apache.kafka.server.common.MetadataVersion;
 
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
@@ -49,8 +47,6 @@ import java.util.List;
 import java.util.Optional;
 import java.util.stream.Collectors;
 
-import static org.apache.kafka.server.common.MetadataVersion.IBP_2_1_IV0;
-
 public class LogValidator {
 
     public interface MetricsRecorder {
@@ -69,15 +65,20 @@ public class LogValidator {
         public final long logAppendTimeMs;
         public final MemoryRecords validatedRecords;
         public final long maxTimestampMs;
+        // we only maintain batch level offset for max timestamp since we want to align the behavior of updating time
+        // indexing entries. The paths of follower append and replica recovery do not iterate all records, so they have no
+        // idea about record level offset for max timestamp.
+        public final long shallowOffsetOfMaxTimestamp;
         public final boolean messageSizeMaybeChanged;
         public final RecordValidationStats recordValidationStats;
 
         public ValidationResult(long logAppendTimeMs, MemoryRecords validatedRecords, long maxTimestampMs,
-                                boolean messageSizeMaybeChanged,
+                                long shallowOffsetOfMaxTimestamp, boolean messageSizeMaybeChanged,
                                 RecordValidationStats recordValidationStats) {
             this.logAppendTimeMs = logAppendTimeMs;
             this.validatedRecords = validatedRecords;
             this.maxTimestampMs = maxTimestampMs;
+            this.shallowOffsetOfMaxTimestamp = shallowOffsetOfMaxTimestamp;
             this.messageSizeMaybeChanged = messageSizeMaybeChanged;
             this.recordValidationStats = recordValidationStats;
         }
@@ -105,7 +106,6 @@ public class LogValidator {
     private final long timestampAfterMaxMs;
     private final int partitionLeaderEpoch;
     private final AppendOrigin origin;
-    private final MetadataVersion interBrokerProtocolVersion;
 
     public LogValidator(MemoryRecords records,
                         TopicPartition topicPartition,
@@ -118,8 +118,7 @@ public class LogValidator {
                         long timestampBeforeMaxMs,
                         long timestampAfterMaxMs,
                         int partitionLeaderEpoch,
-                        AppendOrigin origin,
-                        MetadataVersion interBrokerProtocolVersion) {
+                        AppendOrigin origin) {
         this.records = records;
         this.topicPartition = topicPartition;
         this.time = time;
@@ -132,7 +131,6 @@ public class LogValidator {
         this.timestampAfterMaxMs = timestampAfterMaxMs;
         this.partitionLeaderEpoch = partitionLeaderEpoch;
         this.origin = origin;
-        this.interBrokerProtocolVersion = interBrokerProtocolVersion;
     }
 
     /**
@@ -231,6 +229,7 @@ public class LogValidator {
             now,
             convertedRecords,
             info.maxTimestamp,
+            info.shallowOffsetOfMaxTimestamp,
             true,
             recordValidationStats);
     }
@@ -240,6 +239,8 @@ public class LogValidator {
                                                        MetricsRecorder metricsRecorder) {
         long now = time.milliseconds();
         long maxTimestamp = RecordBatch.NO_TIMESTAMP;
+        long shallowOffsetOfMaxTimestamp = -1L;
+        long initialOffset = offsetCounter.value;
 
         RecordBatch firstBatch = getFirstBatchAndMaybeValidateNoMoreBatches(records, CompressionType.NONE);
 
@@ -268,6 +269,7 @@ public class LogValidator {
 
             if (batch.magic() > RecordBatch.MAGIC_VALUE_V0 && maxBatchTimestamp > maxTimestamp) {
                 maxTimestamp = maxBatchTimestamp;
+                shallowOffsetOfMaxTimestamp = offsetCounter.value - 1;
             }
 
             batch.setLastOffset(offsetCounter.value - 1);
@@ -284,10 +286,23 @@ public class LogValidator {
         }
 
         if (timestampType == TimestampType.LOG_APPEND_TIME) {
-            if (toMagic == RecordBatch.MAGIC_VALUE_V0) {
-                maxTimestamp = RecordBatch.NO_TIMESTAMP;
-            } else {
-                maxTimestamp = now;
+            maxTimestamp = now;
+            // those checks should be equal to MemoryRecordsBuilder#info
+            switch (toMagic) {
+                case RecordBatch.MAGIC_VALUE_V0:
+                    maxTimestamp = RecordBatch.NO_TIMESTAMP;
+                    // value will be the default value: -1
+                    shallowOffsetOfMaxTimestamp = -1;
+                    break;
+                case RecordBatch.MAGIC_VALUE_V1:
+                    // Those single-record batches have same max timestamp, so the initial offset is equal with
+                    // the last offset of earliest batch
+                    shallowOffsetOfMaxTimestamp = initialOffset;
+                    break;
+                default:
+                    // there is only one batch so use the last offset
+                    shallowOffsetOfMaxTimestamp = offsetCounter.value - 1;
+                    break;
             }
         }
 
@@ -295,6 +310,7 @@ public class LogValidator {
             now,
             records,
             maxTimestamp,
+            shallowOffsetOfMaxTimestamp,
             false,
             RecordValidationStats.EMPTY);
     }
@@ -309,10 +325,6 @@ public class LogValidator {
     public ValidationResult validateMessagesAndAssignOffsetsCompressed(LongRef offsetCounter,
                                                                        MetricsRecorder metricsRecorder,
                                                                        BufferSupplier bufferSupplier) {
-        if (targetCompression.type() == CompressionType.ZSTD && interBrokerProtocolVersion.isLessThan(IBP_2_1_IV0))
-            throw new UnsupportedCompressionTypeException("Produce requests to inter.broker.protocol.version < 2.1 broker " +
-                "are not allowed to use ZStandard compression");
-
         // No in place assignment situation 1
         boolean inPlaceAssignment = sourceCompressionType == targetCompression.type();
         long now = time.milliseconds();
@@ -422,6 +434,7 @@ public class LogValidator {
                 now,
                 records,
                 maxTimestamp,
+                lastOffset,
                 false,
                 recordValidationStats);
         }
@@ -463,6 +476,7 @@ public class LogValidator {
             logAppendTime,
             records,
             info.maxTimestamp,
+            info.shallowOffsetOfMaxTimestamp,
             true,
             recordValidationStats);
     }
