@@ -16,14 +16,12 @@
  */
 package org.apache.kafka.server.log.remote.metadata.storage;
 
-import kafka.test.ClusterInstance;
-import kafka.test.annotation.ClusterTest;
-import kafka.test.annotation.ClusterTestDefaults;
-import kafka.test.junit.ClusterTestExtensions;
-
 import org.apache.kafka.common.TopicIdPartition;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.Uuid;
+import org.apache.kafka.common.test.ClusterInstance;
+import org.apache.kafka.common.test.api.ClusterTest;
+import org.apache.kafka.common.test.api.ClusterTestDefaults;
 import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.server.log.remote.storage.RemoteLogMetadataManager;
 import org.apache.kafka.server.log.remote.storage.RemoteLogSegmentId;
@@ -34,8 +32,6 @@ import org.apache.kafka.server.log.remote.storage.RemoteStorageException;
 import org.apache.kafka.storage.internals.log.EpochEntry;
 import org.apache.kafka.test.TestUtils;
 
-import org.junit.jupiter.api.extension.ExtendWith;
-
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
@@ -55,7 +51,6 @@ import static org.mockito.Mockito.spy;
 import static org.mockito.Mockito.verify;
 
 @ClusterTestDefaults(brokers = 3)
-@ExtendWith(value = ClusterTestExtensions.class)
 public class RemoteLogSegmentLifecycleTest {
 
     private final int segSize = 1048576;
