@@ -36,7 +36,6 @@ import org.apache.kafka.streams.errors.TaskCorruptedException;
 import org.apache.kafka.streams.processor.TaskId;
 import org.apache.kafka.streams.processor.internals.Task.State;
 import org.apache.kafka.streams.processor.internals.TaskAndAction.Action;
-import org.apache.kafka.streams.processor.internals.metrics.StreamsMetricsImpl;
 
 import org.slf4j.Logger;
 
@@ -56,7 +55,6 @@ import java.util.Set;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
@@ -91,18 +89,18 @@ public class DefaultStateUpdater implements StateUpdater {
         private volatile KafkaFutureImpl<Uuid> clientInstanceIdFuture = new KafkaFutureImpl<>();
 
         public StateUpdaterThread(final String name,
-                                  final StreamsMetricsImpl metrics,
+                                  final Metrics metrics,
                                   final ChangelogReader changelogReader) {
             super(name);
             this.changelogReader = changelogReader;
             this.updaterMetrics = new StateUpdaterMetrics(metrics, name);
         }
 
-        public Collection<Task> getUpdatingTasks() {
+        public Collection<Task> updatingTasks() {
             return updatingTasks.values();
         }
 
-        public Collection<StandbyTask> getUpdatingStandbyTasks() {
+        public Collection<StandbyTask> updatingStandbyTasks() {
             return updatingTasks.values().stream()
                 .filter(t -> !t.isActive())
                 .map(t -> (StandbyTask) t)
@@ -113,29 +111,29 @@ public class DefaultStateUpdater implements StateUpdater {
             return !updatingTasks.isEmpty() && updatingTasks.values().stream().noneMatch(Task::isActive);
         }
 
-        public Collection<Task> getPausedTasks() {
+        public Collection<Task> pausedTasks() {
             return pausedTasks.values();
         }
 
-        public long getNumUpdatingStandbyTasks() {
+        public long numUpdatingStandbyTasks() {
             return updatingTasks.values().stream()
                 .filter(t -> !t.isActive())
                 .count();
         }
 
-        public long getNumRestoringActiveTasks() {
+        public long numRestoringActiveTasks() {
             return updatingTasks.values().stream()
                 .filter(Task::isActive)
                 .count();
         }
 
-        public long getNumPausedStandbyTasks() {
+        public long numPausedStandbyTasks() {
             return pausedTasks.values().stream()
                 .filter(t -> !t.isActive())
                 .count();
         }
 
-        public long getNumPausedActiveTasks() {
+        public long numPausedActiveTasks() {
             return pausedTasks.values().stream()
                 .filter(Task::isActive)
                 .count();
@@ -154,7 +152,6 @@ public class DefaultStateUpdater implements StateUpdater {
                 clearInputQueue();
                 clearUpdatingAndPausedTasks();
                 updaterMetrics.clear();
-                shutdownGate.countDown();
                 log.info("State updater thread stopped");
             }
         }
@@ -203,7 +200,7 @@ public class DefaultStateUpdater implements StateUpdater {
         private void performActionsOnTasks() {
             tasksAndActionsLock.lock();
             try {
-                for (final TaskAndAction taskAndAction : getTasksAndActions()) {
+                for (final TaskAndAction taskAndAction : tasksAndActions()) {
                     final Action action = taskAndAction.action();
                     switch (action) {
                         case ADD:
@@ -461,7 +458,7 @@ public class DefaultStateUpdater implements StateUpdater {
             changelogReader.clear();
         }
 
-        private List<TaskAndAction> getTasksAndActions() {
+        private List<TaskAndAction> tasksAndActions() {
             final List<TaskAndAction> tasksAndActionsToProcess = new ArrayList<>(tasksAndActions);
             tasksAndActions.clear();
             return tasksAndActionsToProcess;
@@ -748,7 +745,7 @@ public class DefaultStateUpdater implements StateUpdater {
     private final Time time;
     private final Logger log;
     private final String name;
-    private final StreamsMetricsImpl metrics;
+    private final Metrics metrics;
     private final Consumer<byte[], byte[]> restoreConsumer;
     private final ChangelogReader changelogReader;
     private final TopologyMetadata topologyMetadata;
@@ -767,10 +764,9 @@ public class DefaultStateUpdater implements StateUpdater {
     private long lastCommitMs;
 
     private StateUpdaterThread stateUpdaterThread = null;
-    private CountDownLatch shutdownGate;
 
     public DefaultStateUpdater(final String name,
-                               final StreamsMetricsImpl metrics,
+                               final Metrics metrics,
                                final StreamsConfig config,
                                final Consumer<byte[], byte[]> restoreConsumer,
                                final ChangelogReader changelogReader,
@@ -797,7 +793,6 @@ public class DefaultStateUpdater implements StateUpdater {
             }
             stateUpdaterThread = new StateUpdaterThread(name, metrics, changelogReader);
             stateUpdaterThread.start();
-            shutdownGate = new CountDownLatch(1);
 
             // initialize the last commit as of now to prevent first commit happens immediately
             this.lastCommitMs = time.milliseconds();
@@ -822,7 +817,8 @@ public class DefaultStateUpdater implements StateUpdater {
             }
 
             try {
-                if (!shutdownGate.await(timeout.toMillis(), TimeUnit.MILLISECONDS)) {
+                stateUpdaterThread.join(timeout.toMillis());
+                if (stateUpdaterThread.isAlive()) {
                     throw new StreamsException("State updater thread did not shutdown within the timeout");
                 }
                 stateUpdaterThread = null;
@@ -928,20 +924,20 @@ public class DefaultStateUpdater implements StateUpdater {
         }
     }
 
-    public Set<StandbyTask> getUpdatingStandbyTasks() {
+    public Set<StandbyTask> updatingStandbyTasks() {
         return stateUpdaterThread != null
-            ? Collections.unmodifiableSet(new HashSet<>(stateUpdaterThread.getUpdatingStandbyTasks()))
+            ? Collections.unmodifiableSet(new HashSet<>(stateUpdaterThread.updatingStandbyTasks()))
             : Collections.emptySet();
     }
 
     @Override
-    public Set<Task> getUpdatingTasks() {
+    public Set<Task> updatingTasks() {
         return stateUpdaterThread != null
-            ? Collections.unmodifiableSet(new HashSet<>(stateUpdaterThread.getUpdatingTasks()))
+            ? Collections.unmodifiableSet(new HashSet<>(stateUpdaterThread.updatingTasks()))
             : Collections.emptySet();
     }
 
-    public Set<StreamTask> getRestoredActiveTasks() {
+    public Set<StreamTask> restoredActiveTasks() {
         restoredActiveTasksLock.lock();
         try {
             return Collections.unmodifiableSet(new HashSet<>(restoredActiveTasks));
@@ -950,7 +946,7 @@ public class DefaultStateUpdater implements StateUpdater {
         }
     }
 
-    public List<ExceptionAndTask> getExceptionsAndFailedTasks() {
+    public List<ExceptionAndTask> exceptionsAndFailedTasks() {
         exceptionsAndFailedTasksLock.lock();
         try {
             return Collections.unmodifiableList(new ArrayList<>(exceptionsAndFailedTasks));
@@ -959,38 +955,38 @@ public class DefaultStateUpdater implements StateUpdater {
         }
     }
 
-    public Set<Task> getRemovedTasks() {
+    public Set<Task> removedTasks() {
         return Collections.unmodifiableSet(new HashSet<>(removedTasks));
     }
 
-    public Set<Task> getPausedTasks() {
+    public Set<Task> pausedTasks() {
         return stateUpdaterThread != null
-            ? Collections.unmodifiableSet(new HashSet<>(stateUpdaterThread.getPausedTasks()))
+            ? Collections.unmodifiableSet(new HashSet<>(stateUpdaterThread.pausedTasks()))
             : Collections.emptySet();
     }
 
     @Override
-    public Set<Task> getTasks() {
-        return executeWithQueuesLocked(() -> getStreamOfTasks().map(ReadOnlyTask::new).collect(Collectors.toSet()));
+    public Set<Task> tasks() {
+        return executeWithQueuesLocked(() -> streamOfTasks().map(ReadOnlyTask::new).collect(Collectors.toSet()));
     }
 
     @Override
     public boolean restoresActiveTasks() {
         return !executeWithQueuesLocked(
-            () -> getStreamOfTasks().filter(Task::isActive).collect(Collectors.toSet())
+            () -> streamOfTasks().filter(Task::isActive).collect(Collectors.toSet())
         ).isEmpty();
     }
 
-    public Set<StreamTask> getActiveTasks() {
+    public Set<StreamTask> activeTasks() {
         return executeWithQueuesLocked(
-            () -> getStreamOfTasks().filter(Task::isActive).map(t -> (StreamTask) t).collect(Collectors.toSet())
+            () -> streamOfTasks().filter(Task::isActive).map(t -> (StreamTask) t).collect(Collectors.toSet())
         );
     }
 
     @Override
-    public Set<StandbyTask> getStandbyTasks() {
+    public Set<StandbyTask> standbyTasks() {
         return executeWithQueuesLocked(
-            () -> getStreamOfTasks().filter(t -> !t.isActive()).map(t -> (StandbyTask) t).collect(Collectors.toSet())
+            () -> streamOfTasks().filter(t -> !t.isActive()).map(t -> (StandbyTask) t).collect(Collectors.toSet())
         );
     }
 
@@ -1024,22 +1020,22 @@ public class DefaultStateUpdater implements StateUpdater {
         }
     }
 
-    private Stream<Task> getStreamOfTasks() {
+    private Stream<Task> streamOfTasks() {
         return
             Stream.concat(
-                getStreamOfNonPausedTasks(),
-                getPausedTasks().stream()
+                streamOfNonPausedTasks(),
+                pausedTasks().stream()
             );
     }
 
-    private Stream<Task> getStreamOfNonPausedTasks() {
+    private Stream<Task> streamOfNonPausedTasks() {
         return
             Stream.concat(
                 tasksAndActions.stream()
                     .filter(taskAndAction -> taskAndAction.action() == Action.ADD)
                     .map(TaskAndAction::task),
                 Stream.concat(
-                    getUpdatingTasks().stream(),
+                    updatingTasks().stream(),
                     Stream.concat(
                         restoredActiveTasks.stream(),
                         Stream.concat(
@@ -1063,75 +1059,74 @@ public class DefaultStateUpdater implements StateUpdater {
         private final Sensor standbyRestoreRatioSensor;
         private final Sensor checkpointRatioSensor;
 
-        private final Deque<Sensor> allSensors = new LinkedList<>();
+        private final Deque<String> allSensorNames = new LinkedList<>();
         private final Deque<MetricName> allMetricNames = new LinkedList<>();
 
-        private StateUpdaterMetrics(final StreamsMetricsImpl metrics, final String threadId) {
+        private StateUpdaterMetrics(final Metrics metrics, final String threadId) {
             final Map<String, String> threadLevelTags = new LinkedHashMap<>();
             threadLevelTags.put(THREAD_ID_TAG, threadId);
-            final Metrics metricsRegistry = metrics.metricsRegistry();
 
-            MetricName metricName = metricsRegistry.metricName("active-restoring-tasks",
+            MetricName metricName = metrics.metricName("active-restoring-tasks",
                 STATE_LEVEL_GROUP,
                 "The number of active tasks currently undergoing restoration",
                 threadLevelTags);
-            metricsRegistry.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
-                stateUpdaterThread.getNumRestoringActiveTasks() : 0);
+            metrics.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
+                stateUpdaterThread.numRestoringActiveTasks() : 0);
             allMetricNames.push(metricName);
 
-            metricName = metricsRegistry.metricName("standby-updating-tasks",
+            metricName = metrics.metricName("standby-updating-tasks",
                 STATE_LEVEL_GROUP,
                 "The number of standby tasks currently undergoing state update",
                 threadLevelTags);
-            metricsRegistry.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
-                stateUpdaterThread.getNumUpdatingStandbyTasks() : 0);
+            metrics.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
+                stateUpdaterThread.numUpdatingStandbyTasks() : 0);
             allMetricNames.push(metricName);
 
-            metricName = metricsRegistry.metricName("active-paused-tasks",
+            metricName = metrics.metricName("active-paused-tasks",
                 STATE_LEVEL_GROUP,
                 "The number of active tasks paused restoring",
                 threadLevelTags);
-            metricsRegistry.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
-                stateUpdaterThread.getNumPausedActiveTasks() : 0);
+            metrics.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
+                stateUpdaterThread.numPausedActiveTasks() : 0);
             allMetricNames.push(metricName);
 
-            metricName = metricsRegistry.metricName("standby-paused-tasks",
+            metricName = metrics.metricName("standby-paused-tasks",
                 STATE_LEVEL_GROUP,
                 "The number of standby tasks paused state update",
                 threadLevelTags);
-            metricsRegistry.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
-                stateUpdaterThread.getNumPausedStandbyTasks() : 0);
+            metrics.addMetric(metricName, (config, now) -> stateUpdaterThread != null ?
+                stateUpdaterThread.numPausedStandbyTasks() : 0);
             allMetricNames.push(metricName);
 
-            this.idleRatioSensor = metrics.threadLevelSensor(threadId, "idle-ratio", RecordingLevel.INFO);
+            this.idleRatioSensor = metrics.sensor("idle-ratio", RecordingLevel.INFO);
             this.idleRatioSensor.add(new MetricName("idle-ratio", STATE_LEVEL_GROUP, IDLE_RATIO_DESCRIPTION, threadLevelTags), new Avg());
-            allSensors.add(this.idleRatioSensor);
+            allSensorNames.add("idle-ratio");
 
-            this.activeRestoreRatioSensor = metrics.threadLevelSensor(threadId, "active-restore-ratio", RecordingLevel.INFO);
+            this.activeRestoreRatioSensor = metrics.sensor("active-restore-ratio", RecordingLevel.INFO);
             this.activeRestoreRatioSensor.add(new MetricName("active-restore-ratio", STATE_LEVEL_GROUP, RESTORE_RATIO_DESCRIPTION, threadLevelTags), new Avg());
-            allSensors.add(this.activeRestoreRatioSensor);
+            allSensorNames.add("active-restore-ratio");
 
-            this.standbyRestoreRatioSensor = metrics.threadLevelSensor(threadId, "standby-update-ratio", RecordingLevel.INFO);
+            this.standbyRestoreRatioSensor = metrics.sensor("standby-update-ratio", RecordingLevel.INFO);
             this.standbyRestoreRatioSensor.add(new MetricName("standby-update-ratio", STATE_LEVEL_GROUP, UPDATE_RATIO_DESCRIPTION, threadLevelTags), new Avg());
-            allSensors.add(this.standbyRestoreRatioSensor);
+            allSensorNames.add("standby-update-ratio");
 
-            this.checkpointRatioSensor = metrics.threadLevelSensor(threadId, "checkpoint-ratio", RecordingLevel.INFO);
+            this.checkpointRatioSensor = metrics.sensor("checkpoint-ratio", RecordingLevel.INFO);
             this.checkpointRatioSensor.add(new MetricName("checkpoint-ratio", STATE_LEVEL_GROUP, CHECKPOINT_RATIO_DESCRIPTION, threadLevelTags), new Avg());
-            allSensors.add(this.checkpointRatioSensor);
+            allSensorNames.add("checkpoint-ratio");
 
-            this.restoreSensor = metrics.threadLevelSensor(threadId, "restore-records", RecordingLevel.INFO);
+            this.restoreSensor = metrics.sensor("restore-records", RecordingLevel.INFO);
             this.restoreSensor.add(new MetricName("restore-records-rate", STATE_LEVEL_GROUP, RESTORE_RECORDS_RATE_DESCRIPTION, threadLevelTags), new Rate());
             this.restoreSensor.add(new MetricName("restore-call-rate", STATE_LEVEL_GROUP, RESTORE_RATE_DESCRIPTION, threadLevelTags), new Rate(new WindowedCount()));
-            allSensors.add(this.restoreSensor);
+            allSensorNames.add("restore-records");
         }
 
         void clear() {
-            while (!allSensors.isEmpty()) {
-                metrics.removeSensor(allSensors.pop());
+            while (!allSensorNames.isEmpty()) {
+                metrics.removeSensor(allSensorNames.pop());
             }
 
             while (!allMetricNames.isEmpty()) {
-                metrics.metricsRegistry().removeMetric(allMetricNames.pop());
+                metrics.removeMetric(allMetricNames.pop());
             }
         }
     }
