@@ -37,7 +37,6 @@ import static java.util.Arrays.asList;
 import static java.util.Collections.singletonList;
 import static org.apache.kafka.common.utils.Utils.mkEntry;
 import static org.apache.kafka.common.utils.Utils.mkMap;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.PID_1;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.PID_2;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.PID_3;
@@ -84,7 +83,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
 
     @Test
     public void shouldNotAssignStatelessTasksToAnyClients() {
-        final Set<TaskId> statefulTasks = mkSet(
+        final Set<TaskId> statefulTasks = Set.of(
             TASK_1_0,
             TASK_1_1,
             TASK_1_2
@@ -119,7 +118,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
     @Test
     public void shouldRemoveClientToRemainingStandbysAndNotPopulatePendingStandbyTasksToClientIdWhenAllStandbyTasksWereAssigned() {
         final int numStandbyReplicas = 2;
-        final Set<String> rackAwareAssignmentTags = mkSet(ZONE_TAG, CLUSTER_TAG);
+        final Set<String> rackAwareAssignmentTags = Set.of(ZONE_TAG, CLUSTER_TAG);
         final Map<ProcessId, ClientState> clientStates = mkMap(
             mkEntry(PID_1, createClientStateWithCapacity(PID_1, 2, mkMap(mkEntry(ZONE_TAG, ZONE_1), mkEntry(CLUSTER_TAG, CLUSTER_1)), TASK_0_0)),
             mkEntry(PID_2, createClientStateWithCapacity(PID_2, 2, mkMap(mkEntry(ZONE_TAG, ZONE_1), mkEntry(CLUSTER_TAG, CLUSTER_2)), TASK_0_1)),
@@ -161,7 +160,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
 
     @Test
     public void shouldUpdateClientToRemainingStandbysAndPendingStandbyTasksToClientIdWhenNotAllStandbyTasksWereAssigned() {
-        final Set<String> rackAwareAssignmentTags = mkSet(ZONE_TAG, CLUSTER_TAG);
+        final Set<String> rackAwareAssignmentTags = Set.of(ZONE_TAG, CLUSTER_TAG);
         final Map<ProcessId, ClientState> clientStates = mkMap(
             mkEntry(PID_1, createClientStateWithCapacity(PID_1, 2, mkMap(mkEntry(ZONE_TAG, ZONE_1), mkEntry(CLUSTER_TAG, CLUSTER_1)), TASK_0_0)),
             mkEntry(PID_2, createClientStateWithCapacity(PID_2, 2, mkMap(mkEntry(ZONE_TAG, ZONE_1), mkEntry(CLUSTER_TAG, CLUSTER_2)), TASK_0_1)),
@@ -315,7 +314,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_0,
                 clientStates,
                 asList(
-                    mkSet(PID_9, PID_5), mkSet(PID_6, PID_8)
+                    Set.of(PID_9, PID_5), Set.of(PID_6, PID_8)
                 )
             )
         );
@@ -324,7 +323,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_0,
                 clientStates,
                 asList(
-                    mkSet(PID_9, PID_5), mkSet(PID_6, PID_8)
+                    Set.of(PID_9, PID_5), Set.of(PID_6, PID_8)
                 )
             )
         );
@@ -334,7 +333,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_1,
                 clientStates,
                 asList(
-                    mkSet(PID_2, PID_9), mkSet(PID_3, PID_8)
+                    Set.of(PID_2, PID_9), Set.of(PID_3, PID_8)
                 )
             )
         );
@@ -343,7 +342,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_1,
                 clientStates,
                 asList(
-                    mkSet(PID_2, PID_9), mkSet(PID_3, PID_8)
+                    Set.of(PID_2, PID_9), Set.of(PID_3, PID_8)
                 )
             )
         );
@@ -353,7 +352,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_2,
                 clientStates,
                 asList(
-                    mkSet(PID_5, PID_3), mkSet(PID_2, PID_6)
+                    Set.of(PID_5, PID_3), Set.of(PID_2, PID_6)
                 )
             )
         );
@@ -362,7 +361,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_2,
                 clientStates,
                 asList(
-                    mkSet(PID_5, PID_3), mkSet(PID_2, PID_6)
+                    Set.of(PID_5, PID_3), Set.of(PID_2, PID_6)
                 )
             )
         );
@@ -437,14 +436,14 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                     containsStandbyTasks(
                         TASK_0_0,
                         cs,
-                        mkSet(PID_2, PID_3, PID_5, PID_6, PID_8, PID_9)
+                        Set.of(PID_2, PID_3, PID_5, PID_6, PID_8, PID_9)
                     )
                 );
                 assertTrue(
                     containsStandbyTasks(
                         TASK_1_0,
                         cs,
-                        mkSet(PID_2, PID_3, PID_5, PID_6, PID_8, PID_9)
+                        Set.of(PID_2, PID_3, PID_5, PID_6, PID_8, PID_9)
                     )
                 );
 
@@ -452,14 +451,14 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                     containsStandbyTasks(
                         TASK_0_1,
                         cs,
-                        mkSet(PID_1, PID_3, PID_4, PID_6, PID_7, PID_9)
+                        Set.of(PID_1, PID_3, PID_4, PID_6, PID_7, PID_9)
                     )
                 );
                 assertTrue(
                     containsStandbyTasks(
                         TASK_1_1,
                         cs,
-                        mkSet(PID_1, PID_3, PID_4, PID_6, PID_7, PID_9)
+                        Set.of(PID_1, PID_3, PID_4, PID_6, PID_7, PID_9)
                     )
                 );
 
@@ -467,14 +466,14 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                     containsStandbyTasks(
                         TASK_0_2,
                         cs,
-                        mkSet(PID_1, PID_2, PID_4, PID_5, PID_7, PID_8)
+                        Set.of(PID_1, PID_2, PID_4, PID_5, PID_7, PID_8)
                     )
                 );
                 assertTrue(
                     containsStandbyTasks(
                         TASK_1_2,
                         cs,
-                        mkSet(PID_1, PID_2, PID_4, PID_5, PID_7, PID_8)
+                        Set.of(PID_1, PID_2, PID_4, PID_5, PID_7, PID_8)
                     )
                 );
             }
@@ -513,7 +512,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_0,
                 clientStates,
                 asList(
-                    mkSet(PID_9, PID_5), mkSet(PID_6, PID_8)
+                    Set.of(PID_9, PID_5), Set.of(PID_6, PID_8)
                 )
             )
         );
@@ -522,7 +521,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_0,
                 clientStates,
                 asList(
-                    mkSet(PID_9, PID_5), mkSet(PID_6, PID_8)
+                    Set.of(PID_9, PID_5), Set.of(PID_6, PID_8)
                 )
             )
         );
@@ -532,7 +531,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_1,
                 clientStates,
                 asList(
-                    mkSet(PID_4, PID_9), mkSet(PID_6, PID_7)
+                    Set.of(PID_4, PID_9), Set.of(PID_6, PID_7)
                 )
             )
         );
@@ -541,7 +540,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_1,
                 clientStates,
                 asList(
-                    mkSet(PID_4, PID_9), mkSet(PID_6, PID_7)
+                    Set.of(PID_4, PID_9), Set.of(PID_6, PID_7)
                 )
             )
         );
@@ -551,7 +550,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_2,
                 clientStates,
                 asList(
-                    mkSet(PID_5, PID_7), mkSet(PID_4, PID_8)
+                    Set.of(PID_5, PID_7), Set.of(PID_4, PID_8)
                 )
             )
         );
@@ -560,7 +559,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_2,
                 clientStates,
                 asList(
-                    mkSet(PID_5, PID_7), mkSet(PID_4, PID_8)
+                    Set.of(PID_5, PID_7), Set.of(PID_4, PID_8)
                 )
             )
         );
@@ -610,8 +609,8 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                     // For the 2nd standby, either `PID_3` or `PID_6` are valid destinations as
                     // we need to distribute the clients on different `zone`
                     // tags without considering the `cluster` tag value.
-                    mkSet(PID_5, PID_3),
-                    mkSet(PID_5, PID_6)
+                    Set.of(PID_5, PID_3),
+                    Set.of(PID_5, PID_6)
                 )
             )
         );
@@ -624,8 +623,8 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                     // `PID_2` is the ideal distribution on different `cluster`
                     // and `zone` tag dimensions. In contrast, `PID_4` and `PID_1`
                     // satisfy only the partial rack awareness as they are located on a different `zone` tag dimension.
-                    mkSet(PID_2, PID_4),
-                    mkSet(PID_2, PID_1)
+                    Set.of(PID_2, PID_4),
+                    Set.of(PID_2, PID_1)
                 )
             )
         );
@@ -655,7 +654,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_0,
                 clientStates,
                 asList(
-                    mkSet(PID_2), mkSet(PID_5), mkSet(PID_3), mkSet(PID_6)
+                    Set.of(PID_2), Set.of(PID_5), Set.of(PID_3), Set.of(PID_6)
                 )
             )
         );
@@ -664,7 +663,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_0,
                 clientStates,
                 asList(
-                    mkSet(PID_2), mkSet(PID_5), mkSet(PID_3), mkSet(PID_6)
+                    Set.of(PID_2), Set.of(PID_5), Set.of(PID_3), Set.of(PID_6)
                 )
             )
         );
@@ -674,7 +673,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_1,
                 clientStates,
                 asList(
-                    mkSet(PID_1), mkSet(PID_4), mkSet(PID_3), mkSet(PID_6)
+                    Set.of(PID_1), Set.of(PID_4), Set.of(PID_3), Set.of(PID_6)
                 )
             )
         );
@@ -683,7 +682,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_1,
                 clientStates,
                 asList(
-                    mkSet(PID_1), mkSet(PID_4), mkSet(PID_3), mkSet(PID_6)
+                    Set.of(PID_1), Set.of(PID_4), Set.of(PID_3), Set.of(PID_6)
                 )
             )
         );
@@ -693,7 +692,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_2,
                 clientStates,
                 asList(
-                    mkSet(PID_1), mkSet(PID_4), mkSet(PID_2), mkSet(PID_5)
+                    Set.of(PID_1), Set.of(PID_4), Set.of(PID_2), Set.of(PID_5)
                 )
             )
         );
@@ -702,7 +701,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_1_2,
                 clientStates,
                 asList(
-                    mkSet(PID_1), mkSet(PID_4), mkSet(PID_2), mkSet(PID_5)
+                    Set.of(PID_1), Set.of(PID_4), Set.of(PID_2), Set.of(PID_5)
                 )
             )
         );
@@ -744,7 +743,7 @@ public class ClientTagAwareStandbyTaskAssignorTest {
                 TASK_0_0,
                 clientStates,
                 singletonList(
-                    mkSet(PID_2)
+                    Set.of(PID_2)
                 )
             )
         );
