@@ -17,16 +17,7 @@
 package org.apache.kafka.streams.scala
 package kstream
 
-import org.apache.kafka.streams.KeyValue
-import org.apache.kafka.streams.kstream.{
-  GlobalKTable,
-  JoinWindows,
-  KStream => KStreamJ,
-  Printed,
-  TransformerSupplier,
-  ValueTransformerSupplier,
-  ValueTransformerWithKeySupplier
-}
+import org.apache.kafka.streams.kstream.{GlobalKTable, JoinWindows, KStream => KStreamJ, Printed}
 import org.apache.kafka.streams.processor.TopicNameExtractor
 import org.apache.kafka.streams.processor.api.{FixedKeyProcessorSupplier, ProcessorSupplier}
 import org.apache.kafka.streams.scala.FunctionsCompatConversions.{
@@ -36,11 +27,8 @@ import org.apache.kafka.streams.scala.FunctionsCompatConversions.{
   KeyValueMapperFromFunction,
   MapperFromFunction,
   PredicateFromFunction,
-  TransformerSupplierAsJava,
   ValueMapperFromFunction,
-  ValueMapperWithKeyFromFunction,
-  ValueTransformerSupplierAsJava,
-  ValueTransformerSupplierWithKeyAsJava
+  ValueMapperWithKeyFromFunction
 }
 
 import scala.jdk.CollectionConverters._
@@ -325,20 +313,6 @@ class KStream[K, V](val inner: KStreamJ[K, V]) {
   def foreach(action: (K, V) => Unit, named: Named): Unit =
     inner.foreach(action.asForeachAction, named)
 
-  /**
-   * Creates an array of `KStream` from this stream by branching the records in the original stream based on
-   * the supplied predicates.
-   *
-   * @param predicates the ordered list of functions that return a Boolean
-   * @return multiple distinct substreams of this [[KStream]]
-   * @see `org.apache.kafka.streams.kstream.KStream#branch`
-   * @deprecated since 2.8. Use `split` instead.
-   */
-  // noinspection ScalaUnnecessaryParentheses
-  @deprecated("use `split()` instead", "2.8")
-  def branch(predicates: ((K, V) => Boolean)*): Array[KStream[K, V]] =
-    inner.branch(predicates.map(_.asPredicate): _*).map(kstream => new KStream(kstream))
-
   /**
    * Split this stream. [[BranchedKStream]] can be used for routing the records to different branches depending
    * on evaluation against the supplied predicates.
@@ -363,40 +337,6 @@ class KStream[K, V](val inner: KStreamJ[K, V]) {
   def split(named: Named): BranchedKStream[K, V] =
     new BranchedKStream(inner.split(named))
 
-  /**
-   * Materialize this stream to a topic and creates a new [[KStream]] from the topic using the `Produced` instance for
-   * configuration of the `Serde key serde`, `Serde value serde`, and `StreamPartitioner`
-   * <p>
-   * The user can either supply the `Produced` instance as an implicit in scope or they can also provide implicit
-   * key and value serdes that will be converted to a `Produced` instance implicitly.
-   * <p>
-   * {{{
-   * Example:
-   *
-   * // brings implicit serdes in scope
-   * import Serdes._
-   *
-   * //..
-   * val clicksPerRegion: KStream[String, Long] = //..
-   *
-   * // Implicit serdes in scope will generate an implicit Produced instance, which
-   * // will be passed automatically to the call of through below
-   * clicksPerRegion.through(topic)
-   *
-   * // Similarly you can create an implicit Produced and it will be passed implicitly
-   * // to the through call
-   * }}}
-   *
-   * @param topic    the topic name
-   * @param produced the instance of Produced that gives the serdes and `StreamPartitioner`
-   * @return a [[KStream]] that contains the exact same (and potentially repartitioned) records as this [[KStream]]
-   * @see `org.apache.kafka.streams.kstream.KStream#through`
-   * @deprecated use `repartition()` instead
-   */
-  @deprecated("use `repartition()` instead", "2.6.0")
-  def through(topic: String)(implicit produced: Produced[K, V]): KStream[K, V] =
-    new KStream(inner.through(topic, produced))
-
   /**
    * Materialize this stream to a topic and creates a new [[KStream]] from the topic using the `Repartitioned` instance
    * for configuration of the `Serde key serde`, `Serde value serde`, `StreamPartitioner`, number of partitions, and
@@ -543,322 +483,6 @@ class KStream[K, V](val inner: KStreamJ[K, V]) {
   def toTable(named: Named, materialized: Materialized[K, V, ByteArrayKeyValueStore]): KTable[K, V] =
     new KTable(inner.toTable(named, materialized))
 
-  /**
-   * Transform each record of the input stream into zero or more records in the output stream (both key and value type
-   * can be altered arbitrarily).
-   * A `Transformer` (provided by the given `TransformerSupplier`) is applied to each input record
-   * and computes zero or more output records.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `Transformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param transformerSupplier the `TransformerSuplier` that generates `Transformer`
-   * @param stateStoreNames     the names of the state stores used by the processor
-   * @return a [[KStream]] that contains more or less records with new key and value (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transform`
-   */
-  @deprecated(since = "3.3", message = "Use process(ProcessorSupplier, String*) instead.")
-  def transform[K1, V1](
-    transformerSupplier: TransformerSupplier[K, V, KeyValue[K1, V1]],
-    stateStoreNames: String*
-  ): KStream[K1, V1] =
-    new KStream(inner.transform(transformerSupplier, stateStoreNames: _*))
-
-  /**
-   * Transform each record of the input stream into zero or more records in the output stream (both key and value type
-   * can be altered arbitrarily).
-   * A `Transformer` (provided by the given `TransformerSupplier`) is applied to each input record
-   * and computes zero or more output records.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `Transformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param transformerSupplier the `TransformerSuplier` that generates `Transformer`
-   * @param named               a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames     the names of the state stores used by the processor
-   * @return a [[KStream]] that contains more or less records with new key and value (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transform`
-   */
-  @deprecated(since = "3.3", message = "Use process(ProcessorSupplier, Named, String*) instead.")
-  def transform[K1, V1](
-    transformerSupplier: TransformerSupplier[K, V, KeyValue[K1, V1]],
-    named: Named,
-    stateStoreNames: String*
-  ): KStream[K1, V1] =
-    new KStream(inner.transform(transformerSupplier, named, stateStoreNames: _*))
-
-  /**
-   * Transform each record of the input stream into zero or more records in the output stream (both key and value type
-   * can be altered arbitrarily).
-   * A `Transformer` (provided by the given `TransformerSupplier`) is applied to each input record
-   * and computes zero or more output records.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `Transformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param transformerSupplier the `TransformerSuplier` that generates `Transformer`
-   * @param stateStoreNames     the names of the state stores used by the processor
-   * @return a [[KStream]] that contains more or less records with new key and value (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transform`
-   */
-  @deprecated(since = "3.3", message = "Use process(ProcessorSupplier, String*) instead.")
-  def flatTransform[K1, V1](
-    transformerSupplier: TransformerSupplier[K, V, Iterable[KeyValue[K1, V1]]],
-    stateStoreNames: String*
-  ): KStream[K1, V1] =
-    new KStream(inner.flatTransform(transformerSupplier.asJava, stateStoreNames: _*))
-
-  /**
-   * Transform each record of the input stream into zero or more records in the output stream (both key and value type
-   * can be altered arbitrarily).
-   * A `Transformer` (provided by the given `TransformerSupplier`) is applied to each input record
-   * and computes zero or more output records.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `Transformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param transformerSupplier the `TransformerSuplier` that generates `Transformer`
-   * @param named               a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames     the names of the state stores used by the processor
-   * @return a [[KStream]] that contains more or less records with new key and value (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transform`
-   */
-  @deprecated(since = "3.3", message = "Use process(ProcessorSupplier, Named, String*) instead.")
-  def flatTransform[K1, V1](
-    transformerSupplier: TransformerSupplier[K, V, Iterable[KeyValue[K1, V1]]],
-    named: Named,
-    stateStoreNames: String*
-  ): KStream[K1, V1] =
-    new KStream(inner.flatTransform(transformerSupplier.asJava, named, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into zero or more records (with possible new type) in the
-   * output stream.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerSupplier` that generates a `ValueTransformer`
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, Named, String*) instead.")
-  def flatTransformValues[VR](
-    valueTransformerSupplier: ValueTransformerSupplier[V, Iterable[VR]],
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.flatTransformValues[VR](valueTransformerSupplier.asJava, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into zero or more records (with possible new type) in the
-   * output stream.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerSupplier` that generates a `ValueTransformer`
-   * @param named                    a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, Named, String*) instead.")
-  def flatTransformValues[VR](
-    valueTransformerSupplier: ValueTransformerSupplier[V, Iterable[VR]],
-    named: Named,
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.flatTransformValues[VR](valueTransformerSupplier.asJava, named, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into zero or more records (with possible new type) in the
-   * output stream.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerWithKeySupplier` that generates a `ValueTransformerWithKey`
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, String*) instead.")
-  def flatTransformValues[VR](
-    valueTransformerSupplier: ValueTransformerWithKeySupplier[K, V, Iterable[VR]],
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.flatTransformValues[VR](valueTransformerSupplier.asJava, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into zero or more records (with possible new type) in the
-   * output stream.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerWithKeySupplier` that generates a `ValueTransformerWithKey`
-   * @param named                    a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, Named, String*) instead.")
-  def flatTransformValues[VR](
-    valueTransformerSupplier: ValueTransformerWithKeySupplier[K, V, Iterable[VR]],
-    named: Named,
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.flatTransformValues[VR](valueTransformerSupplier.asJava, named, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into a new value (with possible new type) of the output record.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerSupplier` that generates a `ValueTransformer`
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, String*) instead.")
-  def transformValues[VR](
-    valueTransformerSupplier: ValueTransformerSupplier[V, VR],
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.transformValues[VR](valueTransformerSupplier, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into a new value (with possible new type) of the output record.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerSupplier` that generates a `ValueTransformer`
-   * @param named                    a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, Named, String*) instead.")
-  def transformValues[VR](
-    valueTransformerSupplier: ValueTransformerSupplier[V, VR],
-    named: Named,
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.transformValues[VR](valueTransformerSupplier, named, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into a new value (with possible new type) of the output record.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerWithKeySupplier` that generates a `ValueTransformerWithKey`
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, String*) instead.")
-  def transformValues[VR](
-    valueTransformerSupplier: ValueTransformerWithKeySupplier[K, V, VR],
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.transformValues[VR](valueTransformerSupplier, stateStoreNames: _*))
-
-  /**
-   * Transform the value of each input record into a new value (with possible new type) of the output record.
-   * A `ValueTransformer` (provided by the given `ValueTransformerSupplier`) is applied to each input
-   * record value and computes a new value for it.
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `ValueTransformer`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param valueTransformerSupplier a instance of `ValueTransformerWithKeySupplier` that generates a `ValueTransformerWithKey`
-   * @param named                    a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames          the names of the state stores used by the processor
-   * @return a [[KStream]] that contains records with unmodified key and new values (possibly of different type)
-   * @see `org.apache.kafka.streams.kstream.KStream#transformValues`
-   */
-  @deprecated(since = "3.3", message = "Use processValues(FixedKeyProcessorSupplier, Named, String*) instead.")
-  def transformValues[VR](
-    valueTransformerSupplier: ValueTransformerWithKeySupplier[K, V, VR],
-    named: Named,
-    stateStoreNames: String*
-  ): KStream[K, VR] =
-    new KStream(inner.transformValues[VR](valueTransformerSupplier, named, stateStoreNames: _*))
-
-  /**
-   * Process all records in this stream, one record at a time, by applying a `Processor` (provided by the given
-   * `processorSupplier`).
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `Processor`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param processorSupplier a function that generates a `org.apache.kafka.streams.processor.Processor`
-   * @param stateStoreNames   the names of the state store used by the processor
-   * @see `org.apache.kafka.streams.kstream.KStream#process`
-   */
-  @deprecated(since = "3.0", message = "Use process(ProcessorSupplier, String*) instead.")
-  def process(
-    processorSupplier: () => org.apache.kafka.streams.processor.Processor[K, V],
-    stateStoreNames: String*
-  ): Unit = {
-    val processorSupplierJ: org.apache.kafka.streams.processor.ProcessorSupplier[K, V] = () => processorSupplier()
-    inner.process(processorSupplierJ, stateStoreNames: _*)
-  }
-
-  /**
-   * Process all records in this stream, one record at a time, by applying a `Processor` (provided by the given
-   * `processorSupplier`).
-   * In order to assign a state, the state must be created and added via `addStateStore` before they can be connected
-   * to the `Processor`.
-   * It's not required to connect global state stores that are added via `addGlobalStore`;
-   * read-only access to global state stores is available by default.
-   *
-   * @param processorSupplier a function that generates a `org.apache.kafka.streams.processor.Processor`
-   * @param named             a [[Named]] config used to name the processor in the topology
-   * @param stateStoreNames   the names of the state store used by the processor
-   * @see `org.apache.kafka.streams.kstream.KStream#process`
-   */
-  @deprecated(since = "3.0", message = "Use process(ProcessorSupplier, String*) instead.")
-  def process(
-    processorSupplier: () => org.apache.kafka.streams.processor.Processor[K, V],
-    named: Named,
-    stateStoreNames: String*
-  ): Unit = {
-    val processorSupplierJ: org.apache.kafka.streams.processor.ProcessorSupplier[K, V] = () => processorSupplier()
-    inner.process(processorSupplierJ, named, stateStoreNames: _*)
-  }
-
   /**
    * Process all records in this stream, one record at a time, by applying a `Processor` (provided by the given
    * `processorSupplier`).
