@@ -47,10 +47,10 @@ class StreamsSmokeTest(KafkaTest):
         self.driver = StreamsSmokeTestDriverService(test_context, self.kafka)
 
     @cluster(num_nodes=8)
-    @matrix(processing_guarantee=['exactly_once', 'exactly_once_v2', 'at_least_once'],
+    @matrix(processing_guarantee=['exactly_once_v2', 'at_least_once'],
             crash=[True, False],
-            metadata_quorum=quorum.all_non_upgrade)
-    def test_streams(self, processing_guarantee, crash, metadata_quorum=quorum.zk):
+            metadata_quorum=[quorum.combined_kraft])
+    def test_streams(self, processing_guarantee, crash, metadata_quorum):
         processor1 = StreamsSmokeTestJobRunnerService(self.test_context, self.kafka, processing_guarantee)
         processor2 = StreamsSmokeTestJobRunnerService(self.test_context, self.kafka, processing_guarantee)
         processor3 = StreamsSmokeTestJobRunnerService(self.test_context, self.kafka, processing_guarantee)
