@@ -62,21 +62,6 @@ class KafkaVersion(LooseVersion):
 
         return LooseVersion._cmp(self, other)
 
-    def consumer_supports_bootstrap_server(self):
-        """
-        Kafka supported a new consumer beginning with v0.9.0 where
-        we can specify --bootstrap-server instead of --zookeeper.
-
-        This version also allowed a --consumer-config file where we could specify
-        a security protocol other than PLAINTEXT.
-
-        :return: true if the version of Kafka supports a new consumer with --bootstrap-server
-        """
-        return self >= V_0_9_0_0
-
-    def supports_named_listeners(self):
-        return self >= V_0_10_2_0
-
     def acl_command_supports_bootstrap_server(self):
         return self >= V_2_1_0
 
@@ -112,6 +97,9 @@ class KafkaVersion(LooseVersion):
         # -> https://issues.apache.org/jira/browse/KAFKA-14646
         return hasattr(self, "version") and self >= V_3_4_0
 
+    def supports_feature_command(self):
+        return self >= V_3_8_0
+
 def get_version(node=None):
     """Return the version attached to the given node.
     Default to DEV_BRANCH if node or node.version is undefined (aka None)
@@ -122,60 +110,25 @@ def get_version(node=None):
         return DEV_BRANCH
 
 DEV_BRANCH = KafkaVersion("dev")
-DEV_VERSION = KafkaVersion("3.9.1-SNAPSHOT")
+DEV_VERSION = KafkaVersion("4.0.0-SNAPSHOT")
 
+LATEST_STABLE_TRANSACTION_VERSION = 2
 # This should match the LATEST_PRODUCTION version defined in MetadataVersion.java
-LATEST_STABLE_METADATA_VERSION = "3.9-IV0"
-
-# 0.8.2.x versions
-V_0_8_2_1 = KafkaVersion("0.8.2.1")
-V_0_8_2_2 = KafkaVersion("0.8.2.2")
-LATEST_0_8_2 = V_0_8_2_2
-
-# 0.9.0.x versions
-V_0_9_0_0 = KafkaVersion("0.9.0.0")
-V_0_9_0_1 = KafkaVersion("0.9.0.1")
-LATEST_0_9 = V_0_9_0_1
-
-# 0.10.0.x versions
-V_0_10_0_0 = KafkaVersion("0.10.0.0")
-V_0_10_0_1 = KafkaVersion("0.10.0.1")
-LATEST_0_10_0 = V_0_10_0_1
-
-# 0.10.1.x versions
-V_0_10_1_0 = KafkaVersion("0.10.1.0")
-V_0_10_1_1 = KafkaVersion("0.10.1.1")
-LATEST_0_10_1 = V_0_10_1_1
-
-# 0.10.2.x versions
-V_0_10_2_0 = KafkaVersion("0.10.2.0")
-V_0_10_2_1 = KafkaVersion("0.10.2.1")
-V_0_10_2_2 = KafkaVersion("0.10.2.2")
-LATEST_0_10_2 = V_0_10_2_2
-
-LATEST_0_10 = LATEST_0_10_2
+LATEST_STABLE_METADATA_VERSION = "4.0-IV3"
 
 # 0.11.0.x versions
-V_0_11_0_0 = KafkaVersion("0.11.0.0")
-V_0_11_0_1 = KafkaVersion("0.11.0.1")
-V_0_11_0_2 = KafkaVersion("0.11.0.2")
 V_0_11_0_3 = KafkaVersion("0.11.0.3")
-LATEST_0_11_0 = V_0_11_0_3
-LATEST_0_11 = LATEST_0_11_0
+LATEST_0_11 = V_0_11_0_3
 
 # 1.0.x versions
-V_1_0_0 = KafkaVersion("1.0.0")
-V_1_0_1 = KafkaVersion("1.0.1")
 V_1_0_2 = KafkaVersion("1.0.2")
 LATEST_1_0 = V_1_0_2
 
 # 1.1.x versions
-V_1_1_0 = KafkaVersion("1.1.0")
 V_1_1_1 = KafkaVersion("1.1.1")
 LATEST_1_1 = V_1_1_1
 
 # 2.0.x versions
-V_2_0_0 = KafkaVersion("2.0.0")
 V_2_0_1 = KafkaVersion("2.0.1")
 LATEST_2_0 = V_2_0_1
 
@@ -280,3 +233,7 @@ LATEST_3_8 = V_3_8_1
 # 3.9.x version
 V_3_9_0 = KafkaVersion("3.9.0")
 LATEST_3_9 = V_3_9_0
+
+# 4.0.x version
+V_4_0_0 = KafkaVersion("4.0.0")
+LATEST_4_0 = V_4_0_0
