@@ -24,11 +24,13 @@ import org.apache.kafka.clients.admin.MemberDescription;
 import org.apache.kafka.clients.admin.RemoveMembersFromConsumerGroupOptions;
 import org.apache.kafka.clients.consumer.Consumer;
 import org.apache.kafka.clients.consumer.ConsumerConfig;
+import org.apache.kafka.clients.consumer.GroupProtocol;
 import org.apache.kafka.clients.consumer.KafkaConsumer;
 import org.apache.kafka.clients.consumer.OffsetAndTimestamp;
 import org.apache.kafka.common.KafkaFuture;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.annotation.InterfaceStability;
+import org.apache.kafka.common.errors.GroupIdNotFoundException;
 import org.apache.kafka.common.requests.ListOffsetsResponse;
 import org.apache.kafka.common.serialization.ByteArrayDeserializer;
 import org.apache.kafka.common.utils.Exit;
@@ -59,6 +61,9 @@ import joptsimple.OptionException;
 import joptsimple.OptionSpec;
 import joptsimple.OptionSpecBuilder;
 
+import static org.apache.kafka.clients.consumer.ConsumerConfig.GROUP_PROTOCOL_CONFIG;
+
+
 /**
  * {@link StreamsResetter} resets the processing state of a Kafka Streams application so that, for example,
  * you can reprocess its input from scratch.
@@ -93,7 +98,6 @@ public class StreamsResetter {
     private static final String USAGE = "This tool helps to quickly reset an application in order to reprocess "
             + "its data from scratch.\n"
             + "* This tool resets offsets of input topics to the earliest available offset (by default), or to a specific defined position"
-            + " and it skips to the end of intermediate topics (topics that are input and output topics, e.g., used by deprecated through() method).\n"
             + "* This tool deletes the internal topics that were created by Kafka Streams (topics starting with "
             + "\"<application.id>-\").\n"
             + "The tool finds these internal topics automatically. If the topics flagged automatically for deletion by "
@@ -136,8 +140,6 @@ public class StreamsResetter {
             String bootstrapServerValue = "localhost:9092";
             if (options.hasBootstrapServer()) {
                 bootstrapServerValue = options.bootstrapServer();
-            } else if (options.hasBootstrapServers()) {
-                bootstrapServerValue = options.bootstrapServers();
             }
 
             properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServerValue);
@@ -153,6 +155,12 @@ public class StreamsResetter {
                 }
 
                 final HashMap<Object, Object> consumerConfig = new HashMap<>(config);
+                if (consumerConfig.containsKey(GROUP_PROTOCOL_CONFIG) &&
+                    !consumerConfig.get(GROUP_PROTOCOL_CONFIG).toString().equalsIgnoreCase(GroupProtocol.CLASSIC.name())
+                ) {
+                    System.out.println("WARNING: provided group protocol will be ignored. Using supported " + GroupProtocol.CLASSIC.name() + " protocol instead");
+                }
+                consumerConfig.put(GROUP_PROTOCOL_CONFIG, GroupProtocol.CLASSIC.name());
                 consumerConfig.putAll(properties);
                 int exitCode = maybeResetInputAndSeekToEndIntermediateTopicOffsets(consumerConfig, options);
                 exitCode |= maybeDeleteInternalTopics(adminClient, options);
@@ -172,17 +180,24 @@ public class StreamsResetter {
         final DescribeConsumerGroupsResult describeResult = adminClient.describeConsumerGroups(
             Collections.singleton(groupId),
             new DescribeConsumerGroupsOptions().timeoutMs(10 * 1000));
-        final List<MemberDescription> members =
-            new ArrayList<>(describeResult.describedGroups().get(groupId).get().members());
-        if (!members.isEmpty()) {
-            if (options.hasForce()) {
-                System.out.println("Force deleting all active members in the group: " + groupId);
-                adminClient.removeMembersFromConsumerGroup(groupId, new RemoveMembersFromConsumerGroupOptions()).all().get();
-            } else {
-                throw new IllegalStateException("Consumer group '" + groupId + "' is still active "
+        try {
+            final List<MemberDescription> members =
+                new ArrayList<>(describeResult.describedGroups().get(groupId).get().members());
+            if (!members.isEmpty()) {
+                if (options.hasForce()) {
+                    System.out.println("Force deleting all active members in the group: " + groupId);
+                    adminClient.removeMembersFromConsumerGroup(groupId, new RemoveMembersFromConsumerGroupOptions()).all().get();
+                } else {
+                    throw new IllegalStateException("Consumer group '" + groupId + "' is still active "
                         + "and has following members: " + members + ". "
                         + "Make sure to stop all running application instances before running the reset tool."
                         + " You can use option '--force' to remove active members from the group.");
+                }
+            }
+        } catch (ExecutionException ee) {
+            // If the group ID is not found, this is not an error case
+            if (!(ee.getCause() instanceof GroupIdNotFoundException)) {
+                throw ee;
             }
         }
     }
@@ -546,7 +561,6 @@ public class StreamsResetter {
     }
 
     private static class StreamsResetterOptions extends CommandDefaultOptions {
-        private final OptionSpec<String> bootstrapServersOption;
         private final OptionSpec<String> bootstrapServerOption;
         private final OptionSpec<String> applicationIdOption;
         private final OptionSpec<String> inputTopicsOption;
@@ -570,11 +584,7 @@ public class StreamsResetter {
                 .ofType(String.class)
                 .describedAs("id")
                 .required();
-            bootstrapServersOption = parser.accepts("bootstrap-servers", "DEPRECATED: Comma-separated list of broker urls with format: HOST1:PORT1,HOST2:PORT2")
-                .withRequiredArg()
-                .ofType(String.class)
-                .describedAs("urls");
-            bootstrapServerOption = parser.accepts("bootstrap-server", "REQUIRED unless --bootstrap-servers(deprecated) is specified. The server(s) to connect to. The broker list string in the form HOST1:PORT1,HOST2:PORT2. (default: localhost:9092)")
+            bootstrapServerOption = parser.accepts("bootstrap-server", "The server(s) to connect to. The broker list string in the form HOST1:PORT1,HOST2:PORT2. (default: localhost:9092)")
                 .withRequiredArg()
                 .ofType(String.class)
                 .describedAs("server to connect to");
@@ -584,8 +594,8 @@ public class StreamsResetter {
                 .ofType(String.class)
                 .withValuesSeparatedBy(',')
                 .describedAs("list");
-            intermediateTopicsOption = parser.accepts("intermediate-topics", "Comma-separated list of intermediate user topics (topics that are input and output topics, "
-                    + "e.g., used in the deprecated through() method). For these topics, the tool will skip to the end.")
+            intermediateTopicsOption = parser.accepts("intermediate-topics", "[deprecated] Comma-separated list of intermediate user topics (topics that are input and output topics). "
+                    + "For these topics, the tool will skip to the end.")
                 .withRequiredArg()
                 .ofType(String.class)
                 .withValuesSeparatedBy(',')
@@ -668,14 +678,6 @@ public class StreamsResetter {
             return options.valueOf(bootstrapServerOption);
         }
 
-        public boolean hasBootstrapServers() {
-            return options.has(bootstrapServersOption);
-        }
-
-        public String bootstrapServers() {
-            return options.valueOf(bootstrapServersOption);
-        }
-
         public boolean hasForce() {
             return options.has(forceOption);
         }
@@ -685,6 +687,9 @@ public class StreamsResetter {
         }
 
         public List<String> intermediateTopicsOption() {
+            if (options.has(intermediateTopicsOption)) {
+                System.out.println("WARN: `--intermediate-topics` is deprecated and will be removed in a future release");
+            }
             return options.valuesOf(intermediateTopicsOption);
         }
 
