@@ -16,16 +16,9 @@
  */
 package org.apache.kafka.tools.reassign;
 
-import kafka.test.ClusterInstance;
-import kafka.test.annotation.ClusterConfigProperty;
-import kafka.test.annotation.ClusterTest;
-import kafka.test.annotation.ClusterTestDefaults;
-import kafka.test.annotation.ClusterTests;
-import kafka.test.annotation.Type;
-import kafka.test.junit.ClusterTestExtensions;
-
 import org.apache.kafka.clients.CommonClientConfigs;
 import org.apache.kafka.clients.admin.Admin;
+import org.apache.kafka.clients.admin.AdminClientConfig;
 import org.apache.kafka.clients.admin.AlterConfigOp;
 import org.apache.kafka.clients.admin.Config;
 import org.apache.kafka.clients.admin.ConfigEntry;
@@ -47,15 +40,19 @@ import org.apache.kafka.common.TopicPartitionReplica;
 import org.apache.kafka.common.config.ConfigResource;
 import org.apache.kafka.common.serialization.ByteArrayDeserializer;
 import org.apache.kafka.common.serialization.ByteArraySerializer;
+import org.apache.kafka.common.test.ClusterInstance;
+import org.apache.kafka.common.test.api.ClusterConfigProperty;
+import org.apache.kafka.common.test.api.ClusterTest;
+import org.apache.kafka.common.test.api.ClusterTestDefaults;
+import org.apache.kafka.common.test.api.Type;
 import org.apache.kafka.common.utils.Time;
-import org.apache.kafka.server.config.QuotaConfigs;
+import org.apache.kafka.server.config.QuotaConfig;
 import org.apache.kafka.test.TestUtils;
 import org.apache.kafka.tools.TerseException;
 
 import com.fasterxml.jackson.core.JsonProcessingException;
 
 import org.junit.jupiter.api.Assertions;
-import org.junit.jupiter.api.extension.ExtendWith;
 
 import java.time.Duration;
 import java.util.AbstractMap;
@@ -81,12 +78,9 @@ import static java.util.Collections.singleton;
 import static java.util.Collections.singletonList;
 import static java.util.Collections.singletonMap;
 import static org.apache.kafka.common.config.ConfigResource.Type.TOPIC;
-import static org.apache.kafka.server.common.MetadataVersion.IBP_2_7_IV1;
-import static org.apache.kafka.server.common.MetadataVersion.IBP_3_3_IV0;
-import static org.apache.kafka.server.config.QuotaConfigs.FOLLOWER_REPLICATION_THROTTLED_REPLICAS_CONFIG;
-import static org.apache.kafka.server.config.QuotaConfigs.LEADER_REPLICATION_THROTTLED_REPLICAS_CONFIG;
+import static org.apache.kafka.server.config.QuotaConfig.FOLLOWER_REPLICATION_THROTTLED_REPLICAS_CONFIG;
+import static org.apache.kafka.server.config.QuotaConfig.LEADER_REPLICATION_THROTTLED_REPLICAS_CONFIG;
 import static org.apache.kafka.server.config.ReplicationConfigs.AUTO_LEADER_REBALANCE_ENABLE_CONFIG;
-import static org.apache.kafka.server.config.ReplicationConfigs.INTER_BROKER_PROTOCOL_VERSION_CONFIG;
 import static org.apache.kafka.server.config.ReplicationConfigs.REPLICA_FETCH_BACKOFF_MS_CONFIG;
 import static org.apache.kafka.server.config.ReplicationConfigs.REPLICA_LAG_TIME_MAX_MS_CONFIG;
 import static org.apache.kafka.tools.ToolsTestUtils.assignThrottledPartitionReplicas;
@@ -94,6 +88,7 @@ import static org.apache.kafka.tools.ToolsTestUtils.throttleAllBrokersReplicatio
 import static org.apache.kafka.tools.reassign.ReassignPartitionsCommand.BROKER_LEVEL_THROTTLES;
 import static org.apache.kafka.tools.reassign.ReassignPartitionsCommand.cancelAssignment;
 import static org.apache.kafka.tools.reassign.ReassignPartitionsCommand.executeAssignment;
+import static org.apache.kafka.tools.reassign.ReassignPartitionsCommand.generateAssignment;
 import static org.apache.kafka.tools.reassign.ReassignPartitionsCommand.verifyAssignment;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -101,18 +96,17 @@ import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 @ClusterTestDefaults(brokers = 5, disksPerBroker = 3, serverProperties = {
-        // shorter backoff to reduce test durations when no active partitions are eligible for fetching due to throttling
-        @ClusterConfigProperty(key = REPLICA_FETCH_BACKOFF_MS_CONFIG, value = "100"),
-        // Don't move partition leaders automatically.
-        @ClusterConfigProperty(key = AUTO_LEADER_REBALANCE_ENABLE_CONFIG, value = "false"),
-        @ClusterConfigProperty(key = REPLICA_LAG_TIME_MAX_MS_CONFIG, value = "1000"),
-        @ClusterConfigProperty(id = 0, key = "broker.rack", value = "rack0"),
-        @ClusterConfigProperty(id = 1, key = "broker.rack", value = "rack0"),
-        @ClusterConfigProperty(id = 2, key = "broker.rack", value = "rack1"),
-        @ClusterConfigProperty(id = 3, key = "broker.rack", value = "rack1"),
-        @ClusterConfigProperty(id = 4, key = "broker.rack", value = "rack1"),
+    // shorter backoff to reduce test durations when no active partitions are eligible for fetching due to throttling
+    @ClusterConfigProperty(key = REPLICA_FETCH_BACKOFF_MS_CONFIG, value = "100"),
+    // Don't move partition leaders automatically.
+    @ClusterConfigProperty(key = AUTO_LEADER_REBALANCE_ENABLE_CONFIG, value = "false"),
+    @ClusterConfigProperty(key = REPLICA_LAG_TIME_MAX_MS_CONFIG, value = "1000"),
+    @ClusterConfigProperty(id = 0, key = "broker.rack", value = "rack0"),
+    @ClusterConfigProperty(id = 1, key = "broker.rack", value = "rack0"),
+    @ClusterConfigProperty(id = 2, key = "broker.rack", value = "rack1"),
+    @ClusterConfigProperty(id = 3, key = "broker.rack", value = "rack1"),
+    @ClusterConfigProperty(id = 4, key = "broker.rack", value = "rack1"),
 })
-@ExtendWith(ClusterTestExtensions.class)
 public class ReassignPartitionsCommandTest {
     private final ClusterInstance clusterInstance;
     private final Map<Integer, Map<String, Long>> unthrottledBrokerConfigs = IntStream
@@ -132,47 +126,6 @@ public class ReassignPartitionsCommandTest {
         executeAndVerifyReassignment();
     }
 
-    @ClusterTests({
-            @ClusterTest(types = {Type.ZK}, metadataVersion = IBP_2_7_IV1),
-            @ClusterTest(types = {Type.KRAFT, Type.CO_KRAFT}, metadataVersion = IBP_3_3_IV0)
-    })
-    public void testReassignmentWithAlterPartitionDisabled() throws Exception {
-        // Test reassignment when the IBP is on an older version which does not use
-        // the `AlterPartition` API. In this case, the controller will register individual
-        // watches for each reassigning partition so that the reassignment can be
-        // completed as soon as the ISR is expanded.
-        createTopics();
-        executeAndVerifyReassignment();
-    }
-
-    @ClusterTests({
-            @ClusterTest(types = {Type.ZK}, serverProperties = {
-                    @ClusterConfigProperty(id = 1, key = INTER_BROKER_PROTOCOL_VERSION_CONFIG, value = "2.7-IV1"),
-                    @ClusterConfigProperty(id = 2, key = INTER_BROKER_PROTOCOL_VERSION_CONFIG, value = "2.7-IV1"),
-                    @ClusterConfigProperty(id = 3, key = INTER_BROKER_PROTOCOL_VERSION_CONFIG, value = "2.7-IV1"),
-            }),
-            @ClusterTest(types = {Type.KRAFT, Type.CO_KRAFT}, serverProperties = {
-                    @ClusterConfigProperty(id = 1, key = INTER_BROKER_PROTOCOL_VERSION_CONFIG, value = "3.3-IV0"),
-                    @ClusterConfigProperty(id = 2, key = INTER_BROKER_PROTOCOL_VERSION_CONFIG, value = "3.3-IV0"),
-                    @ClusterConfigProperty(id = 3, key = INTER_BROKER_PROTOCOL_VERSION_CONFIG, value = "3.3-IV0"),
-            })
-    })
-    public void testReassignmentCompletionDuringPartialUpgrade() throws Exception {
-        // Test reassignment during a partial upgrade when some brokers are relying on
-        // `AlterPartition` and some rely on the old notification logic through Zookeeper.
-        // In this test case, broker 0 starts up first on the latest IBP and is typically
-        // elected as controller. The three remaining brokers start up on the older IBP.
-        // We want to ensure that reassignment can still complete through the ISR change
-        // notification path even though the controller expects `AlterPartition`.
-
-        // Override change notification settings so that test is not delayed by ISR
-        // change notification delay
-        // ZkAlterPartitionManager.DefaultIsrPropagationConfig_$eq(new IsrChangePropagationConfig(500, 100, 500));
-
-        createTopics();
-        executeAndVerifyReassignment();
-    }
-
     @ClusterTest
     public void testHighWaterMarkAfterPartitionReassignment() throws Exception {
         createTopics();
@@ -198,6 +151,24 @@ public class ReassignPartitionsCommandTest {
         }
     }
 
+    @ClusterTest
+    public void testGenerateAssignmentWithBootstrapServer() throws Exception {
+        createTopics();
+        TopicPartition foo0 = new TopicPartition("foo", 0);
+        produceMessages(foo0.topic(), foo0.partition(), 100);
+
+        try (Admin admin = Admin.create(Collections.singletonMap(CommonClientConfigs.BOOTSTRAP_SERVERS_CONFIG, clusterInstance.bootstrapServers()))) {
+            String assignment = "{\"version\":1,\"partitions\":" +
+                    "[{\"topic\":\"foo\",\"partition\":0,\"replicas\":[3,1,2],\"log_dirs\":[\"any\",\"any\",\"any\"]}" +
+                    "]}";
+            generateAssignment(admin, assignment, "1,2,3", false);
+            Map<TopicPartition, PartitionReassignmentState> finalAssignment = singletonMap(foo0,
+                    new PartitionReassignmentState(asList(0, 1, 2), asList(3, 1, 2), true));
+            waitForVerifyAssignment(admin, assignment, false,
+                    new VerifyAssignmentResult(finalAssignment));
+        }
+    }
+
     @ClusterTest
     public void testAlterReassignmentThrottle() throws Exception {
         createTopics();
@@ -334,47 +305,13 @@ public class ReassignPartitionsCommandTest {
      * Test running a reassignment and then cancelling it.
      */
     @ClusterTest
-    public void testCancellation() throws Exception {
-        createTopics();
-        TopicPartition foo0 = new TopicPartition("foo", 0);
-        TopicPartition baz1 = new TopicPartition("baz", 1);
+    public void testCancellationWithBootstrapServer() throws Exception {
+        testCancellationAction(true);
+    }
 
-        produceMessages(foo0.topic(), foo0.partition(), 200);
-        produceMessages(baz1.topic(), baz1.partition(), 200);
-        String assignment = "{\"version\":1,\"partitions\":" +
-                "[{\"topic\":\"foo\",\"partition\":0,\"replicas\":[0,1,3],\"log_dirs\":[\"any\",\"any\",\"any\"]}," +
-                "{\"topic\":\"baz\",\"partition\":1,\"replicas\":[0,2,3],\"log_dirs\":[\"any\",\"any\",\"any\"]}" +
-                "]}";
-        try (Admin admin = Admin.create(Collections.singletonMap(CommonClientConfigs.BOOTSTRAP_SERVERS_CONFIG, clusterInstance.bootstrapServers()))) {
-            assertEquals(unthrottledBrokerConfigs,
-                    describeBrokerLevelThrottles(admin, unthrottledBrokerConfigs.keySet()));
-            long interBrokerThrottle = 1L;
-            runExecuteAssignment(false, assignment, interBrokerThrottle, -1L);
-            waitForInterBrokerThrottle(admin, asList(0, 1, 2, 3), interBrokerThrottle);
-
-            Map<TopicPartition, PartitionReassignmentState> partStates = new HashMap<>();
-
-            partStates.put(foo0, new PartitionReassignmentState(asList(0, 1, 3, 2), asList(0, 1, 3), false));
-            partStates.put(baz1, new PartitionReassignmentState(asList(0, 2, 3, 1), asList(0, 2, 3), false));
-
-            // Verify that the reassignment is running.  The very low throttle should keep it
-            // from completing before this runs.
-            waitForVerifyAssignment(admin, assignment, true,
-                    new VerifyAssignmentResult(partStates, true, Collections.emptyMap(), false));
-            // Cancel the reassignment.
-            assertEquals(new AbstractMap.SimpleImmutableEntry<>(new HashSet<>(asList(foo0, baz1)), Collections.emptySet()), runCancelAssignment(assignment, true));
-            // Broker throttles are still active because we passed --preserve-throttles
-            waitForInterBrokerThrottle(admin, asList(0, 1, 2, 3), interBrokerThrottle);
-            // Cancelling the reassignment again should reveal nothing to cancel.
-            assertEquals(new AbstractMap.SimpleImmutableEntry<>(Collections.emptySet(), Collections.emptySet()), runCancelAssignment(assignment, false));
-            // This time, the broker throttles were removed.
-            waitForBrokerLevelThrottles(admin, unthrottledBrokerConfigs);
-            // Verify that there are no ongoing reassignments.
-            assertFalse(runVerifyAssignment(admin, assignment, false).partsOngoing);
-        }
-        // Verify that the partition is removed from cancelled replicas
-        verifyReplicaDeleted(new TopicPartitionReplica(foo0.topic(), foo0.partition(), 3));
-        verifyReplicaDeleted(new TopicPartitionReplica(baz1.topic(), baz1.partition(), 3));
+    @ClusterTest(types = {Type.KRAFT, Type.CO_KRAFT})
+    public void testCancellationWithBootstrapController() throws Exception {
+        testCancellationAction(false);
     }
 
     @ClusterTest
@@ -408,7 +345,7 @@ public class ReassignPartitionsCommandTest {
         }
 
         // Now cancel the assignment and verify that the partition is removed from cancelled replicas
-        assertEquals(new AbstractMap.SimpleImmutableEntry<>(singleton(foo0), Collections.emptySet()), runCancelAssignment(assignment, true));
+        assertEquals(new AbstractMap.SimpleImmutableEntry<>(singleton(foo0), Collections.emptySet()), runCancelAssignment(assignment, true, true));
         verifyReplicaDeleted(new TopicPartitionReplica(foo0.topic(), foo0.partition(), 3));
         verifyReplicaDeleted(new TopicPartitionReplica(foo0.topic(), foo0.partition(), 4));
     }
@@ -416,7 +353,7 @@ public class ReassignPartitionsCommandTest {
     /**
      * Test moving partitions between directories.
      */
-    @ClusterTest(types = {Type.ZK, Type.KRAFT})
+    @ClusterTest(types = {Type.KRAFT})
     public void testLogDirReassignment() throws Exception {
         createTopics();
         TopicPartition topicPartition = new TopicPartition("foo", 0);
@@ -446,7 +383,7 @@ public class ReassignPartitionsCommandTest {
             admin.incrementalAlterConfigs(singletonMap(
                             new ConfigResource(ConfigResource.Type.BROKER, "0"),
                             singletonList(new AlterConfigOp(
-                                    new ConfigEntry(QuotaConfigs.REPLICA_ALTER_LOG_DIRS_IO_MAX_BYTES_PER_SECOND_CONFIG, ""), AlterConfigOp.OpType.DELETE))))
+                                    new ConfigEntry(QuotaConfig.REPLICA_ALTER_LOG_DIRS_IO_MAX_BYTES_PER_SECOND_CONFIG, ""), AlterConfigOp.OpType.DELETE))))
                     .all().get();
             waitForBrokerLevelThrottles(admin, unthrottledBrokerConfigs);
 
@@ -464,7 +401,7 @@ public class ReassignPartitionsCommandTest {
         }
     }
 
-    @ClusterTest(types = {Type.ZK, Type.KRAFT})
+    @ClusterTest(types = {Type.KRAFT})
     public void testAlterLogDirReassignmentThrottle() throws Exception {
         createTopics();
         TopicPartition topicPartition = new TopicPartition("foo", 0);
@@ -588,17 +525,17 @@ public class ReassignPartitionsCommandTest {
 
     private void waitForLogDirThrottle(Admin admin, Set<Integer> throttledBrokers, Long logDirThrottle) {
         Map<String, Long> throttledConfigMap = new HashMap<>();
-        throttledConfigMap.put(QuotaConfigs.LEADER_REPLICATION_THROTTLED_RATE_CONFIG, -1L);
-        throttledConfigMap.put(QuotaConfigs.FOLLOWER_REPLICATION_THROTTLED_RATE_CONFIG, -1L);
-        throttledConfigMap.put(QuotaConfigs.REPLICA_ALTER_LOG_DIRS_IO_MAX_BYTES_PER_SECOND_CONFIG, logDirThrottle);
+        throttledConfigMap.put(QuotaConfig.LEADER_REPLICATION_THROTTLED_RATE_CONFIG, -1L);
+        throttledConfigMap.put(QuotaConfig.FOLLOWER_REPLICATION_THROTTLED_RATE_CONFIG, -1L);
+        throttledConfigMap.put(QuotaConfig.REPLICA_ALTER_LOG_DIRS_IO_MAX_BYTES_PER_SECOND_CONFIG, logDirThrottle);
         waitForBrokerThrottles(admin, throttledBrokers, throttledConfigMap);
     }
 
     private void waitForInterBrokerThrottle(Admin admin, List<Integer> throttledBrokers, Long interBrokerThrottle) {
         Map<String, Long> throttledConfigMap = new HashMap<>();
-        throttledConfigMap.put(QuotaConfigs.LEADER_REPLICATION_THROTTLED_RATE_CONFIG, interBrokerThrottle);
-        throttledConfigMap.put(QuotaConfigs.FOLLOWER_REPLICATION_THROTTLED_RATE_CONFIG, interBrokerThrottle);
-        throttledConfigMap.put(QuotaConfigs.REPLICA_ALTER_LOG_DIRS_IO_MAX_BYTES_PER_SECOND_CONFIG, -1L);
+        throttledConfigMap.put(QuotaConfig.LEADER_REPLICATION_THROTTLED_RATE_CONFIG, interBrokerThrottle);
+        throttledConfigMap.put(QuotaConfig.FOLLOWER_REPLICATION_THROTTLED_RATE_CONFIG, interBrokerThrottle);
+        throttledConfigMap.put(QuotaConfig.REPLICA_ALTER_LOG_DIRS_IO_MAX_BYTES_PER_SECOND_CONFIG, -1L);
         waitForBrokerThrottles(admin, throttledBrokers, throttledConfigMap);
     }
 
@@ -725,9 +662,16 @@ public class ReassignPartitionsCommandTest {
 
     private Map.Entry<Set<TopicPartition>, Set<TopicPartitionReplica>> runCancelAssignment(
             String jsonString,
-            Boolean preserveThrottles
+            Boolean preserveThrottles,
+            Boolean useBootstrapServer
     ) {
-        try (Admin admin = Admin.create(Collections.singletonMap(CommonClientConfigs.BOOTSTRAP_SERVERS_CONFIG, clusterInstance.bootstrapServers()))) {
+        Map<String, Object> config;
+        if (useBootstrapServer) {
+            config = Collections.singletonMap(CommonClientConfigs.BOOTSTRAP_SERVERS_CONFIG, clusterInstance.bootstrapServers());
+        } else {
+            config = Collections.singletonMap(AdminClientConfig.BOOTSTRAP_CONTROLLERS_CONFIG, clusterInstance.bootstrapControllers());
+        }
+        try (Admin admin = Admin.create(config)) {
             return cancelAssignment(admin, jsonString, preserveThrottles, 10000L, Time.SYSTEM);
         } catch (ExecutionException | InterruptedException | JsonProcessingException | TerseException e) {
             throw new RuntimeException(e);
@@ -759,6 +703,49 @@ public class ReassignPartitionsCommandTest {
         }
     }
 
+    private void testCancellationAction(boolean useBootstrapServer) throws InterruptedException {
+        createTopics();
+        TopicPartition foo0 = new TopicPartition("foo", 0);
+        TopicPartition baz1 = new TopicPartition("baz", 1);
+
+        produceMessages(foo0.topic(), foo0.partition(), 200);
+        produceMessages(baz1.topic(), baz1.partition(), 200);
+        String assignment = "{\"version\":1,\"partitions\":" +
+                "[{\"topic\":\"foo\",\"partition\":0,\"replicas\":[0,1,3],\"log_dirs\":[\"any\",\"any\",\"any\"]}," +
+                "{\"topic\":\"baz\",\"partition\":1,\"replicas\":[0,2,3],\"log_dirs\":[\"any\",\"any\",\"any\"]}" +
+                "]}";
+        try (Admin admin = Admin.create(Collections.singletonMap(CommonClientConfigs.BOOTSTRAP_SERVERS_CONFIG, clusterInstance.bootstrapServers()))) {
+            assertEquals(unthrottledBrokerConfigs,
+                    describeBrokerLevelThrottles(admin, unthrottledBrokerConfigs.keySet()));
+            long interBrokerThrottle = 1L;
+            runExecuteAssignment(false, assignment, interBrokerThrottle, -1L);
+            waitForInterBrokerThrottle(admin, asList(0, 1, 2, 3), interBrokerThrottle);
+
+            Map<TopicPartition, PartitionReassignmentState> partStates = new HashMap<>();
+
+            partStates.put(foo0, new PartitionReassignmentState(asList(0, 1, 3, 2), asList(0, 1, 3), false));
+            partStates.put(baz1, new PartitionReassignmentState(asList(0, 2, 3, 1), asList(0, 2, 3), false));
+
+            // Verify that the reassignment is running.  The very low throttle should keep it
+            // from completing before this runs.
+            waitForVerifyAssignment(admin, assignment, true,
+                    new VerifyAssignmentResult(partStates, true, Collections.emptyMap(), false));
+            // Cancel the reassignment.
+            assertEquals(new AbstractMap.SimpleImmutableEntry<>(new HashSet<>(asList(foo0, baz1)), Collections.emptySet()), runCancelAssignment(assignment, true, useBootstrapServer));
+            // Broker throttles are still active because we passed --preserve-throttles
+            waitForInterBrokerThrottle(admin, asList(0, 1, 2, 3), interBrokerThrottle);
+            // Cancelling the reassignment again should reveal nothing to cancel.
+            assertEquals(new AbstractMap.SimpleImmutableEntry<>(Collections.emptySet(), Collections.emptySet()), runCancelAssignment(assignment, false, useBootstrapServer));
+            // This time, the broker throttles were removed.
+            waitForBrokerLevelThrottles(admin, unthrottledBrokerConfigs);
+            // Verify that there are no ongoing reassignments.
+            assertFalse(runVerifyAssignment(admin, assignment, false).partsOngoing);
+        }
+        // Verify that the partition is removed from cancelled replicas
+        verifyReplicaDeleted(new TopicPartitionReplica(foo0.topic(), foo0.partition(), 3));
+        verifyReplicaDeleted(new TopicPartitionReplica(baz1.topic(), baz1.partition(), 3));
+    }
+
     /**
      * Remove a set of throttled partitions and reset the overall replication quota.
      */
